/*
 *	Help library.  Uses menu.t to display help in a series of menus.
 *	Contains a list of standard tads2 verbs that can be expanded as 
 *	necessary.  The actual help text has been adapted from instruct.t, by
 *	Michael J. Roberts.
 */

// Uses quicksort to alphabetize the list of verbs
#include "quicksort.t"

helpVerb: deepverb, darkVerb
    verb = 'help' 'about'
    sdesc = "help"
    verblist = ['EXAMINE' 'ASK' 'TELL' 'FOLLOW' 'DIG' 'JUMP' 'PUSH' 'ATTACH' 'WEAR' 'DROP' 'REMOVE' 'OPEN' 'CLOSE' 'PUT' 'TAKE' 'PLUG' 'LOOK' 'SCREW' 'UNSCREW' 'TURN' 'SWITCH' 'FLIP' 'TURN ON' 'TURN OFF' 'SIT' 'LIE' 'GET OUT' 'BOARD' 'AGAIN' 'WAIT' 'INVENTORY' 'BREAK' 'ATTACK' 'CLIMB' 'EAT' 'DRINK' 'GIVE' 'PULL' 'READ' 'THROW' 'STAND' 'HELLO' 'SHOW' 'CLEAN' 'SAY' 'YELL' 'MOVE' 'FASTEN' 'UNFASTEN' 'UNPLUG' 'TYPE' 'LOCK' 'UNLOCK' 'DETACH' 'SLEEP' 'POKE' 'TOUCH' 'CENTER' 'SEARCH' 'KNOCK' 'IN' 'OUT' 'QUIT' 'VERBOSE' 'TERSE' 'SCORE' 'SAVE' 'RESTORE' 'SCRIPT' 'UNSCRIPT' 'RESTART' 'VERSION' 'DEBUG' 'UNDO']
    action(actor) = {
//		clearscreen();
	"Welcome to the main help menu!  This menu contains what we hope is useful information on playing interactive fiction.  There are several sections of assistance available.\b";
    	displayMenu(helpmenu);
    }
;

helpmenu: menu
	sdesc  = "\(Main Help Menu\)"
	ldesc  = "Choose a category: "
	choices = [menuIntroduction menuMovement menuObjects menuCharacters menuTime menuCommands menuReview menuHints]
	enterMsg = "Enter choice:"
	illegalMsg = "Not a valid selection"
	quitMsg = "Return to game"
;

menuIntroduction: menuChoice
	sdesc = "Introduction"
	Chosen = {
		"\bIn a text adventure game, you assume the identity of your player character (PC).  You then play the game by typing commands that describe what you want your character to do.  Unfortunately, the game isn't as smart as you are, so it can't understand nearly as many sentences as a person could.  In this instruction manual, we'll describe most of the types of commands that you will need to use while playing the game.\b
		Note that we've tried to design this game so that you won't need to think of any unusual words or phrases that aren't directly mentioned by the game.  We've especially tried to avoid making you guess a strange verb or an unusual way of phrasing a command.\b
		Each time you see the prompt, >, you type a command.  Your command should be a simple imperative sentence, or a series of imperatives separated by periods.  Press the RETURN (or ENTER) key when you are done typing your command; the game doesn't start interpreting the command until you press RETURN.\b
		You can use capital or small letters in any mixture.  You can use words such as THE and AN when they're appropriate, but you can omit them if you prefer (it's just extra typing).  You can abbreviate any word to six or more letters, but the game will pay attention to all of the letters you type.  For example, you could refer to a FLASHLIGHT with the words FLASHL, FLASHLIG, and so forth, but not with FLASHSDF.";
		morePrompt();
//		clearscreen();
	}
;

menuMovement: menuChoice
	sdesc = "Movement"
	Chosen = {
		"\bAt any time during the game, you are in a location.  The game describes your location when you first enter, and again any time you type LOOK.  In a given location, you can usually reach anything described, so you don't need to type commands to move about within that location.\b
		You move from place to place in the game by typing the direction you want to go.  The game will almost always tell you the directions that you can go from a location, although it usually doesn't tell you what you will find when you go there.  You will probably find it helpful to make a map as you explore the game.  The standard directions the game recognizes are NORTH, SOUTH, EAST, WEST, NORTHEAST, SOUTHEAST, SOUTHWEST, NORTHWEST, UP, and DOWN.  You can abbreviate these to N, S, E, W, NE, SE, NW, SW, U, and D.  In some locations you can also use IN and OUT to enter objects (such as a vehicle or a tent).\b
		Generally, backtracking will take you back to where you started.  For example, if you start off in the kitchen, go north into the living room, then go south, you will be back in the kitchen.  Just remember that there may be some circumstances where this is not the case.\b
		Most of the time, when the game describes a door or doorway, you don't need to open the door to go through the passage; the game will do this for you.  Only when the game explicitly describes a closed door (or other impediment to travel) will you need to type a command to open the door.";
		morePrompt();
//		clearscreen();
	}
;

menuObjects: menuChoice
	sdesc = "Objects"
	Chosen = {
		"\bIn the game, you will find many objects that you can carry or otherwise manipulate.  When you want to do something with an object, type a simple command that tells the game what you want to do; be explicit.  For example, you could type READ THE BOOK or OPEN THE DRAWER.  Most of the objects in the game have fairly obvious uses; you shouldn't have to think of any obscure or unrelated words to manipulate the objects.\b
		You generally don't have to specify exactly where you want to put an object that you wish to carry; you can just type TAKE (followed by the object's name) to carry an object.  We didn't think it was particularly interesting to force you to specify which object you wish to put in your left pocket, which you wish to carry in your right hand, and so forth. However, there is a limit to how many objects you can carry at once, and to how much weight you can handle.  You can carry more objects (but not more weight, of course) by putting some items inside containers (for example, you may be able to put several objects into a box, and carry the box), since this reduces the number of objects you actually have to juggle at once.\b
		You can usually use multiple objects in your sentences.  You separate the objects by the word AND or a comma.  For example:\b
		<TT>\tTAKE THE BOX, THE FLOPPY DISK, AND THE ROPE\n
		\tPUT DISK AND ROPE IN BOX\n
		\tDROP BOX AND BALL</TT>\b
		You can use the word ALL to refer to everything that is applicable to your command, and you can use EXCEPT (right after the word ALL) to exclude certain objects.\b
		\(\"It\" and \"Them\"\)\b
		You can use IT and THEM to refer to the last object or objects that you used in a command.  Some examples:\b
		<TT>\tTAKE THE BOX\n
		\tOPEN IT\n
		\tTAKE THE DISK AND THE ROPE\n
		\tPUT THEM IN THE BOX</TT>";
		morePrompt();
//		clearscreen();
	}
;

menuCharacters: menuChoice
	sdesc = "Other Characters"
	Chosen = {
		"\bYou may encounter other characters in the game.  You can interact in certain ways with these characters.  For example, you can GIVE things to them, SHOW things to them, and you could try to attack them.  In addition, you can ask characters about certain topics and tell them about things:\b
		<TT>\tASK WIZARD ABOUT WAND\n
		\tTELL DRAGON ABOUT SWORD</TT>\b
		You can also tell characters to do something.  To do this, type the character's name, then a comma, then a command that you want the character to perform.  You can type several commands for the character all on the same line by separating the commands with periods.  For example:\b
		<TT>\tROBOT, GIVE ME THE GUN. GO NORTH. PUSH BUTTON. GO SOUTH.</TT>\b
		Of course, you shouldn't expect that characters will always follow your instructions; most characters have minds of their own, and won't automatically do what you ask.";
		morePrompt();
//		clearscreen();
	}
;

menuTime: menuChoice
	sdesc = "Time"
	Chosen = {
		"\bTime passes only in response to commands you type.  Nothing happens while the game is waiting for you to type something.  Each turn takes about the same amount of time.  If you want to let some game time pass, because you think something is about to happen, you can type WAIT (or just Z).  On the other hand, typing a command that the game does not recognize or misspelling a word will not advance the game clock.";
		morePrompt();
//		clearscreen();
	}
;

menuCommands: menuChoice
	sdesc = "Commands"
	Chosen = {
//		clearscreen();
		displayMenu(commandmenu);
	}
;

commandmenu: menu
	sdesc  = "\(Commands help menu\)"
	ldesc  = "The command menu offers help on the instructions you give to the game as you play.\bChoose a category: "
	choices = [menuEntering menuMultiple menuAmbiguous menuUnknown menuSpecial menuEditing]
	enterMsg = "Enter choice:"
	illegalMsg = "Not a valid selection"
	quitMsg = "Return to main help menu"
;

menuEntering: menuChoice
	sdesc = "Entering Commands"
	Chosen = {
		local i, j, cnt, idx, tmp, sorted;
		"\bSome basic verbs that you will use frequently are TAKE (to pick up an object), DROP (to drop an object), OPEN and CLOSE, and EXAMINE (which you can abbreviate to X).  You can PUT an object IN or ON another object when appropriate.  The game recognizes many other verbs as well.  We tried to make all of the verbs obvious; if you find a knob, you will be able to TURN it, and if you find a button, you will be able to PUSH it.  By the same token, you probably won't need to turn the button or push the knob.\b
		Some examples of commands that the game recognizes are shown below:\b";
		sorted := quicksort(helpVerb.verblist);
		cnt := length(sorted);
		"<TT><TABLE>";
		for (i:=1; i<=((cnt+4)/5); i++) {
			"<TR>";
			for (j:=1; j<=5; j++) {
				idx:=((i-1)*5)+j;
				if (idx<=length(sorted)) {
					"<TD>";
					say(sorted[idx]);
					tmp:=14-length(sorted[idx]);
					"\ </TD>";
					say(substr('\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ', 1, tmp*2));
				}
			}
			"</TR>\n";
		}
		"</TABLE></TT>";
/*		<TT>\tGO NORTH\n
		\tNORTH\n
		\tN\n
		\tUP\n
		\tTAKE THE BOX\n
		\tPUT THE FLOPPY DISK INTO THE BOX\n
		\tCLOSE BOX\n
		\tLOOK AT DISK\n
		\tTAKE DISK OUT OF BOX\n
		\tLOOK IN BOX\n
		\tWEAR THE CONICAL HAT\n
		\tTAKE OFF HAT\n
		\tCLOSE BOX\n
		\tTURN ON THE LANTERN\n
		\tLIGHT MATCH\n
		\tLIGHT CANDLE WITH MATCH\n
		\tRING BELL\n
		\tPOUR WATER INTO BUCKET\n
		\tPUSH BUTTON\n
		\tTURN KNOB\n
		\tEAT COOKIE\n
		\tDRINK MILK\n
		\tHIDE BEHIND THE COUCH\n
		\tSTAND UP\n
		\tTHROW KNIFE AT THIEF\n
		\tKILL TROLL WITH SWORD\n
		\tREAD NEWSPAPER\n
		\tLOOK THROUGH WINDOW\n
		\tUNLOCK DOOR WITH KEY\n
		\tTIE THE ROPE TO THE HOOK\n
		\tCLIMB UP THE LADDER\n
		\tTURN THE KNOB\n
		\tJUMP\n
		\tTYPE \"HELLO\" ON THE KEYBOARD\n
		\tTYPE 1234 ON THE KEYPAD\n
		\tGET IN THE CAR\n
		\tGET OUT OF THE CAR\n
		\tGET ON THE HORSE\n
		\tGIVE WAND TO WIZARD\n
		\tASK WIZARD ABOUT WAND</TT>";*/
		morePrompt();
//		clearscreen();
	}
;

menuMultiple: menuChoice
	sdesc = "Multiple commands on one line"
	Chosen = {
		"\bYou can put multiple commands on a single input line by separating the commands with periods or the word THEN, or with a comma or the word AND. Each command still counts as a separate turn.  For example:\b
		<TT>\tTAKE THE DISK AND PUT IT IN THE BOX\n
		\tTAKE BOX. OPEN IT.\n
		\tUNLOCK THE DOOR WITH THE KEY. OPEN IT, AND THEN GO NORTH</TT>\b
		If the game doesn't understand one of the commands on the input line, it will tell you what it couldn't understand, and it will ignore the rest of the commands on the line.";
		morePrompt();
//		clearscreen();
	}
;

menuAmbiguous: menuChoice
	sdesc = "Ambiguous commands"
	Chosen = {
		"\bIf you type a command that leaves out some important information, the game will try to figure out what you mean anyway.  When the game can be reasonably sure about what you mean, because only one object would make sense with the command, the game will make an assumption about the missing information and act as though you had supplied it.  For example,\b
		<TT>\t>TIE THE ROPE\n
		\t(to the hook)\n
		\tThe rope is now tied to the hook.  The end of the\n
		\trope nearly reaches the floor of the pit below.\b</TT>
		If your command is ambiguous enough that the game doesn't feel safe making assumptions about what you meant, the game will ask you for more information. You can answer these questions by typing the missing information.  If you decide you didn't want to bother with the command after all, you can just type a new command; the game will ignore the question it asked.  For example:\b
		<TT>\t>UNLOCK THE DOOR\n
		\tWhat do you want to unlock the door with?\b
		\t>THE KEY\n
		\tWhich key do you mean, the gold key, or the silver key?\b
		\t>GOLD\n
		\tThe door is now unlocked.</TT>";
		morePrompt();
//		clearscreen();
	}
;

menuUnknown: menuChoice
	sdesc = "Unknown words"
	Chosen = {
		"\bThe game will sometimes use words in its descriptions that it doesn't understand in your commands.  For example, you may see a description such as, \"The planet's rings are visible as a thin arc high overhead, glimmering in the sunlight.\"  If the game doesn't know words such as \"rings,\" you can assume that they're not needed to play the game; they're in the descriptions simply to make the story more interesting. For those objects that are important, the game recognizes many synonyms; if the game doesn't understand a word you use, or any of its common synonyms, you are probably trying something that is not necessary to continue the game.";
		morePrompt();
//		clearscreen();
	}
;

menuSpecial: menuChoice
	sdesc = "Special commands"
	Chosen = {
		"\bThe game understands several special commands that you can use to control the game.  You can use these commands at any prompt.\b
		\t\(About the game\)\b
		\(ABOUT\):  Displays whatever information the author thought necessary to give about the game, such as its history, influences, what the author was wearing when he/she wrote it, etc.\b
		\(CREDITS\):  Show a list of the people that developed this game.\b
		\(HELP\):  You're looking at it.\b
		\(HINT\):  Some games will include an automated hint system to help guide you through the trickier parts.\b
		\(VERSION\):  Shows you the current version of the game.\b
		\t\(Command related\)\b
		\(AGAIN\) or \(G\):  Repeats your last command.  If your last input line was composed of several commands, only the last command on the line is repeated.\b
		\(OOPS\):  Allows you to correct the spelling of a word in the last command. You can use OOPS when the game displays this complaint:  \"I don't know the word &lt;word&gt;.\"  Immediately after this message, you can type OOPS followed by the corrected spelling of the misspelled word.  You can only type one word after OOPS, so this command doesn't allow you to correct certain types of errors, such as when you run two words together without a space.\b
		\(UNDO\):  Take back the last command.  This can be used multiple times to take back a series of commands in sequence.  The number of commands that you can undo at any given time varies, but you can generally undo over a hundred commands.\b
		\(WAIT\) or \(Z\):  Causes game time to pass.  When the game is waiting for you to type command, no game time passes; you can use this command to wait for something to happen.\b
		\t\(In-game information\)\b
		\(DIAGNOSE\):  Displays information about your player character's health.\b
		\(INVENTORY\) or \(I\):  Shows the list of items your player character is carrying.\b
		\(LOOK\) or \(L\):  Shows the full description of your location.\b
		\(TERSE\):  Tells the game to only show the full description of a location the first time you enter it.  Later visits to that room will only display the short description.  Of course, you can get a full description at any time by typing LOOK.  This is the default mode for most games.\b
		\(VERBOSE\):  Tells the game to show you the full description of a location every time you enter it, whether or not you have seen the description before.\b
		\t\(Score related\)\b
		\(SCORE\):  Shows you your current score, the maximum possible score, and the number of turns you have taken so far.\b
		\(FULL\):  Lists all of the points you've earned so far and how you earned them.\b
		\(NOTIFY\):  Tells the game whether you want to be notified of score changes when they happen.  When the game starts, NOTIFY is turned on, so you will see a message whenever you do something that changes your score.  If you'd prefer not to see these messages, type NOTIFY. (If you later change your mind, typing NOTIFY again will turn notification back on.)\b
		\t\(Game state control\)\b
		\(QUIT\):  Stops the game, and returns you to your operating system.\b
		\(RESTART\):  Starts the game over from the beginning.\b
		\(RESTORE\):  Restores a position previously saved with the SAVE command.\b
		\(SAVE\):  Stores the current state of the game in a disk file, so that you can come back to the same place later (with the RESTORE command).\b
		\(SCRIPT\):  Starts writing everything you see on the screen (your commands and the game's responses) to a disk file.  The game will ask you for a filename to be used for the transcript; you should select a filename that does not yet exist on your disk, because if you use an existing filename, data in the file will be destroyed.  Use the UNSCRIPT command to stop making the transcript.\b
		\(UNSCRIPT\):  Turns off the transcript being made with the SCRIPT command.";
		morePrompt();
//		clearscreen();
	}
;

menuEditing: menuChoice
	sdesc = "Command editing and recall"
	Chosen = {
		"\bOn most computer systems, the game has a special feature that allows you to use your keyboard's editing keys to modify an input line as you are typing it, and to recall commands that you have previously typed for editing and re-entry.  The specific keys you use vary depending on your system, and some systems don't support this feature at all; see the system-specific documentation for more information.  The following describe the default Windows keystrokes.\b
		While you are typing a command, the game allows you to go back and change part of the line without backspacing over the rest of the line to get there.  Simply use your left and right cursor-arrow keys to move the cursor to any point in the command line.  The BACKSPACE key deletes a character to the left of the cursor, and the DELETE key deletes the character at which the cursor is located.\b
		You can insert new text at the cursor simply by typing the text.  You can press the RETURN (or ENTER) key with the cursor at any point in the line (the cursor need not be at the end of the command line).\b
		You can recall the previous command that you entered by pressing the up cursor-arrow key; pressing the up-arrow key again recalls the command before that, and so forth.  Using the down cursor-arrow key reverses this process, until you get back to the original command that you were typing before you started pressing the up-arrow key.\b
		Once you have recalled a prior command, you can re-enter it by pressing the RETURN key.  In addition, you can edit the command, as described above, before entering the command.\b
		The exact number of commands the game retains depends on the lengths of the commands, but more than a hundred of the most recent commands are generally retained at any given time.";
		morePrompt();
//		clearscreen();
	}
;

menuReview: menuChoice
	sdesc = "Review Mode"
	Chosen = {
		"\bAnother special feature that the game supports on many computer systems is called \"review mode.\"  The game remembers text as it \"scrolls\" off the screen; by invoking recall mode, you can go back and look at text that is no longer visible on the screen.  On most systems, review mode is activated by pressing the PAGE UP key.\b
		Once in review mode, the status line that is normally at the top of the screen will be replaced by the review mode help line.  This line shows the keystrokes you use to view previous screenfuls of text, and also shows you the key that exits review mode and resumes normal game play (this is generally the game key that you used to activate review mode).\b
		While in review mode, your screen becomes a window onto the text that the game has stored away.  When you first activate review mode, you are looking at the very bottom of this text, which is the screenful of text that was just displayed.  Use the up and down cursor-arrow keys to move the window up and down.  Pressing the up cursor-arrow key moves the window up one line, showing you one line of text that has scrolled off the screen.  Most systems also provide keys to move up and down by a full screenful (also called a \"page.\")\b
		To resume game play, press the same key that you used to activate review mode.\b
		The number of screenfuls of text that the game stores away for review depends on how much text is actually on each screen, since the game has a limit on the number of characters it can store, not on the number of lines.  Normally, more than twenty of the most recent screens of text are saved and available for review at any given time.";
		morePrompt();
//		clearscreen();
	}
;

menuHints: menuChoice
	sdesc = "Game Hints"
	Chosen = {
		"\bYou can access the hints menu at any time by typing \"\(hints\)\" at the command prompt.  Only the hints appropriate to your current circumstances are displayed, so you won't get any unwanted spoiler information about later stages of the game.  Using the hints won't affect your final score at all. ";
		morePrompt();
//		clearscreen();
	}
;
