/*
 *  TADS 2 hiding system - Quintin Stone 2002
 *  Creates a new class of object, hideitem, that a player can hide behind
 *  using the hideVerb.  A hideitem is a type of fixed nested room, like a
 *  chair or bed.
 */

// First, the modified classes

modify thing
	verDoHidebehind(actor) =
	{
		"You can't hide behind that.";
	}
	verDoHideBehind(actor, iobj) = {
	}
	verIoHideBehind(actor) = {
		"You can't hide anything behind that. ";
	}
	verDoHideIn(actor, iobj) = {
		self.verDoPutIn(actor, iobj);
	}
	verIoHideIn(actor) = {
		self.verIoPutIn(actor);
	}
	ioHideIn(actor, dobj) = {
		self.ioPutIn(actor, dobj);
	}
	verDoHideUnder(actor, iobj) = {
		self.verDoPutUnder(actor, iobj);
	}
	verIoHideUnder(actor) = {
		self.verIoPutUnder(actor);
	}
	ioHideUnder(actor, dobj) = {
		self.ioPutUnder(actor, dobj);
	}
;

modify basicMe
	verDoHide(actor) = {
	}
	doHide(actor) = {
		hideVerb.action(actor);
	}
;

modify outVerb
	action(actor) = {
		if (actor.location.canHideBehind)
			return standVerb.action(actor);
		pass action;
	}
;


/*
 *  Hide behind <object>
 *
 *  Lets the player hide behind some objects for cover or
 *  concealment.  Transports the player to a nestedroom, as if
 *  entering an open container or vehicle.
 */
hidebehindVerb: deepverb
	verb = 'hide behind' 'get behind' 'hide in' 'hide under' 'get under'
	sdesc = "hide behind"
	doAction = 'Hidebehind'
	rejectMultiDobj(prep) =
	{
		"You can only hide behind one thing at a time. ";
		return true;
	}
;

/*
 *  Hide
 *
 *  As hide behind, but takes no arguments.  If more than one hiding
 *  location is present, prompts the user.
 */
hideVerb: deepverb
	verb = 'hide' 'duck'
	val = 1
	sdesc = "hide"
	hidedesc = { self.thedesc; }
	action(actor) = {
		//askdo;
		local ret, rem, cur, rem2, cur2, tot, i, tot2, j, stat;

		ret := [];

		// Return all hide items
		rem := actor.location.contents;
		tot := length(rem);
		i := 1;
		while (i <= tot)
		{
			cur := rem[i];
			if (cur.canHideBehind)
			{
				ret := ret + cur;
			}
			i := i + 1;
		}
		if (actor.location.canHideBehind) {
			"You are already hiding! ";
			return;
		}	
		if (length(ret) != 1) {
			"There's nothing to hide behind. ";
		} else {
			"(behind ";
			say(ret[1].hidedesc);
			")\n";
			ret[1].doHidebehind(actor);
		}
	}
//	doAction = 'HideBehind'
	prepDefault = behindPrep
	ioAction(behindPrep) = 'HideBehind'
	ioAction(inPrep) = 'HideIn'
	ioAction(underPrep) = 'HideIn'
;

/*
 *  hideitem
 *
 *  This is an item that a player can hide behind.
 */
class hideitem: nestedroom, fixeditem
	canHideBehind = true
	statusPrep = "behind"
	outOfPrep = "out from behind"
	maxbulk = 9999
	isfixed = true
	isListed = nil
	protects = nil
	reachable = [] + self
	hidedesc = self.thedesc
	hideMessage = {
		"You crouch down and hide behind "; self.hidedesc; ".";
	}
	verDoHidebehind(actor) = {}
	doHidebehind(actor) =
	{
		"<<self.hideMessage>> ";
		actor.travelTo(self);
		actor.hideObject := self;
	}
	enterRoom(actor) = {}
	noexit = {
		"%You're% not going anywhere until %you% get%s% <<self.outOfPrep>> <<self.hidedesc>>. ";
		return nil;
	}
	cantReachRoom(otherRoom) =
	{
		"%You% can't reach that from <<self.statusPrep>> <<self.hidedesc>>. ";
	}
	verIoHideBehind(actor) = {}
	ioHideBehind(actor, dobj) = {
		if (addbulk(self.contents) + dobj.bulk > self.maxbulk)
		{
			"There's not enough room behind "; self.hidedesc; ". ";
		} else {
			dobj.doPutIn(actor, self);
		}
	}
;

