/*
 *   Scavenger: A Night's Edge text adventure
 *   Copyright (c) 2003 by Quintin Stone.  All Rights Reserved.
 */

#define VERSION_NUMBER '1.2 (post-comp)'

#define USE_HTML_STATUS

#define USE_HTML_PROMPT

/* Include the TADS "adventure" library. */
#include <adv.t>

/* Include the "game information" utility library. */
#include <gameinfo.t>

/* Include the TADS "standard" library. */
#include <std.t>

/* Extend library, by Neil deMause (neild@echonyc.com), 3/26/96 */
#include <extend.t>

/* Menu system by Daniel Shiovitz */
#include <menu.t>

/* Sackitem by Kevin Forchione */
#include <sackitem.t>

/* AskTell by Suzanne Skinner, 1999 */
#include <asktell.t>

#include "basic.t"
#include "gun.t"
#include "askto.t"
#include "hide.t"
#include "merchant.t"
#include "help.t"
#include "scenery.t"
#include "conversation.t"
#include "hint.t"
#include "throwdir.t"
#include "advancedsack.t"

//#include <wizard.t>

guardDaemon: function;
hallDaemon: function;
healthDaemon: function;
crowdDaemon: function;

/*
 *   Replace the function "commontInit" defined in std.t so that we turn
 *   on HTML mode.
 */
replace commonInit: function
{
	local key;
	randomize();
	initSearch();
	// New verbs to list in the help menu
	helpVerb.verblist += ['HIDE' 'KISS' 'BUY' 'FULL' 'CREDITS' 'HELP' 'VERSION' 'DIAGNOSE' 'NOTIFY' 'HINT' 'REPLY' 'SHOOT' 'FIRE' 'PRY' 'LOAD' 'UNLOAD' 'POUR' 'TRADE' 'GREET' 'HUG' 'TALK TO' 'SCAN'];
	setdaemon(guardDaemon, guard);
	setdaemon(healthDaemon, guard);
	setdaemon(crowdDaemon, parserGetMe());
	// Randomize the name on the security badge
	securitybadge.initialize;
	// Randomize the user security codes
	personDB.initialize;
	// Randomize the conference guard password
	confguard.initialize;
	/* display the special code sequence to turn on HTML recognition */
	"\H+";
	// Set up the 'about box' text
	global.aboutbox;
	"<<global.dispParagraph>>Unbelievable.  This could very well be it.  The big score.
	<<global.dispParagraph>>Every scavenger dreams of that one discovery that will grant riches beyond the dreams of avarice.  Every scavenger envisions walking into the hidden chamber of some pre-war military compound or corporate R&D facility and finding the forgotten secrets of humanity's genius.  Of course, you can count the number of times it's happened on one hand.  While everyone in your line of work wishes for it, no one really expects it to happen.  That included you.
	<<global.dispParagraph>>Until you found the scrap of paper, that is.<<global.dispParagraph>>";
	morePrompt();
	clearscreen();
	// Clearscreen erases the 'about box' text too.  Refresh it.
	global.aboutbox;
}

/* 
 *   Retrieve the card-catalog information for the game.  This information
 *   is written to a file called GameInfo.txt during preinit, and can then
 *   be stored in the compiled .gam file for retrieval by automated tools.
 *   This information is useful for archive maintainers and other
 *   searching and browsing tools.  
 */
getGameInfo: function
{
	/* build and return the game information list */
	return ['Name', 'Scavenger',
			'Byline', 'Quintin Stone',
			'Desc', 'Search a war-shattered world for secrets of the past.',
			'Version', VERSION_NUMBER,
			'ReleaseDate', getGameInfoToday(),
			'Language', 'en-US'];
}

replace initRestart: function(parm)
{
	global.restarting := true;
	clearscreen();
	global.aboutbox;
}

replace eatDaemon: function(parm) {}
replace sleepDaemon: function(parm) {}

modify global
	turnsofar = 0		// no turns have transpired so far
	score = 0			// no points have been accumulated yet
	won = nil			// Player has not won yet
	maxscore = 14		// maximum possible score
	verbose = true		// we are currently in TERSE mode
	awakeTime = 0		// time that has elapsed since the player slept
	sleepTime = 400		// interval between sleeping times (longest time awake)
	lastMealTime = 0	// time that has elapsed since the player ate
	eatTime = 200		// interval between meals (longest time without food)
	lamplist = []		// list of all known light providers in the game
	inbase = nil		// for scoring, if player has entered base
	notified = true		// player notified of score increases
	time = 'Afternoon'	// Time
	chapter = 1			// Chapter
	heardpassword = nil	// Password
	secondpower = nil	// Does second floor have power?
	thirdpower = nil	// Does third floor have power?
	sicktime = 0		// How long since drinking the filthy water
	radtime = 0			// How long since being radiated
	radlevel = [nil nil nil nil nil nil]	// For radiation effects
	pilltime = 0		// Rad pill timer
	genvented = nil		// Generator vented?
	venttime = 0		// When vented
	gothelmet = nil		// Indicates got helmet
	insecond = nil		// Entered second floor
	incommissary = nil	// Entered commissary
	inthird = nil		// Entered third floor
	inbarracks = nil	// Entered barracks
	opendoor = nil		// Opened the security door for first time
	broughthelm = nil	// Whether the helm was brought back to Arcadia
	
	// Scores
	scDiscount = nil			// 1
	scBuyGPS = nil				// 1
	scEnterBase = nil			// 1
	scFoundBadge = nil			// 1
	scFixPanel = nil			// 1
	scFoundCode = nil			// 1
	scSecondFloor = nil			// 1
	scThirdFloor = nil			// 1
	scPastRad = nil				// 1
	scOpenVault = nil			// 1
	scKillLeader = nil			// 1
	scRescueGirl = nil			// 1
	scDestroyBase = nil			// 1
	scSellDevice = nil			// 1
	scDestroyDevice = nil		// 1
	scKeepDevice = nil			// 1
	scKillGirl = nil			// -5
	scCheated = nil				// -5

	// Game ranks and the thresholds to achieve those ranks
	gameranks = ['Hobo' 'Deadbeat' 'Drifter' 'Scrounger' 'Fortune-hunter' 'Speculator' 'Daredevil' 'Pathfinder' 'Trailblazer' 'Expeditionist' 'Scavenger']
	gamescores = [0 2 4 6 8 9 10 11 12 13 14]

	credits = {
		"\b\(Scavenger\nNight's Edge interactive fiction\)\n
		Written and programmed by Quintin Stone\n
		Visit the Scavenger home page at <A HREF=\"http://www.rps.net/NE/Scavenger\">http://www.rps.net/NE/Scavenger</A>\n
		Runs on TADS 2.5 by Michael J. Roberts\n
		Includes:
		\n\tHelp text by Michael J. Roberts
		\n\tExtend library by Neil deMause (neild@echonyc.com)
		\n\tMenu system by Daniel Shiovitz
		\n\tSack item library by Kevin Forchione\b
		Alpha testing by LouZiffer and Bad Mojo.  Thanks, guys.\b
		Beta testing by Greg Pallis, Stark Springs, (the infamous tester) Ricardo Signes, Frank Otto, Daphne Brinkerhoff, Mike Sousa, Jessica Knoch, Peter Berman, and Guilherme De Sousa.  Many thanks!\b
		Special thanks for Bad Mojo for co-creating the Night's Edge world.\b
		And my appreciation to everyone who voted for Scavenger and helped it place 3rd in the 2003 IF Comp.";
	}
	notes = {
		"<<global.dispParagraph>>As I stated in my readme file, Scavenger grew out of an idea for a short story I had.  The game itself was not meant to be particularly focused on any one particular style, but a mix of puzzles, exploration, and atmosphere.  I began working on the game in September of 2002 while waiting for the release of the 2002 IF Competition games.  Most of the code was finished by November, and the next several months were spent polishing and reacting to the input from testers.<<global.dispParagraph>>";
		"The Infocom game that probably made the biggest impression on me in my youth was Planetfall, which probably comes as no surprise now that you've played Scavenger.  There are likely more parallels than even I realize.<<global.dispParagraph>>";
		"As I play through various \"modern\" IF from the past 10 years, I'm learning how some ideas repeat themselves.  While playing the well-made game \"Babel\", I was rather sadly surprised to find a block-door-with-pole puzzle nearly identical to my own, and at a point where my game was effectively finished.  Not relishing the thought of rewriting of one of the few major puzzles, I decided to simply leave it (and the Babel-ish corroded door hinges) in and throw  myself on the mercy of the court.  ";
		"Great minds think alike and all that, right?  Ironically, I then found a review of \"Babel\" that criticized it for duplicating a central idea found in the game \"Delusions\".  I guess it's just the nature of the beast.  There are so many amateur works of interactive fiction now floating around the internet that no author can be expected to have played them all.  I'm sure as I play though the archive of games I'll come across one that's pretty much identical to the idea for my next game, forcing me to smash my computer with a crowbar.  Ah well.<<global.dispParagraph>>";
		"Quintin Stone 5/3/03";
	}
	amusing = {"Have you tried...
		\n\t...winning without buying anything but the GPS?
		\n\t...winning with each of the extra items from the shop?
		\n\t...getting all 14 points?
		\n\t...asking Charlie about various famous IF authors?
		\n\t...attacking Charlie with your flashlight?
		\n\t...dropping a live grenade in Charlie's shop?
		\n\t...dropping a live grenade in the crumbling building?
		\n\t...spending a full 40 turns in the barracks?
		\n\t...shooting Charlie?
		\n\t...shooting yourself?
		\n\t...eating the grenade?
		\n\t...smelling bandits?
		\n\t...drinking the screwdriver?
		\n\t...pushing all of the various buttons on the generator?
		\n\t...walking through the radiation without a rad suit?
		\n\t...and then dropping the radiation poisoning?
		\n\t...typing \"cheat\"?
		\n\t...typing \"xyzzy\"?
		\n\t...typing it more than once?";
	}
	aboutbox = {
		"<ABOUTBOX><body BGCOLOR=#CBB99D TEXT=#006A25>
		<CENTER><H1>Scavenger</H1><H3>Night's Edge Interactive Fiction</H3>
		<B>By Quintin Stone</B>\n<B>Copyright 2003</B><BR>&nbsp;<P>
		http://www.rps.net/NE/Scavenger/index.shtml</CENTER></ABOUTBOX>";
	}
;

modify version
	sdesc = {
		local ver;
		"\b\(Scavenger\nNight's Edge interactive fiction\)\n
		By Quintin Stone\n
		Version ";
		ver := getGameInfo();
		say(ver[8]);
		"\nBuild Date: ";
		say(__DATE__);
		"\nType \(CREDITS\) or \(HELP\) for more information about this game.";
	}
;

// Full score listing
fullVerb: deepverb
	verb = 'full' 
	sdesc = "full"
	action(actor)= {
		self.displayFull;
		abort;	
	}
	displayFull = {
		if (global.score=0 && (!global.scCheated) && (!global.won)) {
			// The computer gets sullen if you type xyzzy too many times
			if (global.jerk) {
				"You have no points, jerk. ";
			} else {
				"You have no points. ";
			}
		} else {
			"You have earned the following:\b";
	
			if (global.scDiscount) "\n\t1 point for getting the store discount";
			if (global.scBuyGPS) "\n\t1 point for buying the GPS";
			if (global.scEnterBase) "\n\t1 point for infiltrating the base";
			if (global.scFoundBadge) "\n\t1 point for finding the security badge";
			if (global.scFixPanel) "\n\t1 point for repairing the security door panel";
			if (global.scFoundCode) "\n\t1 point for finding the security code";
			if (global.scSecondFloor) "\n\t1 point for activating the second floor power";
			if (global.scThirdFloor) "\n\t1 point for activating the third floor power";
			if (global.scPastRad) "\n\t1 point for bypassing the radiation";
			if (global.scOpenVault) "\n\t1 point for opening the containment vault";
			if (global.scKillLeader) "\n\t1 point for killing the bandit leader";
			if (global.scRescueGirl) "\n\t1 point for rescuing Iona";
			if (global.scDestroyBase) "\n\t1 point for destroying the raider stronghold";
			if (global.scSellDevice) "\n\t1 points for selling the device";
			if (global.scDestroyDevice) "\n\t1 point for destroying the device";
			if (global.scKeepDevice) "\n\t1 point for keeping the device for yourself";
			if (global.scKillGirl) "\n\t-5 points for killing Iona";
			if (global.won && global.scCheated) "\n\t-5 points for forgetting that cheaters never prosper";
			if (global.score = 1)
				"\bTotal score: <<global.score>> point out of <<global.maxscore>>\b";
			else
				"\bTotal score: <<global.score>> points out of <<global.maxscore>>\b";
		}
		"This gives you a rank of ";
		say(getRanking(global.score));
		".\n";
	}
;

// A bit of an easter egg
cheatVerb: darkVerb
	verb= 'cheat' 
	sdesc = "cheat"
	action(actor)= {
		"You're feeling much more prosperous. ";
		actor.money += 1000;
		global.scCheated := true;
		abort;
	}
;

modify xyzzyVerb
	count = 0
	action(actor) = {
		switch (count) {
			case 0:
				"Nothing happens.<<global.dispParagraph>>You weren't really expecting anything, were you?  It just doesn't fit in with the genre.<<global.dispParagraph>>Oh come on, don't tell me you're disappointed.  I know, I know, you were expecting some hilarious little blip, or the Spanish Inquisition, or something.  Well, no, not the Spanish Inquisition, because no one ever expects them.  But I'm sitting here as I write this, watching television and stuffing barbeque potato chips into my mouth, and absolutely nothing humorous is coming to mind.  So... hey, maybe next time.  Okay?";
				break;
			case 1:
				"Nothing happens... again. ";
				break;
			case 2:
				"Again... nothing happens. ";
				break;
			case 3:
				"Still nothing happens. ";
				break;
			case 4:
				"Stop doing that. ";
				break;
			case 5:
				"Please stop doing that. ";
				break;
			case 6:
				"Okay, fine.  I'm disabling the command now. ";
				break;
			case 7:
				"I don't know the word \"xyzzy\". ";
				break;
			case 8:
				"I really don't know the word \"xyzzy\". ";
				break;
			case 9:
				"Well, that didn't work.  Look, here's 5 dollars.  Just... take it and promise to stop typing that.  Nothing is going to happen! ";
				parserGetMe().money += 5;
				break;
			case 10:
				"Oh, I see.  Okay, fine.  If that's the way you want to be.... ";
				global.score := 0;
				global.jerk := true;
				endgame(nil);
				break;
		}
		count++;
	}
;

// The global.time property is either Morning, Afternoon, or Night
replace scoreFormat: function(points, turns)
{
	/* return a string with the standard points/turns display */
	return global.time + '\ \ \ \ ' + cvtstr(points) + '/' + cvtstr(turns);
}

// Fix a couple of diambiguation issues: the outdoor guard is has a 
// 'phantom' guard counterpart so he can be viewed from a distance.  The
// radiation poisoning is an easter egg that I make sure never gets
// confused with the radiation suit.
modify deepverb
	disambigDobj(actor, prep, iobj, verprop, wordlist, objlist, 
	flaglist, numberWanted, isAmbiguous, silent) = {
		local obj, retlist, i;
		if (isAmbiguous && inList(radpoisoning, objlist)) {
//			objlist := objlist - radpoisoning;
//			if (length(objlist) = 1) {
				isAmbiguous := nil;
				retlist := [];
				for (i:=1; i<=length(objlist); i++) {
					if (objlist[i] != radpoisoning) {
						retlist := retlist + objlist[i];
						retlist := retlist + flaglist[i];
					}
				}
				return retlist;
//			}
		}
		if (isAmbiguous && inList(guard, objlist) && inList(phantomguard, objlist)) {
			isAmbiguous := nil;
			retlist := [];
			if (phantomguard.location = nil) {
				for (i:=1; i<=length(objlist); i++) {
					if (objlist[i] != guard) {
						retlist := retlist + objlist[i];
						retlist := retlist + flaglist[i];
					}
				}
				return retlist;
			} else {
				for (i:=1; i<=length(objlist); i++) {
					if (objlist[i] != phantomguard) {
						retlist := retlist + objlist[i];
						retlist := retlist + flaglist[i];
					}
				}
				return retlist;
			}
		}
		pass disambigDobj;
	}
;

modify thing
	takeMessage = {
		if (location = brambles) {
			return "\^%you% reach carefully into the twisty thorns
				and manage to snag <<self.thedesc>> without
				scratching %your%self up too badly. ";
		}
		pass takeMessage;
	}
	verDoSmell(actor)={
		if (radsuit.isworn) {
			"You can't smell anything while wearing a radiation suit.  The filters effectively screen out any odors. ";
		}
	}
;

modify room
	// For the grenade.  The scorchdesc here is the default, but it can
	// be overridden if the explosion modifies a room in a particular way.
	scorchdesc = {
		"It appears that there's been an explosion in this <<self.roomtype>>.  There's a small crater in the ground, surrounded by blackened scorch marks that radiate outwards. ";
	}
	roomtype = {
		if (self.bOutdoor)
			"area";
		else
			"room";
	}
	// Show telepathy messages while wearing the helmet
	lookAround(verbosity) = {
		inherited.lookAround(verbosity);
		helmet.detect(self);
	}
	// Some ceilings may look unusual
	ceilingdesc = "You see nothing unusual about it."
;

modify darkroom
	lookAround(verbosity) = {
		if (self.islit)
			inherited.lookAround(verbosity);
		else
			"It's pitch black. ";
//		helmet.detect(self);
	}
;

modify nestedroom
    lookAround(verbosity) =
    {
		inherited.lookAround(verbosity);
		helmet.detect(self);
    }
;

modify Me
	hideObject = nil
	ldesc = "Well, you don't have a mirror handy, but you're pretty sure that your hair looks good enough for wandering around in a radioactive wasteland. "
	maxweight = 100
	maxbulk = 40
	weight = 180		// actors are pretty heavy
	bulk = 20		// and pretty bulky
	money = 900
	health = 0
	// When entering a new room, show the Geiger counter message
	travelTo(room) = {
		inherited.travelTo(room);
		geiger.detect(room);
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	// Suicide
	doAttack(weapon) = {
		if (weapon = pistol) {
			"Waves of despair overcome you.  You'll never succeed, you're a failure, nobody loves you, everybody hates you, etc.  You place the end of the silencer into your mouth and pull the trigger. ";
			lose(true);
		}
		if (weapon = rifle) {
			"Waves of despair overcome you.  You'll never succeed, you're a failure, nobody loves you, everybody hates you, etc.  You place the end of the short barrel into your mouth and pull the trigger. ";
			lose(true);
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			"Attempting suicide with <<weapon.adesc>>?  You <i>are</i> desperate. ";
		}
		if (weapon = parserGetMe()) {
			"Your attempt to strangle yourself has failed. ";
		}
	}
	verIoAttackWith(actor) = {}
	ioAttackWith(actor, dobj) = {
		dobj.doAttack(self);
	}
	smelldesc = {
		switch (global.chapter) {
			case 1:
				"Not too bad. ";
				break;
			case 2: case 3:
				"You don't get many chances to shower in the wasteland. ";
				break;
		}
	}
;

modify diagnosisVerb
	action(actor) = {
		switch(actor.health) {
			case 0:
				"You are in perfect health. "; break;
			case 1:
				"You're feeling a bit under the weather. "; break;
			case 2:
				"You feel decidedly unwell. "; break;
			case 3:
				"Not so good. "; break;
			case 4:
				"You're barely hanging on. "; break;
			case 5:
				"Better get a doctor... or a priest. "; break;
			case 6:
				"It was nice knowing you. "; break;
		}
	}
;

// Verb used in testing the endgame
/*
magicVerb: deepverb
	verb = 'magic'
	sdesc = "magic"
	action(actor) = {
		local obj;
		for (obj := firstobj(); obj != nil; obj := nextobj(obj)) {
			if (obj.location && !obj.isfixed)
				obj.moveInto(parserGetMe());
		}
		circuitbreaker.moveInto(parserGetMe());
		desk1.gotbadge := true;
		photo.moveInto(securitybadge);
		grenade.exploded := nil;
		grenade.armed := nil;
		pin.moveInto(grenade);

		flashlight.isActive := true;
		actor.travelTo(lab);
	}
;
*/

/*
 *   We must define the initial location, where the player starts the
 *   game.  By the convention used in std.t, this location is called
 *   "startroom".
 *   
 *   We'll make startroom an object of class "room", which is the basic
 *   class defined in adv.t for a location in the game.  
 */
startroom: room
	sdesc = {
		// Spacing between the version info and the room text
		if (!self.isseen)
			"\b\b";
		"Howe Street, Arcadia";
	}
	ldesc = "The few street lights that still work on this run down section of road do little to dispel the everpresent gloom of the city.  Even in the middle of the day the sun is barely more than a faint glow in the thick clouds above.  The street itself is ancient crumbling asphalt; they simply don't maintain the roads this far from the city core.  This one heads north, back towards your meager home, and south, out of the city.  A cross street leads east towards CyTek Corporation's main compound, and a small merchant shop you know well sits among derelict buildings to the west. "
	bOutdoor = true
	north = home
	east = corp
	in = {
		if (global.chapter = 1)
			return shop;
		"A 'Closed' sign hangs on the door.  It doesn't look like Charlie has opened yet today. ";
	}
	west = shopdoor
	south = southgate
	listendesc = "The sounds of the city are fairly muted this far out on the fringes. "
	exitlist = {
		if (global.chapter = 1) {
			"Roads run north, south and east.  To the west is an open shop. ";
		} else {
			"Home lies to the north.  CyTek Corporation is down the road to the east. ";
		}
		return nil;
	}
	listContents = {
		"You can see "; listcont(self); " lying on the street. ";
	}
;

shopdoor: doorway, lockableDoorway, keyedLockable
	/* the door is in the starting room */
	location = startroom
	isopen = true
	islocked = nil
	mykey = shopdoor

	/* define the words the player can use to refer to the door */
	noun = 'west' 'w' 'door'

	/* the short description */
	sdesc = "shop door"
	ldesc = {
		if (global.chapter = 1) {
			if (displaycase.shot > 2) {
				"It appears that Charlie has closed and locked the door. ";
			} else {
				if (self.isopen)
					"The door is open and leads into the shop. ";
				else
					"The door into the shop is closed. ";
			}
		} else {
			"The door has a closed sign hanging on it.  It doesn't look like Charlie has opened yet today. ";
		}
	}
	doordest = shop
	destination = {
		if (displaycase.shot > 2) {
			"It appears the door is locked. ";
			return nil;
		}
		if (global.chapter = 1)
			return doordest;
		"The door has a closed sign hanging on it.  It doesn't look like Charlie has opened yet today. ";
		return nil;
	}
	verDoClose(actor) = {
		if (!self.isopen) {
			"The door is already closed. ";
		}
		"A voice from inside shouts, \"Hey!  Leave that alone.\" ";
	}
;

/*
 *   Define the north road.  This is one possible ending in chapter 3.
 */
home: doorway
	/* the door is in the starting room */
	location = startroom
	isopen = true

	/* define the words the player can use to refer to the door */
	noun = 'north' 'n'

	/* the short description */
	sdesc = "north road"
	ldesc = "The broken pavement of the north road heads further into Arcadia, towards home. "
	doordest = {
		local helm, girl;
		if (global.chapter = 1) {
			"You can't head back now; your journey is just beginning. ";
			return nil;
		} else {
			helm := isinside(helmet, parserGetMe());
			if (global.broughthelm && !helmet.isbroken && !helm) {
				"You can't bring yourself to just leave the helm behind, after all you went through to find it. ";
				return nil;
			}
			girl := (parserGetMe().myfollower = iona);
			if (helmet.isbroken) {
				incscore(1);
				global.scDestroyDevice := true;
				"You return to your meager dwelling, your heart heavy with the knowledge that you have failed in your quest... or did you?  There's a large part of you that feels you did the right thing.  That maybe there was a time when mankind could have done good with such a device, but that time is now gone.  Now, your life continues as normal.  ";
				if (girl) {
					incscore(1);
					global.scRescueGirl := true;
					"Almost normal.  With an adopted daughter, you can't help but foresee your life changing.  For the better?  Only the future will tell. ";
				} else {
					"Maybe the next time will be... the big score. ";
				}
			} else if (helm) {
				incscore(1);
				global.scKeepDevice := true;
				"Your journey has come full circle, now with your final decision made.  The technology of a century past will remain in your hands.  ";
				if (girl)
					"You and Iona return to your meager dwelling, where you place the helmet on the bed, and simply stare at it.  ";
				else
					"You return to your meager dwelling, place the helmet on the bed, and simply stare at it.  ";
				"Your mission was a success, and though you did not find the riches you'd hoped by selling the device to a corporation, you begin to believe that maybe, just maybe, you'll be able to do better on your own";
				if (girl) {
					": just you and your newly adopted daughter Iona";
					incscore(1);
					global.scRescueGirl := true;
				}
				".  You aren't sure what comes next, but the one thing you are sure about is that your life as a scavenger is over. ";
			} else {
			}
			doScoreNotify();
			"<<global.dispParagraph>>";
			endgame(true);
		}
	}
;

/*
 *   Define the east road.  This is another possible ending in chapter 3.
 */
corp: doorway
	/* the door is in the starting room */
	location = startroom
	isopen = true

	/* define the words the player can use to refer to the door */
	noun = 'east' 'e'

	/* the short description */
	sdesc = "east road"
	ldesc = "A side street leading east toward the CyTek corporate headquarters. "
	doordest = {
		local girl := (parserGetMe().myfollower = iona);
		if (isinside(helmet, parserGetMe()) && !helmet.isbroken) {
			"With some trepidation, you head down the cross street towards CyTek";
			if (girl)
				", Iona trailing closely behind you";
			".  The process is familiar to you and every other scavenger.  The pitch.  The demonstration.  The negotiation.  And, finally, the sale.  The only real difference this time is the expression of shock on the agent's face... and the size of the purse.  With some effort, you put out of your mind thoughts of what exactly CyTek plans to do with the technology you've delivered to them.  That's none of your concern, is it?  All you know is that with this money, ";
			if (girl) {
				"Iona and yourself";
				incscore(1);
				global.scRescueGirl := true;
			} else {
				"you";
			}
			incscore(1);
			global.scSellDevice := true;
			" will live quite comfortably for some time. ";
			doScoreNotify();
			"<<global.dispParagraph>>";
			endgame(true);
		} else {
			if (global.chapter = 1)
				"Maybe when you have something to offer the corporation.  Until then, it's probably a good idea just to leave it be. ";
			else {
				if (global.broughthelm)
					"Without the helm, you have nothing to offer them. ";
				else
					"You have returned with nothing to offer them. ";
			}
		}
		return nil;
	}
;

pack: sackItem, knownTopic
	location = Me
	sdesc = "survival pack"
	noun = 'pack'
	adjective = 'leather' 'survival'
	bulk = 14
	weight = 7
	maxbulk = 160
	bulkthresh = 20
	isworn = true
	isopen = nil
	ldesc = {
		"This is your trusty survival pack.  You've been carrying it for years.  Sure, the leather is worn thin in a few spots, and yeah, some of the seams have been re-sewn two or three times, but you just can't bring yourself to replace it. ";
		switch (global.chapter) {
			case 1:
				"It's full of the travel rations you expect to need on your journey. ";
				break;
			case 2:
				"A little more than half of your travel rations remain inside. ";
				break;
		}
		if (self.isopen) {
			"The pack is currently open and ";
			if (self.contentsVisible and itemcnt(self.contents) <> 0) {
				"inside %you% also see%s% "; listcont(self); ". ";
			} else {
				"otherwise empty. ";
			}
		} else {
			"The pack is currently closed. ";
		}
	}
	smelldesc = "It has a distinct odor of old leather. "
	doSearch(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0)
			"Aside from the travel rations, %you% see%s% <<listcont(self)>>. ";
		else
			"Aside from the travel rations, there's nothing in <<self.thedesc>>. ";
	}
	doOpen(actor) =
	{
		if (itemcnt(self.contents))
			"Opening <<self.thedesc>> reveals <<listcont(self)>>. ";
		else
			"The pack contains your travel rations and nothing else. ";
		self.isopen := true;
	}
;

// Hands are basically scenery, though you can hit things with your fists
// if you like.
hands: item, floatingItem
	sdesc = "your hands"
	thedesc = "your hands"
	noun = 'fist' 'fists' 'hand' 'hands'
	location = { return parserGetMe(); }
	isThem = true
	isListed = nil
	verDoDrop(actor) = "Your hands appear to be attached to your arms. "
	verDoPutIn(actor, iobj) = "Your hands appear to be attached to your arms. "
	verDoPutOn(actor, iobj) = "Your hands appear to be attached to your arms. "
	verDoPutUnder(actor, iobj) = {
		if (iobj = desk1) {
			"Your arm is too large to fit under the desk. ";
		} else {
			"Your hands appear to be attached to your arms. ";
		}
	}
	verDoThrow(actor) = "Your hands appear to be attached to your arms. "
	verDoThrowAt(actor, iobj) = "Your hands appear to be attached to your arms. "
//	verIoAttackWith(actor) = {}
	ioAttackWith -> Me
	verDoAttack(actor) = "You clap your hands together violently. "
	verDoAttackWith(actor, iobj) = "You need your hands in good condition. "
;

/*
 * Player's trusty flashlight
 */ 
flashlight: switchLightItemNotFixed, lightsource
	location = Me
	sdesc = "flashlight"
	noun = 'flashlight' 'light' 'torch'
	adjective = 'flash'
	bulk = 3
	weight = 3
	ldesc = {
		"Your standard SureFire pocket flashlight.  About six inches long, with a rubberized, waterproof exterior.  You've had it for a number of years and it's come in handy more than once.  It puts out a bright beam that's nice and uniform.  Has a battery life of 60 continuous hours of function.  ";
		if (self.isActive)
			"It's currently on. ";
		else
			"It's currently off. ";
	}
	islit = {
		if (self.location = pack)
			return nil;
		return isActive;
	}
	moveInto(obj) = {
		local waslit := parserGetMe().location.islit;
		inherited.moveInto(obj);
		if (waslit && !parserGetMe().location.islit)
			"It's now pitch black. ";
	}
	verIoAttackWith(actor) = {}
	ioAttackWith(actor, dobj) = {
		dobj.doAttack(self);
	}
	doTurnoff(actor) =
	{
		self.isActive := nil;
		self.islit := self.isActive;
		"Okay, "; self.thedesc; " <<self.isdesc>> now switched off. ";
		if (not actor.location.islit) {
			if (findRootRoom(actor) = fadedpath)
				"It's now quite dark, except for a bright, flickering light to the southeast. ";
			else
				"It's now pitch black. ";
		}
	}
	verDoOpen(actor) = {
		"The only reason to open it would be to replace the batteries, which are fine. ";
	}
	verDoClose(actor) = {
		"It's already closed. ";
	}
;

/*
 * Geiger counter, detects radiation
 */ 
geiger: switchItemNotFixed, readable, knownTopic
	location = Me
	sdesc = "Geiger counter"
	noun = 'counter'
	adjective = 'geiger'
	bulk = 5
	weight = 6
	ldesc = {
		"Your basic 22nd century Geiger counter, this one from Rabisi Corporation.  This device counts radioactive particles as they pass through its detector, determining the overall level of radioactivity in an area.  No scavenger would last long out in the wastes without one of these.  Like all modern Geiger counters, this one has a directional indicator display, alerting you to nearby dangerous levels of radiation. ";
		if (self.isActive) {
			"It's currently on";
			switch (parserGetMe().location) {
				case powerplant:
					" and flashing wildly.  An arrow pointing south is brightly lit"; break;
				case lab:
					" and flashing wildly.  An arrow pointing west is brightly lit"; break;
				case steamy:
					" and flashing wildly.  The arrows in all directions are brightly lit"; break;
			}
			". ";
		} else {
			"It's currently off. ";
		}
	}
	doTurnon(actor) = {
		inherited.doTurnon(actor);
		self.detect(actor.location);
	}
	// This gets called whenever a room is entered or looked at
	detect(room) = {
		local i;
		if (room = nil)
			return;
		if (!self.isActive)
			return;
		if (self.done)
			return;
		if (!isinside(self,parserGetMe()))
			return;
		if (room.radmessage) {
			"<<global.dispParagraph>><<room.radmessage>> ";
		} else {
			if (room = powerplant || room = lab || room = steamy)
				return;
			switch (rand(6)) {
				case 1:
					"<<global.dispParagraph>>Your Geiger counter chitters quietly. ";
					break;
				case 2:
					"<<global.dispParagraph>>A brief burst of noise comes from your Geiger counter but it quickly falls silent again. ";
					break;
				case 3:
					"<<global.dispParagraph>>You hear a faint ticking from your Geiger counter, but nothing to be concerned about. ";
			}
		}
		self.done := true;
		notify(self, &reset, 1);
	}
	reset = {
		self.done := nil;
	}
	readdesc = {
		if (!self.isActive) {
			"The Geiger counter isn't on. ";
			return;
		}
		if (findRootRoom(parserGetMe()).radmessage) {
			"<<findRootRoom(parserGetMe()).radmessage>> ";
		} else {
			"Radiation levels appear to be in the safe zone. ";
		}
	}
	verIoScanWith(actor) = {}
	ioScanWith(actor, dobj) = {
		if (findRootRoom(parserGetMe()) = steamy) {
			"As far as you can determine, everything in this room is giving off lethal levels of radiation. ";
		} else {
			"The readings appear safe enough. ";
		}
	}
;

/*
 * Scrap of paper
 */ 
scrapofpaper: readable, paper, knownTopic
	location = Me
	sdesc = "scrap of paper"
	noun = 'paper' 'message'
	adjective = 'scrap'
	bulk = 0
	weight = 0
	ldesc = {
		"It was about four days west of Arcadia that you came upon the ruins of a pre-war airbase.  The entire complex had been nearly leveled, unfortunately.  No sign of radiation, though, so you went in anyway.  There wasn't much to see, and by the looks of it, you guessed that others had already stripped the place clean.  On your way out, though, this scrap of paper caught your eye.  It was scorched in multiple places, but the dry air managed to preserve it remarkably well for the past hundred or so years.  Some of the original message is still readable. ";
	}
	readdesc = {
		"The only pieces of the message that you've been able to make out  so far are:\b\b
		...IEFS OF STAFF
		...NERAL ARTH...NALDS
		...ROJECT DISTANT PATH\b
		DUE TO THE INCREASED INS...ITY FROM INSURGENT FORC...ITARY RESEARCH COM...ADO SPRINGS, ALL PERSONN...RMATION RELATED TO PROJE...NT PATH ARE BEIN... COMPLETED HAVEN RESEAR... UNDER THE HEA...ES FROM 83RD ARMORED INFANTRY REASSI... TO OVERSEE TRANSFER OF MATERIALS.  GPS C... 33-28-19.158N / 111-57-49.524W.  LOCATION CLASSIFIE... SECRET CLEARANCE AND BE AP...\b\b
		You'd probably be able to get a little more with a highly detailed scan, but you don't have the necessary equipment and you simply can't entrust this information to anyone else. ";
	}
	verDoDrop(actor) = "You feel that it's much too important to leave lying around. "
	verDoPutIn(actor, io) = {
		if ((io != pack) && (io != scenerycampfire2))
			"You'd rather not. ";
	}
	verDoPutOn(actor, io) = "You'd rather not. "
	verDoGiveTo(actor, io) = "You'd rather not. "
	verDoShowTo(actor, io) = "You'd rather not. "
	moveInto(obj) = {
		if (obj = nil)
			pass moveInto;
		if ((obj != pack) && (obj != parserGetMe()) && (!global.radlevel[5])) {
			" Quickly, you snatch up the scrap of paper. ";
			inherited.moveInto(parserGetMe());
			return;
		}
		pass moveInto;
	}
	doBurn(actor) = {
		"It's served its purpose.  You toss the paper into the fire and watch as it is quickly reduced to ash. ";
		self.moveInto(nil);
	}
	doThrowAt(actor, io) = "You'd rather not. "
	doThrowTo(actor, io) = "You'd rather not. "
;

rations: item, knownTopic
	sdesc = "your travel rations"
	adesc = self.sdesc
	thedesc = self.sdesc
	noun = 'ration' 'rations' 'food' 'meat' 'biscuits' 'cheese'
	adjective = 'travel' 'freeze' 'dried' 'hard' 'tack'
	location = pack
	bulk = 0
	isThem = true
//	isListed = nil
	ldesc = {
		"These are fairly typical travel rations.  They mainly consist of freeze dried meat, hard tack biscuits, and something claiming to be cheese (a dubious assertion); all individually wrapped to give them a shelf life of several years.  The rations are also quite tasteless, with a texture somewhere between cardboard and asphalt.  Still, they do keep you alive. ";
		switch (global.chapter) {
			case 1:
				"You should have all you need for your trip, and then some. ";
				break;
			case 2:
				"A little more than half of your travel rations remain. ";
				break;
			case 3:
				"Very little remains. ";
				break;
		}
	}
	verDoEat(actor) ={
		"You're not feeling hungry right now. ";
	}
	smelldesc = "They're just odorless as they are tasteless. "
	jugglemessage = "There are too many packets to juggle.  They're best left in your pack. "
	bettermessage = "The rations are best left where they are.  You're going to need them later. "
	verDoTake(actor) = self.jugglemessage
	verDoTakeOut(actor, io) = self.jugglemessage
	verDoDrop(actor) = self.bettermessage
	verDoTakeOff(actor, io) = self.jugglemessage
	verDoPutIn(actor, io) = self.bettermessage
	verDoPutOn(actor, io) = self.bettermessage
	verDoMove(actor) = self.bettermessage
	verDoThrowAt(actor, iobj) = self.bettermessage
;

/*
 *   The hallway to the north of the starting room 
 */
shop: room
	sdesc = "Charlie's Shop"
	ldesc = {
		"It's not the selection that keeps bringing you back.  It's just the convenient location.  Plus you and Charlie have known each other for a few years now so he gives you a bit of a discount.  He can usually be counted on to have what you need.  The store itself is fairly small, just a single room. ";
		if (charlie.location = self)
			"Charlie leans against a glass countertop that doubles as a display case, flipping through one of the Arcadia newspapers with a bored expression on his face. ";
		else
			"The only exit, to the east, is closed. ";
	}
	enterRoom(actor) = {
		inherited.enterRoom(actor);
		"<<global.dispParagraph>>Charlie nods in your direction.  \"Welcome back.\" ";
	}
	east = exitfromshop
	out = exitfromshop
	listendesc = "Just the occasional sound of Charlie flipping through the paper. "
	exitlist = { "The only way out is to the east. "; return nil; }
	listContents = {
		"\^%you% see%s% "; listcont(self); " lying on the floor. ";
	}
;

exitfromshop: doorway, lockableDoorway, keyedLockable
	location = shop
	isopen = true
	islocked = nil
	mykey = leader

	/* define the words the player can use to refer to the door */
	noun = 'east' 'e' 'door' 'exit'

	/* the short description */
	sdesc = "door"
	ldesc = "This is the eastern door that leads out to the street. "
	doordest = startroom
	destination = {
		if (!self.isopen) {
			"The door is closed and, more importantly, locked. ";
			return nil;
		} else
			return doordest;
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
	verDoClose(actor) = {
		"Charlie stops you with a shout.  \"Hey!  When the store is open, the door stays open.  Okay?\" ";
	}
;

// This is the stockroom item for Charlie.  It can also be used as cover
// from the grenade (it's a hideitem so you can hide behind it).
displaycase: keyedLockable, transparentItem, fixeditem, hideitem, knownTopic
	noun = 'case' 'countertop'
	adjective = 'glass' 'display'
	location = shop
	isopen = nil
	isopenable = true
	islocked = true
	contentsVisible = true
	protects = true
	shot = 0
//	isListed = nil
	sdesc = "display case"
	ldesc =  {
		"A wide glass display case about eight feet long, two and a half feet tall, and two feet deep.  It is currently";
			if (self.isopen) { " open"; }
		else { " closed"; }
		if (itemcnt(self.contents) <> 0) {
			".  Inside %you% see%s% "; listcont(self); ". ";
		} else { " and empty. "; }
	}
	cantReach(actor) = {
		"If you want anything in the case, you'll have to buy it from Charlie first. ";
	}
	verGrab(obj) = {
		if (self.isopenable and not self.isopen)
			"If you want anything in the case, you'll have to buy it from Charlie first. ";
	}
	verIoPutOn(actor) = {
		if (charlie.location = self.location)
			"\"Hey, hey, hey,\" Charlie scolds you.  \"Keep your junk offa there.\" ";
		else
			"There doesn't seem to be much point. ";
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if ((weapon = pistol) || (weapon = rifle)) {
			"The bullet careens off of the display case and imbeds itself into the wall. ";
			if (displaycase.shot = 0)
				"Apparently it's not glass after all. ";
			if (charlie.location = self.location) {
				switch (displaycase.shot) {
					case 0:
						"  \"SHIT!\" Charlie screams at you.  \"Be careful with that thing!  It's not a damn toy!\"  He glares at you. ";
						break;
					case 1:
						"  \"Dammit, I said be careful!\" Charlie yells.  \"Put that thing away!\" ";
						break;
					default:
						"Growling, Charlie grabs you and physically hurls you through the open door.  When you roughly hit the ground, you can hear the door slam closed.\b";
						parserGetMe().travelTo(startroom);
						shopdoor.setIsopen(nil);
						shopdoor.setIslocked(true);
				}
				displaycase.shot++;
			}
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = wrench) || (weapon = parserGetMe())) {
			if (charlie.location = self.location)
				"A glare from Charlie changes your mind. ";
			else
				"You give the case a hard whack.  Not even a scratch. ";
		}
	}
	doHidebehind(actor) = {
		if (charlie.location = self.location) {
			"Charlie glares at you.  \"Hey, customers stay on <I>that</I> side of the counter, remember?\" ";
			return;
		}
		pass doHidebehind;
	}

;

charlie: merchant, Actor, knownTopic
	sdesc = "Charlie"
	thedesc = "Charlie"
	noun = 'charlie' 'merchant' 'shopkeeper' 'himself' 'coveralls' 'shirt'
	adjective = 'pair' 'pair of' 'grimy' 'yellow'
	location = shop
	isHim = true
	stockroom = displaycase
	isListed = nil
	actorDesc = nil
	showActorDesc = nil
	discount = 100
	greeted = 0
	ldesc = "Charlie's dressed in drab, simple clothing.  A pair of coveralls and a grimy yellow shirt.  There's a touch of gray in his short brown hair.  He's currently reading up on some local events. "
	askTopics(topic, words) = {
		return askCharlie(self, topic);
	}
	tellTopics(topic, words) = {
		switch (topic) {
			case scrapofpaper:
				"No... no, you decide that you can't trust even Charlie with this information. ";
				break;
			case rifle: case pistol: case newspaper: case radpills: case grenade: case gps:
				"There's nothing you call tell Charlie about <<topic.thedesc>> that he doesn't already know. ";
				break;
			case metopic:
				"He's known you for years. ";
				break;
			default:
				return nil;
		}
		return true;
	}
	disavow = "Charlie shrugs and continues to read his paper. "
	telldisavow = "It doesn't look as though <<self.thedesc>> is at all interested by the information. "
	ioGiveTo(actor, dobj) = {
		if (dobj = cash) {
			"If you want something from Charlie, you can just buy it from him. ";
		}
		else
			pass ioGiveTo;
	}
	verDoTake(actor) = "You find yourself overestimating your physique. "
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	doAttack(weapon) = {
		if ((weapon = pistol) || (weapon = rifle)) {
			inherited.doAttack(weapon);
			"Charlie's look of surprise is quickly replaced by a grimace of pain as the bullet tears into him.  He falls back behind the case, and you hear a distinct 'click!' as he disappears from view.  Suddenly a wall of iron bars falls across the doorway, accompanied by a deafening alarm.  You have no choice but to wait for the Regulators to arrive.  If you're lucky, you'll live long enough to be executed. ";
			lose(nil);
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"Charlie bitchslaps you to the floor.  \"Quit fooling around,\" he mutters. ";
		}
	}
	announce(actor, obj, val) = {
		if (obj = grenade) {
			switch (val) {
				case 1:
					"Charlie's eyes widen.  \"Holy SHIT!  What the hell are you doing?  Put the pin back in!\" ";
					break;
				case 2:
					"Charlie wipes his brow.  \"Damn, don't do that again.\" ";
					break;
				case 3:
					"\"Hit the deck!\" Charlie shouts, and leaps through the doorway.  The door slams closed behind him. ";
					exitfromshop.isopen := nil;
					exitfromshop.islocked := true;
					self.moveInto(nil);
					break;
				case 4:
					"A moment later, the grenade bounces back into the <<findRootRoom(parserGetMe()).roomtype>> and lays to rest on the ground. ";
					obj.moveInto(findRootRoom(parserGetMe()));
					break;
			}
		}
	}
	verifyPurchase(actor, dobj) = {
		if (dobj = gps)
			return true;
		if ((gps.location != displaycase) || (gps.cost = nil))
			return true;
		if ((actor.money - (dobj.cost * self.discount / 100)) < gps.cost) {
			"But then you wouldn't have enough money for the GPS unit, which you'll need if you want to find where you're going. ";
			return nil;
		}
		return true;
	}
	verDoBuyFrom(actor, iobj) = {
		"Charlie may be cheap, but he isn't for sale. ";
	}
	verDoReplyto(actor) = {
		"It was a rhetorical question. ";
	}
	touchdesc = "One glare and you change your mind. "
	verDoTellAbout(actor, io) = {
	}
	actorAction(v, d, p, i) = {
		if (v = helloVerb)
			if (self.greeted = 0) {
				"Charlie replies, \"Guess you're stocking up on another trip out?  Well, take a look around.  Ain't got much this week, but still, the stuff I got is good.\" ";
				self.greeted := 1;
			} else
				"Charlie nods. ";
		else if (v = swearVerb)
			"The shopkeeper snickers.  \"Yeah, fuck you.\" ";
		else if (v = yesVerb)
			"Charlie says, \"You'll probably need that positive attitude out in the wastes.\" ";
		else if (v = noVerb)
			"Charlie says, \"That negative attitude won't help you much out there.\" ";
		else if (v = noVerb || v = yesVerb)
			"Charlie mutters, \"Actions speak louder than words.\" ";
		else if (isclass(v, passwordverb))
			"\"Huh?\" Charlie says, confused. ";
		else
			"Charlie mutters, \"Buy something or don't.  Just don't give me orders, okay?\" ";
		exit;
	}
	smelldesc = "Charlie looks at you oddly. "
	refuseToBuy = "Charlie shakes his head.  \"I'm not buying any new merchandise until the end of the month.  Sorry.\" "
	doBargainwith(actor) = {
		askCharlie(charlie, discount1);
	}
	doHug(actor) = {
		"\"I don't like you like that,\" he growls, pushing you away. ";
	}
	doTalkto(actor) = {
		"You and Charlie shoot the breeze for a bit, ";
		switch (rand(3)) {
			case 1:
				"discussing the rising number of kidnappings in the area. ";
				break;
			case 2:
				"talking about the recent infiltration of the CyTek corporate compound. ";
				break;
			case 3:
				"debating the likely effects of the Arcadia Security Commissioner's recent assassination. ";
				break;
		}
	}
	doConcentrateon(actor) = {
		"Charlie scowls.  \"Quit staring at me like that.  You're creeping me out.\" ";
	}
;

newspaper: fixeditem, readable, knownTopic
	sdesc = "newspaper"
	noun = 'newspaper' 'paper' 'news'
	adjective = 'local' 'Arcadia'
	location = shop
	ldesc = "One of the city rags.  They're pretty much all the same, only difference is which corp prints it.  Charlie's arm is covering some of it, but most of it is visible if you want to read it. "
	readdesc = "You glance down at the paper Charlie's reading.  \"The Daily Events - Mar. 12, 2160\" it says at the top.  This is the Vortex Incorporated daily paper.  \"\(Raiders Strike City Fringe\)\" reads the main headline.  Scanning down the page, you look at some of the other headlines:  \"\(Violent Criminal Ring Broken Up By Arcadian Security\)\"  \"\(Unknown Assailants Infiltrate CyTek Compound\)\"  \"\(Cowardly Assassin Kills Security Commissioner\)\"  The rest of the print is too small and too far away to read upside down. "
	verDoTake(actor) = "Charlie growls, \"Hey, I'm reading that!  Get your own.\" "
;

// This can only be bought if the player used the 'cheat' command.  It's one
// special effect is that, if full, it can be used to kill all the raiders
// in the barracks.
rifle: gun, iron, knownTopic
	sdesc = "rifle"
	noun = 'rifle' 'ruger' 'mini-14' 'mini14' '.223' 'gun'
	adjective = 'semiautomatic'
	location = displaycase
	cost = 1000
	owner = charlie
	rounds = 20
	bulk = 9
	weight = 12
	ldesc = {
		"A Ruger Mini-14 semiautomatic rifle.  A pretty handy little carbine, or so you've heard.  Your last rifle, lost on your last trek around the Urktel mountains, wasn't nearly this nice. ";
		if (cost != nil)
			"  This rifle has a price tag on it for $<<self.cost>>. ";
		else
			"It is loaded with <<self.rounds>> rounds. ";
		if (self.isbroken) {
			"You can see that the barrel is bulging outwards just past the receiver, with a blackened fissures along its length. ";
		}
	}
	verDoShootWith(actor, io) = {
		if (io = self) {
			"That'd be a neat trick. ";
			return;
		}
		pass verDoShootWith;
	}
	ioAttackWith(actor, dobj) = {
		if (self.location != actor) {
			"You can't shoot with <<self.thedesc>> unless you're holding it. ";
		} else if (rounds < 1) {
			"Your rifle is out of ammo! ";
		} else if (self.isbroken) {
			"You pull the trigger but nothing happens. ";
		} else {
			if (clay.location = self) {
				"The rifle explodes in your hands!  Although you are not injured, you can see that the barrel is bulging outwards just past the receiver, with a blackened fissures along its length. ";
				self.isbroken := true;
				return;
			}
			dobj.doAttack(self);
			rounds--;
		}
	}
	verIoPutIn(actor) = {}
	ioPutIn(actor, dobj) = {
		if (dobj != clay) {
			"You can't figure out how to get <<dobj.thedesc>> into <<self.thedesc>>. ";
			return;
		}
		"You cram as much of the clay as you can into the barrel of the rifle. ";
		clay.moveInto(self);
	}
	smelldesc = "It has the faint scent of gun oil. "
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

// Can buy from Charlie
pistol: gun, iron, knownTopic
	sdesc = "silenced pistol"
	noun = 'pistol' 'handgun' 'SIG' 'P239' '9mm' 'gun'
	adjective = 'semiautomatic' 'silenced'
	location = displaycase
	cost = 300
	rounds = 12
	bulk = 5
	weight = 5
	owner = charlie
	ldesc = {
		"From the markings on it, it's a SIG P239 semiautomatic handgun in 9mm.  It has a dark matte finish to it and a short cylinder attached to the barrel that you recognize as a silencer. ";
		if (cost != nil) {
			"Price is listed as $<<self.cost>>. ";
		} else {
			"It is loaded with <<self.rounds>> round";
			if (self.rounds>1)
				"s";
			". ";
		}
		if (self.isbroken) {
			"You can see that the chamber and slide are both bulging outwards, with a blackened fissure along the length. ";
		}
	}
	verDoShootWith(actor, io) = {
		if (io = self) {
			"That'd be a neat trick. ";
			return;
		}
		pass verDoShootWith;
	}
	ioAttackWith(actor, dobj) = {
		if (self.location != actor) {
			"You can't shoot with <<self.thedesc>> unless you're holding it. ";
		} else if (rounds < 1) {
			"Your pistol is out of ammo! ";
		} else if (self.isbroken) {
			"You pull the trigger but nothing happens. ";
		} else {
			if (clay.location = self) {
				"The pistol explodes in your hands!  Although you are not injured, you can see that the chamber and slide are both bulging outwards, with a blackened fissure along the length. ";
				self.isbroken := true;
				return;
			}
			dobj.doAttack(self);
			rounds--;
		}
	}
	verIoPutIn(actor) = {}
	ioPutIn(actor, dobj) = {
		if (dobj != clay) {
			"You can't figure out how to get <<dobj.thedesc>> into <<self.thedesc>>. ";
			return;
		}
		"You cram as much of the clay as you can into the silencer. ";
		clay.moveInto(self);
	}
	smelldesc = "It has the faint scent of gun oil. "
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

// Can buy from Charlie
radpills: fooditem, knownTopic
	sdesc = "radiation pills"
	noun = 'pill' 'pills'
	adjective = 'radiation' 'anti-radiation' 'rad' 'packet'
	location = displaycase
	bulk = 4
	weight = 3
	number = 12
	owner = charlie
	adesc = {
		"some "; self.sdesc;   // default is "a <name>"; "self" is current object
	}
	isThem = true
	cost = 200
	ldesc = {
		"Anti-radiation pills.  A vital necessity for any scavenger who's feeling bold enough to explore radiated areas.  This packet of pills contains several doses, each of which will protect the user from lethal radiation for about 20 minutes.  However, you know that the pills work as a preventative measure: not a cure.  ";
		if (self.number > 1)
			"There are <<self.number>> pills left. ";
		else if (self.number = 1)
			"There is <<self.number>> pill left. ";
		else
			"The box is empty. ";
		if (cost != nil)
			"  The box has a price tag of $<<self.cost>>. ";
	}
	verDoEat(actor) = {
		if (number = 0)
		"There aren't any left. ";
		self.verifyRemove(actor);
	}
	doEat(actor) = {
		"You swallow one of the radiation pills.  You immediately feel a tingly sensation spread throughout your body. ";
		self.number--;
		global.pilltime += 21;
	}
	verDoUse(actor) = { self.verDoEat(actor); }
	doUse(actor) = { self.doEat(actor); }
	smelldesc = "The radiation pills are odorless. "
	verDoPourOn(actor, io) = "It's easiest to keep the pills inside their box. "
	verDoPourIn(actor, io) = "It's easiest to keep the pills inside their box. "
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

// Can buy from Charlie.  This is required to get to chapter 2.
gps: item, readable, knownTopic
	sdesc = "GPS unit"
	noun = 'GPS' 'system' 'unit'
	adjective = 'global' 'positioning' 'GPS'
	location = displaycase
	cost = 500
	bulk = 5
	weight = 5
	owner = charlie
	ldesc = {
		"This is a GPS unit: a hand-held device that uses the global positioning system from before the war.  It's accurate to within several yards.  Very useful to scavengers such as yourself.  You lost your last one in a card game.  Oops. ";
		if (cost != nil)
			"  Cost: $<<self.cost>>. ";
		if (global.chapter = 2) {
			if (findRootRoom(parserGetMe()) = clearing)
				"The readout indicates that you're at the right coordinates. ";
			else
				"According to this, the clearing you arrived at matched the coordinates from the scrap of paper. ";
		}
	}
	doPostBuy(actor) = {
		incscore(1);
		global.scBuyGPS := true;
	}
	verDoUse(actor) = {
		self.verDoTurnon(actor);
	}
	verDoTurnon(actor) = {
		switch (global.chapter) {
			case 1:
				"You won't need to use it until you begin your journey. ";
				break;
			case 2:
				if (findRootRoom(parserGetMe()) = clearing)
					"The readout indicates that you're at the right coordinates. ";
				else
					"According to this, the clearing you arrived at matched the coordinates from the scrap of paper. ";
				break;
			case 3:
				"You won't be needing it again any time soon. ";
				break;
		}
	}
	verDoSwitch(actor) = {
		self.verDoTurnon(actor);
	}
	readdesc = {
		self.verDoTurnon(parserGetMe());
	}
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

// Can buy from Charlie.  It becomes armed when the pin is pulled and the 
// grenade is then dropped or thrown.  Explodes the next turn.  The detonate
// method is called by a fuse.  Can be eaten.
grenade: qcontainer, iron, knownTopic
	sdesc = "hand grenade"
	noun = 'grenade' 'pineapple' 'M81' 'spoon' 'lever' 'safety lever'
	adjective = 'frag' 'fragmentation' 'hand'
	location = displaycase
	isopen = true
	isopenable = nil
	armed = nil
	exploded = nil
	cost = 220
	bulk = 5
	weight = 4
	owner = charlie
	ldesc = {
		"An M81 fragmentation grenade, according to the white lettering on the side.  Must pack quite a punch. ";
		if (pin.location = self)
			"It has a pin at the top holding tight the \"spoon\", or safety lever. ";
		"When the pin is pulled and the grenade dropped or thrown, the safety lever separates from the grenade and its fuse is armed. ";
		if (pin.location != self)
			"The pin on this grenade has been pulled. ";
		if (self.armed)
			"The safety lever is missing.  It's only a moment until it explodes. ";
		if (cost != nil)
			"  A price tag indicates $<<self.cost>>. ";
	}
	doThrowAt(actor, io) = {
		"\^<<self.thedesc>> lands on the ground near <<io.thedesc>>. ";
		self.moveInto(actor.location);
	}
	verDoPutIn(actor, io) = {
		if (io = grenadelauncher) {
			"As far as you can tell, the grenade launcher fires a specialized type of self-launching grenade.  Your hand grenade won't fit. ";
		}
		if ((io = pack) && (pin.location != self)) {
			switch (global.chapter) {
				case 1: "If you blow up your pack, you'll never be able to make the journey. "; break;
				case 2: "If you blow up your pack, you'll never survive the trip home. "; break;
				case 3: "Blowing up your pack seems like a fruitless exercise. "; break;
			}
		}
	}
	verIoPutIn(actor) = {
		local dobject := parserGetObj(PO_DOBJ);
		if (dobject != pin) {
			"%You% can't put anything into "; self.thedesc; ". ";
		} else {
			if (self.armed)
				"Uh, you didn't know you can't put the pin back in the grenade once it's armed?  You may be in trouble. ";
			}
	}
	verIoAttackWith(actor) = "Beating someone with a grenade would be an impressive feat, but probably unlikely. "
	detonate = {
		local direction, detloc, playerloc, dead, actorlist, act, i, msg;
		exploded := true;
		msg := nil;
		playerloc := findRootRoom(parserGetMe());
		detloc := findRootRoom(self);
		if (detloc = grenade)
			return;
		if (playerloc = detloc) {
			if ((parserGetMe().location != self.location) && !isinside(self, parserGetMe())) {
				if (parserGetMe().location.protects) {
					"<<global.dispParagraph>>\^<<self.thedesc>> explodes, blasting everything in the area with hundreds of tungsten ball bearings.  Fortunately, <<parserGetMe().location.thedesc>> shields you from the shrapnel. ";
				} else {
					"<<global.dispParagraph>>\^<<self.thedesc>> explodes, blasting everything in the area with hundreds of tungsten ball bearings.  Unfortunately, <<parserGetMe().location.thedesc>> doesn't provide a bit of protection. ";
					lose(true);
				}
			} else {
				"<<global.dispParagraph>>\^<<self.thedesc>> explodes, blasting everything in the area with hundreds of tungsten ball bearings.  Unfortunately, that includes you. ";
				lose(true);
			}
			if (self.location = shop) {
				"Before you can congratulate yourself for your quick thinking, however, the ceiling of the run-down shop creaks and collapses down upon you. ";
				lose(true);
			}
		} else {
			direction := findDirection(detloc, playerloc);
			if (direction != nil) {
				msg := true;
				"<<global.dispParagraph>>You hear an explosion from <<direction>>. ";
				if (self.location = crumblingbuilding) {
					"The large wooden building groans in protest.  Then, as if in slow motion, it begins to collapse in on itself.  With an eruption of dust and smoke, the structure implodes, leaving only a twisted pile of timbers behind. ";
					if ((parserGetMe().location = basement) ||(parserGetMe().location.location = basement)) {
						"<<global.dispParagraph>>Unfortunately, the roof above isn't quite strong enough to hold back the avalanche of tumbling rubble.  You are crushed beneath the splintered flood. ";
						lose(true);
					}
				}
				if (self.location = shack) {
					"Pieces of wood are hurled from the shack by the force of the grenade.  The decrepit structure comes crashing down, now nothing more than a mass of splinters. ";
					shackdoor1.moveInto(nil);
				}
			}
		}
		// Kill local actors
		actorlist := [coyotes iona guard leader];
		for (i:=1; i<=length(actorlist); i++) {
			act := actorlist[i];
			if (isinside(act,self.location)) {
				act.doAttack(self);
			}
		}
		scorchmark.moveInto(detloc);
		self.location.grenadeblast := true;
		self.location.handleExplosion(parserGetMe(), playerloc, msg);
		self.moveInto(nil);
	}
	moveInto(obj) = {
		if ((pin.location != self) && (obj != parserGetMe()) && !self.armed && !self.exploded) {
			self.arm;
		}
		pass moveInto;
	}
	arm = {
		self.armed := true;
		"The grenade arms as its safety lever goes flying. ";
		notify(self, &detonate, 3);
		informActors(parserGetMe(), grenade, 3);
	}
	verDoEat(actor) = {}
	doEat(actor) = {
		"Yum.  Tastes pineappley. ";
		self.exploded := true;
		self.moveInto(nil);
		if ((findRootRoom(actor) = shop) && exitfromshop.islocked)
			"<<global.dispParagraph>>You get the distinct impression that you may be stuck here now.  Oops. ";
	}
	doBurn(actor) = "There are easier ways to kill yourself. "
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

// When the grenade explodes, this pseudo-actor moves into the room to 
// represent the effects of the explosion.
scorchmark: fixeditem
	sdesc = "crater"
	noun = 'explosion' 'crater' 'marks'
	adjective = 'small' 'blackened' 'scorch'
	isactor = true
	showActorDesc = true
	ldesc = {
		"<<location.scorchdesc>> ";
	}
	actorDesc = {
		"<<location.scorchdesc>> ";
	}
	smelldesc = "It has a very strong and acrid scent. "
	verDoTake(actor) = "That doesn't seem possible. "
	verDoLookbehind(actor) = "You can't look behind that. "
;

pin: item, iron
	sdesc = "grenade pin"
	noun = 'pin'
	adjective = 'grenade'
	location = grenade
	bulk = 1
	weight = 0
	takeMessage= {
		if (self.location = grenade) {
			if (grenade.location = pack) {
				"Pulling the grenade out of the pack, you yank out the pin. ";
				grenade.moveInto(parserGetMe());
			} else
				"You yank the pin out of the grenade. ";
			if (grenade.location != parserGetObj(PO_ACTOR)) {
				grenade.arm;
				informActors(parserGetMe(), grenade, 3);
			} else {
				informActors(parserGetMe(), grenade, 1);
			}
		} else {
			pass takeMessage;
		}
	}
	ldesc = {
		"A small, thin metal pin, with a loop at one end. ";
	}
	doPutIn(actor, io) = {
		self.moveInto(io);
		"You put the pin back into the grenade, making it again safe to release. ";
		informActors(parserGetMe(), grenade, 2);
	}
	doSynonym('Take') = 'Pull'
;

// This is just scenery, it can't be bought or used.
laser: gun, knownTopic
	sdesc = "laser gun"
	noun = 'laser' 'gun'
	adjective = 'laser' 'pistol'
	location = displaycase
	ldesc = "The laser gun is a bulky-looking pistol with a plastic housing of green and dark gray camouflage.  On top of the cylindrical barrel is a thin holographic sight.  The side of the gun, next to a stylized \(V\), bears the text: \"Vortex Model 17\".  There's a small tag attached to the grip with the word \"SOLD!\" written on it. "
	cost = 'Sold'
	readdesc = self.ldesc
	verDoBuyFrom(actor, iobj) = {
		"\"Sorry,\" Charlie says.  \"Someone already bought it.\" ";
	}
	doPostSell(actor) = {
		delword(self, &adjective, 'tag');
	}
;

tags: fixeditem, knownTopic
	sdesc = "price tags"
	noun = 'tag' 'tags'
	adjective = 'price'
	location = displaycase
	ldesc = {
		local list, i, tot;
		"You scan the price tags inside the display case...\n";
		list := displaycase.contents;
		tot := length(list);
		for (i:=1; i<= tot; i++) {
			if (list[i] != self)
				"\t\^<<list[i].sdesc>>: ";
				if (datatype(list[i].cost) = 1)
					"$";
				"<<list[i].cost>>\n";
		}

	}
	readdesc = self.ldesc
;

southgate: room
	sdesc = "South Gate"
	ldesc = "The high cement wall of Arcadia stretches as far as you can see to the east and west.  Directly south of you, at the end of the street, lies the massive gate that leads out of the city and into the wasteland.  This entrance stays guarded at all times; the familiar shapes of armor-clad Enforcers are visible in the lookout posts above and the squat bunkers on either side the street.  Howe Street draws a dark path in the opposite direction, further into the city. "
	north = startroom
	south = roadsouth
	east = { "Several large and well-armed men block your entrance into the bunker. "; return nil; }
	west = { "Several large and well-armed men block your entrance into the bunker. "; return nil; }
	bOutdoor = true
	listendesc = "The sounds of the city are fairly muted this far out on the fringes.  Occasionally the shifting armor of an Enforcer produces a rasping noise, barely audible from the street. "
	smelldesc = "You think you catch a faint whiff of gunpowder. "
	listContents = {
		"You can see "; listcont(self); " lying on the street. ";
	}
	exitlist = "The road heads north, further into the city, and south, through the gate. "
;

// This door requires 3 items to pass: flashlight, pack, and geiger counter
// because the PC needs them to survive.
roadsouth: doorway
	location = southgate
	isopen = true
	require = [flashlight pack geiger]

	/* define the words the player can use to refer to the door */
	noun = 'south' 's'

	/* the short description */
	sdesc = "south road"
	ldesc = "The road south passes through a heavily guarded gate in the city wall.  Once you pass through there, you're on your own. "
	doordest = clearing
	destination = {
		local actor := parserGetMe();
		local i;
		if (global.chapter > 1) {
			"There's nothing else out there for you right now. ";
			return nil;
		}
		if (!isinside(gps,actor)) {
			"You're never going to find the location without a GPS unit to guide you to the coordinates on the scrap of paper.  Too bad you lost your last one. ";
			return nil;
		}
		for (i:=1; i<=length(require); i++) {
			if (!isinside(require[i], actor)) {
				"You're going to regret it if you head off without your <<require[i].sdesc>>. ";
				return nil;
			}
		}
		"<<global.dispParagraph>>Shifting your pack, you approach the massive gate.  As the Enforcers swing open the door, you take your last breath of city air for the foreseeable future and head out on your way.<<global.dispParagraph>>The journey takes you several days east of Arcadia, across the blasted plains of the wasteland.  It likely would have been shorter except for the wide detours around radioactive hot-spots exposed by your Geiger counter.  But finally, with the help of your global positioning system, you arrive at the coordinates specified on the scrap of paper.  It's about midnight when you arrive at...\b\b\b";
		morePrompt();
		clearscreen();
		global.aboutbox;
		flashlight.isActive := true;
		flashlight.islit := true;
		geiger.isActive := true;
		flashlight.moveInto(parserGetMe());
		global.time := 'Night';
		global.chapter := 2;
		shopdoor.setIsopen(nil);
		shopdoor.setIslocked(true);
		scoreStatus(global.score, global.turnsofar);
		cityHint.active := nil;
		return doordest;
	}
;

enforcers: Actor, knownTopic
	sdesc = "Enforcers"
	noun = 'enforcer' 'enforcers' 'police' 'cop' 'cops' 'force' 'armor' 'men' 'man' 'guard' 'guards'
	adjective = 'security' 'members' 'of' 'arcadia' 'security' 'armor-clad' 'armored' 'well-armed'
	ldesc = "There are almost a dozen of the well-armed, well-armored members of the Arcadia Security Force.  Four are located atop the walls, inside of the elevated outposts.  These Enforcers, you've been told, are armed with powerful long-range rifles, either conventional or the new laser versions.  The rest of the men are manning the two squat bunkers that flank Howe Street as it approaches the city's south gate.  This group is more heavily armed, with autolasers, pulse rifles, automatic shotguns, and tripod-mounted machine guns.  The Enforcers here are tasked with keeping watch on the areas both inside and outside of the south gate. "
	location = southgate
	isListed = nil
	actorDesc = nil
	thedesc = "The Enforcers"
	adesc = "an Enforcer"
	isThem = true
	isHim = true
	count = 1
	showActorDesc = nil
	verDoTake(actor) = "You find yourself overestimating your physique. "
	verDoTellAbout(actor, iobj) = {
	}
	askWord(word, lst) = {
		if (word = 'gate') {
			"One of the Enforcers leans towards you and grumbles, \"We're here to guard this entrance and open the gate for travelers.  That's it.\" ";
			return true;
		} else {
			return nil;
		}
	}
	doTellAbout(actor, iobj) = {
		self.disavow;
	}
	doTalkto(actor) = {
		self.disavow;
	}
	// They're grumpy folk
	disavow = {
		switch (count) {
			case 1:
				"You start to speak and an Enforcer cuts you off.  \"We're on duty.  Move along,\" he says brusquely.  The soldier's expression is hidden behind his helmet's tinted plasteel face mask. ";
				break;
			case 2: case 3: case 4:
				"\"I told you to move along,\" snarls an Enforcer before you even finish your sentence. ";
				break;
			case 5:
				"\"Hey, asshole, are you hard of hearing?  Get the fuck out of here.\" ";
				break;
			case 6:
				"\"Okay, joker, you asked for it.\"  Several large and irritated men storm out of the bunker and savagely wrestle you to the ground.  Battered and bruised, you are quickly processed, tried, and sentenced for \"disturbing the peace.\"  A year in prison might teach you some wisdom.<<global.dispParagraph>>If you survive. ";
				lose(nil);
				break;
		}
		count++;
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if (!io.isGun)
			"That wouldn't be advisable. ";
	}
	doAttack(weapon) = {
		if (weapon != grenade)
			inherited.doAttack(weapon);
		self.killplayer;
	}
	killplayer = {
		"Those bright flashes coming from the openings in the bunkers can only be muzzle flashes.  The thunderous weapon reports seem oddly distant, though, as you crumple to the ground. ";
		lose(true);
	}
	announce(actor, obj, val) = {
		if (obj = grenade) {
			switch (val) {
				case 1:
					"<<global.dispParagraph>>";
					self.killplayer;
					break;
				case 3:
					"<<global.dispParagraph>>";
					self.killplayer;
					break;
			}
		}
	}
	verDoRape(actor) = {
		"Not advisable. ";
	}
	verDoScrew(actor) = { self.verDoRape(actor); }
	doScrew(actor) = { self.doRape(actor); }
	actorAction(v, d, p, i) = {
		if ((v = openVerb) && (d = scenerygate)) {
			parserGetMe().travelTo(roadsouth);
		} else if (v = swearVerb) {
			"\"Okay, joker, you asked for it.\"  Several large and irritated men storm out of the bunker and savagely wrestle you to the ground.  Battered and bruised, you are quickly processed, tried, and sentenced for \"disturbing the peace.\"  A year in prison might teach you some wisdom.<<global.dispParagraph>>If you survive. ";
			lose(nil);
		} else {
			self.disavow;
		}
		exit;
	}
	smelldesc = "Gunpowder. "
	verGrab(item) = {
		"Enforcers aren't known for their intelligence.  They are, however, quite famous for their irritability and brutality. ";
	}
	doHug(actor) = {
		"Enforcers aren't known for their intelligence.  They are, however, quite famous for their irritability and brutality. ";
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"Violence.  Greed.  Hatred.  You'd always known the Enforcers were brutal creatures.  Until now, you had no idea just how right you were. ";
		else
			pass doConcentrateon;
	}
;

// First room in chapter 2.  Heading west returns to Arcadia.
clearing: darkroom
	sdesc = "Clearing"
	ldesc = {
		if (!self.isseen) {
			"Flashlight held out as a ward against the darkness, you step through a break in the scrub and take a look around.  Browned vegetation encircles this patch of flat dusty ground.  Scraggly brush and thickets wave slowly in the glow of your flashlight.  The rustling of branches is the only sound you hear; the area seems eerily devoid of any other noise, even crickets.  Though you can make your way east and south, thorny brambles impede your movement in most other directions.  Behind you, the break in the scrub leads west back towards Arcadia. ";
		} else {
			"Browned vegetation encircles this patch of flat dusty ground.  Scraggly brush and thickets wave slowly in the glow of your flashlight.  The rustling of branches is the only sound you hear; the area seems eerily devoid of any other noise, even crickets.  Though you can make your way east and south, thorny brambles impede your movement in most other directions.  An opening to the west leads back towards Arcadia";
			if (iona.foundpath) {
				", and a small gap leads southwest through the thickets";
			}
			". ";
		}
	}
	bOutdoor = true
	west = arcadiapath
	south = fadedpath
	east = edgeofvillage
	sw = brambles
	north = { return self.badexit('The brambles block your way. '); }
	listendesc = {
		switch (guard.mark) {
			case 2: 
				"Over the chatter of whispering branches, you can barely make out the sound of footsteps to the south. "; break;
			case 3:
			case 5:
				"The faint noise of footsteps can be heard to the south. "; break;
			default:
				"Strain as you may, you can't hear anything beyond the sound of rustling vegetation. ";
		}
	}
	exitlist = {
		if (iona.foundpath) {
			"The only breaks in vegetation lie to west, east, south, and southwest.";
		} else {
			"The only breaks in vegetation lie to west, east, and south.";
		}
		return nil;
	}
	listContents = {
		"Nestled among the grass %you% can make%s% out "; listcont(self); ". ";
	}
;

brambles: qcontainer, fixeditem, doorway
	sdesc = "brambles"
	noun = 'brambles' 'thorn' 'thicket' 'thorns' 'thickets' 'patch'
	adjective = 'thorny' 'briar'
	location = clearing
	isopenable = true
	isopen = nil
	isThem = true
	contentsReachable = { return self.isopen; }
	contentsVisible = { return self.isopen; }
	ldesc =  {
		"Thickets snarl around the southwestern and northern sections of the clearing.  The brambles are comprised of a dense twisting of thick branches, and just oozing with angry thorns. ";
		if (iona.foundpath) {
			"There is a well-hidden gap in the brambles leading to the southwest. ";
		} else {
			if (self.isopen) {
				if (rabbit.location = self)
					"Movement through them looks impossible, though you can see the remains of a rabbit lying within. ";
				else
					"Movement through them looks impossible. ";
			} else {
				"Movement through them looks impossible, though you're worried about what might be hiding in them. ";
			}
		}
	}
	doSearch(actor) = {
		if (itemcnt(self.contents) <> 0) {
			"Peering between the branches, you spot the motionless form of an expired rabbit. ";
			setit(rabbit);
			self.isopen := true;
		} else {
			"There's nothing in <<self.thedesc>>. ";
		}
	}
	verIoPutIn(actor) = {
		"%You% can't put anything into "; self.thedesc; ". ";
	}
	doordest = hiddenpath
	destination = {
		if (iona.foundpath) {
			"You manage to crawl along the trail through the brambles with only minor scratches.\b";
			return self.doordest;
		} else {
			"The brambles block your way. ";
			return nil;
		}
	}
	verDoTake(actor) = "You can't carry that. "
	verDoLookbehind(actor) = "You can't see behind the brambles. "
	smelldesc = "The wind carries a sickly smell. "
;

// For feeding coyotes
rabbit: item
	sdesc = "rabbit corpse"
	noun = 'corpse' 'bunny' 'hare' 'rabbit' 'form' 'body'
	adjective = 'wild' 'expired' 'motionless'
	location = brambles
	bulk = 8
	weight = 3
	ldesc = "The body of a wild rabbit, brown in fur, and crawling with flies.  It's not readily apparent what the cause of death was.  It looks like the briar patch may not have been the safest place for it after all. "
	doThrowAt(actor, io) = {
		if (actor.location = centerofvillage) {
			centerofvillage.roomDrop(self);
		} else {
			"\^<<self.thedesc>> lands on the ground right in front of <<io.thedesc>>. ";
			self.moveInto(actor.location);
		}
	}
	moveInto(obj) = {
		if (obj = centerofvillage) {
			coyotes.feed(nil);
		} else {
			pass moveInto;
		}
	}
	smelldesc = "Whew!  You quickly turn away. "
;

// Travel back home.  The pack, flashlight, and geiger counter are needed
// to survive the journey.  If the helmet isn't carried, the player is 
// asked if they're sure.  This is the exit for getting to chapter 3.
arcadiapath: doorway
	location = clearing
	isopen = true
	lasttried = -5
	require = [flashlight pack geiger]

	/* define the words the player can use to refer to the door */
	noun = 'west' 'w'

	/* the short description */
	sdesc = "path to Arcadia"
	ldesc = "A break in the vegetation leads back in the direction of Arcadia.  It'll be a long trip back, but you're used to long journeys. "
	doordest = southgate
	destination = {
		local girl, helm, counter, i;
		if (!location.islit) {
			darkTravel();
			return nil;
		}
		if (!global.gothelmet) {
			if (global.turnsofar - self.lasttried>5) {
				"If you really want to head back empty-handed... ";
				self.lasttried := global.turnsofar;
				return nil;
			} else {
				for (i:=1; i<=length(require); i++) {
					if (!isinside(require[i], parserGetMe())) {
						"You're going to regret it if you head off without your <<require[i].sdesc>>. ";
						return nil;
					}
				}
				if ((global.radtime > 0) || (global.sicktime > 0)) {
					"You begin the arduous journey back home, your heart heavy with the knowledge that you have failed in your quest.  As you pick your way through the blasted wastelands, you find your strength ebbing with practically every step you take. ";
					if (global.radtime > 0)
						"Delirious from radiation poisoning, you collapse to the ground and never rise again. ";
					else if (global.sicktime > 0)
						"Delirious from fever, you collapse to the ground and never rise again.  Must've been something you drank. ";
					lose(true);
				} else {
					"You begin the arduous journey back home, your heart heavy with the knowledge that you have failed in your quest.  This isn't the first time you've come home empty-handed, and you doubt it will be the last time.  Still... you were so sure this time would be different.  That it would be your triumphant discovery.  Deep down you know that this failure was your own fault.  Maybe someday, you will return. ";
					lose(nil);
				}
			}
		} else {
			for (i:=1; i<=length(require); i++) {
				if (!isinside(require[i], parserGetMe())) {
					"You're going to regret it if you head off without your <<require[i].sdesc>>. ";
					return nil;
				}
			}
			girl := (parserGetMe().myfollower = iona);
			helm := isinside(helmet, parserGetMe());
			global.broughthelm := true;
			if (!helm && !helmet.isbroken && !global.genvented) {
				"And just leave the helmet behind?  After all you've gone through to get it? ";
				return nil;
			}
			if ((global.radtime > 0) || (global.sicktime > 0)) {
				"You hurriedly begin the arduous journey back home";
				if (girl) {
					", Iona in tow. ";
				}
				"As you pick your way through the blasted wastelands, you find your strength ebbing with practically every step you take.  ";
				if (global.radtime > 0)
					"Delirious from radiation poisoning, you collapse to the ground and never rise again. ";
				else if (global.sicktime > 0)
					"Delirious from fever, you collapse to the ground and never rise again.  Must've been something you drank. ";
				lose(true);
			}
			"You hurriedly begin the arduous journey back home";
			if (girl) {
				iona.moveInto(doordest);
				", Iona in tow.  You are amazed to find that she hardly slows you down.  She obviously comes from tough stock";
			}
			".  ";
			if (helm && !helmet.isbroken) {
				"The helmet you keep tucked safely within your survival pack. ";
			}
			if (global.genvented) {
				"Moving as quickly as you can, you rush from the base in search of shelter.  When you come across a series of cliffs you waste no time in taking refuge behind them.  And not a moment too soon.  The sky erupts in dazzling light, followed quickly by thunderous shockwave.  A brilliant mushroom cloud rises up over the location of the research facility as its reactor goes critical.  ";
				if (girl) {
					"The two of you";
				} else {
					"You";
				}
				" spend several hours in the shadow, then make your way towards the awaiting city of Arcadia. ";
				incscore(1);
				global.scDestroyBase := true;

				doScoreNotify();
				if (!helm)
					helmet.isbroken := true;
			}
			"<<global.dispParagraph>>The return trip takes slightly longer than the first leg of the journey, though you soon find yourself gazing upon the Cyclopean structures of your home city.  The guards scrutinize ";
			if (girl) {
				"yourself and young Iona";
			} else {
				"you";
			}
			", but within no time you find yourself once again walking Arcadia's decaying streets. ";
			global.time := 'Dusk';
			global.chapter := 3;
			scenerysign.moveInto(startroom);
			scoreHint.active := true;
			helmet.isworn := nil;
			enforcers.count := 1;
			scoreStatus(global.score, global.turnsofar);
			morePrompt();
			clearscreen();
			global.aboutbox;
			shopdoor.isopen := nil;
			shopdoor.islocked := true;
		}
		pass destination;
	}
;

swbramble: doorway
	pathRevealed = nil
	location = clearing
	isopen = true

	/* define the words the player can use to refer to the door */
	noun = 'southwest' 'sw'

	/* the short description */
	sdesc = "southwest brambles"
	ldesc = brambles.ldesc
	doordest = hiddenpath
	destination = {
		if (!location.islit) {
			darkTravel();
			return nil;
		}
		if (pathRevealed)
			return hiddenpath;
		return nil;
	}
;

edgeofvillage: darkroom
	sdesc = "Edge of Village"
	ldesc = {
		"You're standing on the west edge of a small village made up of decaying wooden buildings.  Most are little more than one room shacks.  Every window is dark and no noise reaches your straining ears.  The crooked structures to the south don't look particularly inviting";
		if (crumblingbuilding.collapsed) {
			" and the large building to the north has collapsed into a mangled pile of timbers. ";
		} else {
			", but no less than the gaping hole in the wall of a large building to your north. ";
		}
		"The clearing where you first arrived lies to the west. ";
	}
	bOutdoor = true
	north = doortocrumblingbuilding1
	east = centerofvillage
	west = clearing
	south = { return self.badexit('You peek your head inside one of the buildings and find nothing but ghosts. '); }
	listendesc = "The area seems deathly silent. "
	exitlist = { "Besides the obvious exits to east and west, the large hole in the building to the north appears large enough to fit through. "; return nil; }
	listContents = {
		"Among the clods of dirt, %you% see%s% "; listcont(self); ". ";
	}
	enterRoom(actor) = {
		if ((iona.location = nil) && !global.scKillGirl && !iona.greeted) {
			iona.moveInto(shack);
		}
		pass enterRoom;
	}
;

doortocrumblingbuilding1: doorway
	location = edgeofvillage
	isopen = true

	/* define the words the player can use to refer to the door */

	/* the short description */
	sdesc = "crumbling building"
	ldesc = {
		if (crumblingbuilding.collapsed) {
			"The large building is gone.  In its place is a mangled pile of timbers.  You wonder if perhaps you should be a little more careful in the future. ";
		} else {
			"This is clearly the largest building in the village: possibly a lodge or meeting hall built from uneven pieces of lumber.  It's at least three times as large as any other structure you've seen.  And yet, it's also in the worst shape.  There are huge holes in the roof";
			if (findRootRoom(parserGetMe()) = edgeofvillage)
				"and the south wall";
			else
				" and you notice a distinct lean to the building's profile";
			".  If in a month this building is still standing, you'd be very surprised. ";
		}
	}
	doordest = crumblingbuilding
	destination = {
		if (!self.location.islit) {
			darkTravel();
			return nil;
		}
		if (crumblingbuilding.collapsed) {
			"There's no way you're going to be able to enter that pile of debris. ";
			return nil;
		}
		return doordest;
	}
;

// Entering this room triggers (and retriggers) the coyote(s) actor.
// Leaving will reset how close they are to the player.
centerofvillage: darkroom
	sdesc = "Center of Village"
	ldesc = {
		" The heart of this deserted village holds no more sign of human life than its border.  Your flashlight spills over irregular buildings of poor craftsmanship and even poorer materials.  A collection of shanties lies to your south, and to your east ";
		if (shack.collapsed) {
			"are the splintered remains of a wooden shack.  ";
		} else {
			"is a single shack with a";
			if (shackdoor1.isopen)
				"n open";
			else
				" closed";
			" door.  ";
		}
		"The road from the west curves north here, leading to the rest of the village.  While the town looks deserted, there is sign of recent activity here.  Tracks of some kind break up the relatively smooth surface of the road. ";
	}
	bOutdoor = true
	north = northvillage
	west = edgeofvillage
	east = shackdoor1
	south = { return self.badexit('You peek your head inside one of the buildings and find nothing but ghosts. '); }
	enterRoom(actor) =	  // sent to room as actor is entering it
	{
		inherited.enterRoom(actor);
		coyotes.closeness := 0;
		notify(coyotes, &advance, 2);
	}
	roomDrop(obj) = {
		if (obj = rabbit) {
			coyotes.feed(true);
			return;
		}
		"Dropped. ";
		obj.moveInto(self);
	}
	listendesc = {
		if (coyotes.location = self) {
			"You can't hear anything over the growling of <<coyotes.thedesc>>. ";
		} else {
			"The area seems deathly silent. ";
		}
	}
	exitlist = { "The road runs to the north and west.  A shack lies to the east. "; return nil; }
	listContents = {
		"Among the clods of dirt, %you% see%s% "; listcont(self); ". ";
	}
;

// While alive, the coyotes prevent travel east.  One actor represents both
// live coyotes.
coyotes: Actor, obstacle, knownTopic
	noun = 'coyote' 'coyotes' 'dog' 'dogs' 'doggies'
	adjective = 'emaciated' 'pair'
	num = 2
	offscreen = true
	closeness = 0
	location = centerofvillage
	contentsVisible = { return nil; }
	sdesc = {
		if (self.num = 2)
			"coyotes";
		else if (self.num = 1)
			"coyote";
		else
			"no coyote";
	}
	ldesc = {
		if (self.num = 2)
			"A pair of mangy, scraggly coyotes.  Their coats are filthy, matted together, and beneath you can clearly see the outlines of their bones.  They glare at you with a disturbing mix of hunger and hatred. ";
		else if (self.num = 1)
			"A mangy, scraggly coyote.  Its coat is filthy, matted together, and beneath you can clearly see the outlines of its bones.  It glares at you with a disturbing mix of hunger and hatred. ";
		else
			"No coyote here.  Move along please. ";
	}
	isThem = {
		if (self.num > 1)
			return true;
		return nil;
	}
	verbEnd = {
		if (self.num = 1)
			"s";
	}
	pluralEnd = {
		if (self.num = 2)
			"s";
	}
	possessive = {
		if (self.num = 2)
			"their";
		else
			"its";
	}
	adesc = {
		if (self.num > 1)
			"a pair of <<self.sdesc>>";
		else
			"a <<self.sdesc>>";
	}
	actorDesc = {
		if (offscreen) {
			coyoteHint.active := true;
			if (flashlight.islit)
				"A sound from the east causes you to spin about and aim your flashlight in that direction.  From between two of the buildings, a pair of emaciated coyotes slink into the street, their eyes glowing in the beam of your light.  They immediately turn their attention towards you. ";
			else
				"From the east you hear the sound of heavy panting and shifting dirt in the darkness.  A low growling takes its place. ";
			offscreen := nil;
		} else {
			if (flashlight.islit)
				"There is <<self.adesc>> here, eyeing you hungrily. ";
			else
				"You hear the sound of panting and low growling from nearby. ";
		}
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if (!io.isGun)
			"That wouldn't be advisable. ";
	}
	doAttack(weapon) = {
		if (weapon = pistol) {
			if (self.num = 2) {
				"The P239 coughs quietly and one of the coyotes stumbles and falls.  The other doesn't even seem to notice that its companion lies dead on the ground. ";
			} else {
				"A blossom of red appears on the feral animal.  It crashes to the ground, twitches slightly, and then is still. ";
			}
			self.num--;
			corpsecoyote.moveInto(self.location);
			corpsecoyote.num++;
			if (self.num < 1) {
				self.moveInto(nil);
				coyoteHint.active := nil;
			}
		}
		if (weapon = rifle) {
			if (self.num = 2) {
				"The Ruger booms in the still air and one the coyotes stumbles and falls.  The other snarls and springs to the side, wary, but still approaching. ";
			} else {
				"A blossom of red appears on the feral animal.  It crashes to the ground, twitches slightly, and then is still. ";
			}
			self.num--;
			corpsecoyote.moveInto(self.location);
			corpsecoyote.num++;
			if (self.num < 1) {
				self.moveInto(nil);
				coyoteHint.active := nil;
			}
		}
		if (weapon = grenade) {
			self.num:=0;
			coyoteHint.active := nil;
			corpsecoyote.moveInto(self.location);
			corpsecoyote.num:=2;
			self.moveInto(nil);
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"You lunge forward.  With a snarl, <<self.thedesc>> leap<<self.verbEnd>> into the air and slam<<self.verbEnd>> into your chest.  Your throat is quickly torn out by the ravenous beast<<self.pluralEnd>>. ";
			lose(true);
		}
	}
	advance = {
		if (self.num = 0)
			return;
		if (parserGetMe().location != self.location)
			return;
		if (self.location = nil)
			return;
		switch (self.closeness) {
			case 0:
				"<<global.dispParagraph>>You hear a deep growling coming from <<self.thedesc>> as <<self.itnomdesc>> slowly make<<self.verbEnd>> <<self.possessive>> way towards you. ";
				break;
			case 1:
				if (parserGetMe().location.islit) {
					"<<global.dispParagraph>>\^<<self.thedesc>> continue";
					self.isThem ? "" : "s";
					" to advance, and you see flakes of foam falling from <<self.possessive>> rancid mouth<<self.pluralEnd>>. ";
				} else {
					"<<global.dispParagraph>>You can hear <<self.thedesc>> continue to advance. ";
				}
				break;
			case 2:
				"<<global.dispParagraph>>\^<<self.thedesc>> ";
				self.isThem ? "are" : "is";
				" almost on top of you! ";
				break;
			case 3:
				"<<global.dispParagraph>>With a snarl, <<self.thedesc>> leap<<self.verbEnd>> into the air and slam<<self.verbEnd>> against your chest.  Your throat is quickly torn out by the ravenous beast<<self.pluralEnd>>. ";
				lose(true);
				break;
		}
		self.closeness++;
		notify(self, &advance, 1);
	}
	verIoShowTo(actor) = {}
	ioShowTo(actor, dobj) = {
		if (dobj = rabbit) {
			if (self.num = 1)
				"The coyote's eyes narrow and its nostrils flare. ";
			else
				"The coyotes' eyes narrow and their nostrils flare. ";
		} else {
			pass ioShowTo;
		}
	}
	verIoThrowAt(actor) = {}
	verIoThrowTo(actor) = {}
	ioThrowTo(actor, dobj) =
	{
		"\^<<self.thedesc>> move<<self.verbs>> with surprising quickness, darting out of the way. ";
		self.moveInto(actor.location);
	}
	ioThrowAt(actor, dobj) =
	{
		self.ioThrowTo(actor, dobj);
	}
	smelldesc = "You get a whiff of something foul. "
	touchdesc = "You'd prefer to keep your hands. "
	listendesc = {
		if (self.num = 1) {
			"The coyote is growling quite loudly. ";
		} else {
			"The coyotes are growling quite loudly. ";
		}
	}
	ioGiveTo(actor, dobj) = {
		if (dobj = rabbit) {
			self.feed(true);
			return;
		}
		"\^<<self.thedesc>> ";
		if (self.num = 1)
			"seems";
		else
			"seem";
		" a lot more interested in you.";
	}
	feed(start) = {
		if (coyotes.num = 2) {
			if (findRootRoom(parserGetMe()) = self.location) {
				if (start)
					"You toss the body of the rabbit in front of the coyotes.  As the smell hits them";
				else
					"  As the smell hits the coyotes";
				", their entire attention turns to the small animal corpse.  One of them leaps forward, snatching the rabbit in its jaws.  The second lets out a howl that is both poignantly mournful and angry.  The first coyote dashes off between the buildings with the dead bunny in its jaws.  The other animal chases after it, close on its heels. ";
			}
			coyoteHint.active := nil;
			coyotes.moveInto(nil);
			rabbit.moveInto(coyotes);
			coyotes.num := 0;
			return;
		} else if (coyotes.num = 1) {
			if (findRootRoom(parserGetMe()) = self.location) {
				if (start)
					"You toss the body of the rabbit in front of the coyote.  As the smell hits it";
				else
					"  As the smell hits the coyote";
				", its entire attention turns to the small animal corpse.  Suddenly, it darts forward, grasping the rabbit in its jaws.  With one last hateful look in your direction, it turns and dashes off between the buildings to consume its meal. ";
			}
			coyoteHint.active := nil;
			coyotes.moveInto(nil);
			rabbit.moveInto(coyotes);
			return;
		}
	}
	doHug(actor) = {
		"\^<<self.itnomdesc>> would chew through your throat in a matter of seconds. ";
	}
	announce(actor, obj, val) = {
		if (obj = rabbit) {
			self.moveInto(nil);
		}
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"The sensation is odd.  The thoughts are base and primitive and consist mainly of the desire for food. ";
		else
			pass doConcentrateon;
	}
;

// When killed, this represents the coyote corpse(s).
corpsecoyote: fixeditem
	noun = 'corpse' 'body' 'corpses' 'bodies'
	adjective = 'coyote' 'coyotes'
	location = coyotes
	isListed = true
	isfixed = true
	num = 0
	sdesc = {
		if (self.num = 2)
			"coyote corpses";
		else if (self.num = 1)
			"coyote corpse";
		else
			"no coyote corpse";
	}	
	ldesc = {
		if (self.num = 2)
			"A pair of mangy, scraggly, dead coyotes.  Their coats are filthy, matted with blood, and beneath you can clearly see the outlines of their bones.  Their tongues loll from their mouths. ";
		else if (self.num = 1)
			"A mangy, scraggly, dead coyote.  Its coat is filthy, matted with blood, and beneath you can clearly see the outlines of its bones.  Its tongue lolls from its mouth. ";
		else
			"No coyote corpse here.  Move along please. ";
	}
	isThem = {
		if (self.num > 1)
			return true;
		return nil;
	}
	adesc = {
		if (self.num > 1)
			"a pair of <<self.sdesc>>";
		else
			"a <<self.sdesc>>";
	}
	touchdesc = "The fur is matted and tangled.  Not a pleasant sensation. "
	smelldesc = "Do you often go around smelling corpses? "
	verDoTake(actor) = "\^<<self.thedesc>> won't do you any good. "
;	

shack: darkroom
	sdesc = "Battered Shack"
	noun = 'shack' 'shed'
	adjective = 'battered' 'cramped'
	ldesc = "The cramped confines of this shack leave little room for movement.  Walls of bug-chewed wood and a roof of warped planks are the only thing between you and the cloudy night sky.  The shack itself contains little of notice besides the pungent odor of ammonia.  A few amorphous lumps of wood, stone, and other less identifiable materials lie scattered about.  A pile of discarded boards leans against the wall in one corner of the room.  The door to the west leads out into the street. "
	west = shackdoor2
	collapsed = nil
	enterRoom(actor) =	  // sent to room as actor is entering it
	{
		inherited.enterRoom(actor);
		boards.noisecount := 2;
		notify(boards, &noise, 3);
	}
	listendesc = "You think you hear some rustling or scraping.  It's very faint. "
	exitlist = { "The only exit lies to the west. "; return nil; }
	listContents = {
		"Discarded on the ground you find "; listcont(self); ". ";
	}
	smelldesc = "The smell of ammonia is nearly overwhelming. "
	handleExplosion(actor, playerloc, heard) = {
		self.collapsed := true;
	}
;

shackdoor1: doorway
	sdesc = "shack door"
	noun = 'door'
	adjective = 'shack' 'wooden'
	plural = 'doors'
	location = centerofvillage
	ldesc = "A very basic wooden door, made from planks with wide gaps between them. "
	doordest = shack
	destination = {
		if ((coyotes.num < 1) && shack.collapsed) {
			"That shack is nothing but a heap of splintered wood now. ";
			return nil;
		}
		if (coyotes.location != nil) {
			"The snarling <<coyotes.sdesc>> block<<coyotes.verbs>> your way. ";
			return nil;
		}

		if (self.isopen)
			return self.doordest;
		else if (not self.islocked and not self.noAutoOpen)
		{
			self.setIsopen(true);
			"(Opening << self.thedesc >>)\n";
			return self.doordest;
		}
		else
		{
			"%You%'ll have to open << self.thedesc >> first. ";
			setit(self);
			return nil;
		}
	}
	noAutoOpen = nil
	otherside = shackdoor2
	isReachable(actor) = {
		if (coyotes.location != nil) {
			return nil;
		}
		return true;
	}
	cantReach(actor) = {
		"The snarling <<coyotes.sdesc>> block<<coyotes.verbs>> your way. ";
	}
;

shackdoor2: doorway
	sdesc = "shack door"
	noun = 'door'
	adjective = 'shack' 'wooden'
	ldesc = "A very basic wooden door, made from planks with wide gaps between them. "
	location = shack
	doordest = centerofvillage
	noAutoOpen = nil
	otherside = shackdoor1
;

odor: intangible, scenery
	sdesc = "odor"
	noun = 'odor' 'ammonia'
	adjective = 'odor' 'odor of'
	location = shack
	smelldesc = "It's a strong scent of ammonia. "
	scenerymessage = "You can't figure out how to do that to an odor. "
;

// A container that hides Iona
boards: qcontainer, fixeditem
	sdesc = "pile of boards"
	noun = 'boards' 'wood' 'pile' 'corner' 'board' 'shadow'
	adjective = 'pile of' 'discarded'
	location = shack
	isopenable = true
	isopen = nil
	contentsReachable = { return self.isopen; }
	contentsVisible = { return self.isopen; }
	noisecount = 0
	ldesc = "Pieces of lumber in various sizes and shapes.  This shack may have served as a storage shed for the village's construction materials.  The large pile is stacked in one corner of the shack, leaning against the wall.  There is a considerable pocket of deep shadow behind the boards. "
	doSearch(actor) = {
		if (iona.location = self) {
			"You angle the boards away from the wall to peer behind them.  With a sudden cry, a small figure scrambles out from behind them.  The diminutive shape dashes across the shack and huddles in the opposite corner.  You turn the beam of your flashlight toward it and see that it is an emaciated little girl, covered in grime. ";
			iona.moveInto(location);
			ionaHint.active := true;
			setit(iona, 2);
		} else {
			"\^%You% look behind <<self.thedesc>> but find nothing. ";
		}
	}
	verDoLookbehind(actor) = {
		if (iona.location != self)
			"\^%You% look behind <<self.thedesc>> but find nothing. ";
	}
	doLookbehind(actor) = {
		self.doSearch(actor);
	}
	verDoLookunder(actor) = {
		if (iona.location != self)
			"\^%You% look under <<self.thedesc>> but find nothing. ";
	}
	doLookunder(actor) = {
		self.doSearch(actor);
	}
	verDoMove(actor) = {
		self.verDoLookbehind(actor);
	}
	verDoMoveTo(actor, io) = {
		self.verDoLookbehind(actor);
	}
	doMove(actor) = {
		self.doSearch(actor);
	}
	doMoveTo(actor, io) = {
		self.doSearch(actor);
	}
	verIoPutIn(actor) = {
		"%You% can't put anything into "; self.thedesc; ". ";
	}
	noise = {
		if (noisecount > 0) {
			if ((iona.location = self) && (parserGetMe().location = location)) {
				switch (rand(4)) {
					case 1:
						"<<global.dispParagraph>>You hear a faint rustling noise coming from behind the boards.  It lasts for only a brief moment and then all is quiet again. ";
						break;
					case 2:
						"<<global.dispParagraph>>The silence is broken by what sounds like a quiet cough. ";
						break;
					case 3:
						"<<global.dispParagraph>>The boards in the corner rattle suddenly, as if they were bumped or kicked. ";
						break;
					case 4:
						"<<global.dispParagraph>>There's a quiet thud from the corner of the shack, followed by a whispered, \"Ow!\" ";
						break;
				}
				notify(self, &noise, rand(3)+1);
			}
			noisecount--;
		}
	}
	listendesc = "It almost sounds like the boards are... breathing. "
	verDoTake(actor) = "\^<<self.thedesc>> would probably give you a very painful bunch of splinters. "
	smelldesc = "The smell of ammonia saturates everything. "
;

iona: Actor, knownTopic
	noun = 'girl' 'iona' 'herself' 'her' 'figure'
	adjective = 'small' 'little' 'scrawny'
	// It used to be that the player just had to ask the girl for her name
	// to get her talking.  This was changed to giving her food, but the
	// property retains the original name.
	greeted = nil
	askedpath = nil
	foundpath = nil
	gotdoll = nil
	isHer = true
	isHim = nil
	sdesc = {
		if (self.greeted)
			"Iona";
		else
			"little girl";
	}
	thedesc = {
		if (self.greeted)
			"<<self.sdesc>>";
		else
			"the <<self.sdesc>>";
	}
	adesc = {
		if (self.greeted)
			"<<self.sdesc>>";
		else
			"a <<self.sdesc>>";
	}
	location = boards
	isListed = nil
	actorDesc = {
		iona3Hint.active := nil;
		if (self.greeted)
			"\^<<self.adesc>> is standing close beside you.  She smiles broadly. ";
		else
			"There is a little girl here, huddled in a corner of the shack. ";
	}
	ldesc = {
		"A gaunt little girl, probably about five or six years old.  She has dirty brown hair which would probably reach to her shoulders if it wasn't so tangled.  She wears a plain brown dress of a coarse (and very dirty) fabric.  Her face is also covered in grime; her eyes red and watery.  You can't help but notice her sunken cheekbones and eye sockets. ";
		if (self.gotdoll) {
			"Clutched tightly to her chest is a threadbare rag doll. ";
		}
		if (self.greeted) {
			"She smiles up at you, obviously happy to see you. ";
		} else {
			"Her body language betrays her obvious fear of you.  She is huddled in the corner, lips trembling, eyes wide and darting in all directions. ";
		}
	}
	// If the player is dressed like a raider, the poor girl gets
	// understandably upset.
	askTopics(topic, words) = {
		if (wearingUniform(parserGetMe())) {
			"\^<<self.thedesc>> takes one look at your clothing, lets out a shriek, and dashes out of the shack. ";
			iona.moveInto(nil);
			iona3Hint.active := true;
			return true;
		}
 		if (!greeted) {
			"\^<<self.thedesc>> seems too afraid of you to respond. ";
			return true;
 		}
		return askIona(self, topic);
	}
	tellTopics(topic, words) = {
		if (!self.greeted) {
			"\^<<self.thedesc>> seems too afraid of you to respond. ";
			return;
		}
		switch (topic) {
			case metopic: case arcadiacity:
				"You tell Iona a little bit about yourself and Arcadia.  Her face holds an expression of amazement and disbelief. ";
				break;
			default:
				return nil;
		}
		return true;
	}
	telldisavow = "Iona simply nods at you, eyes blank. "
	disavow = "<<self.thedesc>> simply looks up at you with wide eyes. "
	hesitatemsg = "You can't help but hesitate. "
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if (global.chapter = 3) {
			"You can't bring yourself to do it. ";
		}
	}
	doAttack(weapon) = {
		if (weapon != grenade)
			inherited.doAttack(weapon);
		if ((weapon = pistol) || (weapon = rifle)) {
			"\^<<self.thedesc>>'s eyes go wide with shock.  She stumbles around for a few steps before she collapses on the ground. ";
			ionacorpse.moveInto(self.location);
			iona.moveInto(nil);
			incscore(-5);
			global.scKillGirl := true;
		}
		if (weapon = grenade) {
			ionacorpse.moveInto(self.location);
			iona.moveInto(nil);
			incscore(-5);
			global.scKillGirl := true;
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"You lunge forward at the girl, who shrieks and darts between your legs, fleeing the shack. ";
			iona3Hint.active := true;
			self.moveInto(nil);
		}
	}
	announce(actor, obj, val) = {
		if (obj = grenade) {
			switch (val) {
				case 3:
					"\^<<self.thedesc>> stares blankly at the grenade. ";
					break;
			}
		}
	}
	ioGiveTo(actor, dobj) = {
		if (dobj = doll) {
			if (self.greeted) {
				if (!self.askedpath) {
					"Iona's face immediately brightens.  \"My dolly!\" she squeals in delight as she hugs the rag doll.  \"Thank you so much!\" ";
					self.gotdoll := true;
					dobj.moveInto(iona);
				} else {
					"Iona's face immediately brightens.  \"My dolly!\" she squeals in delight as she hugs the rag doll.  She grabs your hand and pulls you towards the door.  \"Let me show you the secret way past the bad men.\"  She leads you back to the clearing.\b";
					dobj.moveInto(iona);
					iona.moveInto(clearing);
					actor.travelTo(clearing);
					"<<global.dispParagraph>>Iona leads you over to the brambles.  \"Down here,\" she says, getting down on her hands and knees.  As you do the same, she points to a well-hidden gap in the brambles to the southwest.  \"Back there.  You can crawl to the secret way in.\"  Giggling, she stands up.  She skips back to the shack, hugging her rag doll close and waving to you. ";
					self.foundpath := true;
					self.gotdoll := true;
					iona.moveInto(shack);
					iona2Hint.active := nil;
					addword(brambles, &noun, 'gap');
				}
			} else {
				"The little girl's eyes go wide at the sight of the rag doll, but then they look back at you and her fear seems to overtake her again. ";
			}
		} else if (dobj = rations) {
			// If the player is dressed like a raider, the poor girl gets
			// understandably upset.
			if (!self.greeted) {
				if (wearingUniform(parserGetMe())) {
					"\^<<iona.thedesc>> takes one look at your clothing, lets out a shriek, and dashes out of the shack. ";
					iona.moveInto(nil);
					return;
				}
				"You unwrap one of the small packages and hand it to her.  Tentatively she leans forward and accepts it, quickly pulling back.  She takes a tiny bite... and a huge grin spreads across her face.  She wolfs the remainder down.<<global.dispParagraph>>\"What's your name, little girl?\" you ask.<<global.dispParagraph>>\"Iona,\" the little girl whispers.  You smile at her and she smiles back.  She doesn't seem quite so afraid of you anymore. ";
				iona.greeted := true;
				notify(iona, &randomaction, rand(8)+5);
				ionaHint.active := nil;
				iona2Hint.active := true;
				self.greeted := true;
			} else {
				"<<self.thedesc>> accepts the bit of food with a grin, then quickly devours it. ";
			}
		} else {
			"\^<<self.thedesc>> shakes her head. ";
		}
	}
	ioShowTo(actor, dobj) = {
		if (dobj = doll) {
			self.ioGiveTo(actor, dobj);
		} else if ((dobj = pistol) || (dobj = rifle)) {
			"\^<<self.thedesc>> shies away. ";
		} else if ((dobj = necklace) && self.greeted) {
			"\"It used to be my mom's.\"  She sniffles.  \"But you can keep it.  Since you're so nice.\" ";
		} else if (dobj = rations) {
			if (self.greeted) {
				"<<self.thedesc>> accepts the bit of food with a grin, then quickly devours it. ";
			} else {
				"\^<<self.thedesc>>'s eyes are riveted on the small piece of food you hold up.  She begins to tremble slightly. ";
			}
		} else {
			"\^<<self.thedesc>> blinks with wide eyes. ";
		}
	}
	verIoShowTo(actor) = {}
	verDoKiss(actor) = {}
	doKiss(actor) = {
		if (self.greeted) {
			"You kiss Iona on the forehead and she smiles up at you. ";
		} else {
			"As you move in towards the little girl, her eyes widen and she screams.  With surprising agility, she dashes between your legs and through the shack door. ";
			self.moveInto(nil);
		}
	}
	// Zork 2 reference here
	verDoRape(actor) = {}
	doRape(actor) = {
		"The girl screams as you approach. \"Won't someone deliver me from this awful fate?\" she cries.  Just in time, the Wizard of Frobozz appears, seeming to unroll himself out of nothing like a window shade.  \"Fry!\" he intones, and a massive bolt of lightning reduces you to a pile of smoking ashes.  (Serves you right, too, if you ask me.)";
		lose(true);
	}
	verDoScrew(actor) = { self.verDoRape(actor); }
	doScrew(actor) = { self.doRape(actor); }
	
	actorAction(v, d, p, i) = {
		if (v = helloVerb) {
			// If the player is dressed like a raider, the poor girl gets
			// understandably upset.
			if (wearingUniform(parserGetMe())) {
				"\^<<iona.thedesc>> takes one look at your clothing, lets out a shriek, and dashes out of the shack. ";
				iona.moveInto(nil);
				iona3Hint.active := true;
				exit;
			}
			if (self.greeted) {
				"Iona smiles up at you. ";
			} else {
				"\^<<self.thedesc>> seems too afraid of you to respond. ";
//				"\"What's your name, little girl?\"<<global.dispParagraph>>\"Iona,\" the little girl whispers.  You smile at her and she smiles back.  She doesn't seem quite so afraid of you anymore. ";
//				notify(iona, &randomaction, rand(8)+5);
//				self.greeted := true;
//				ionaHint.active := nil;
//				iona2Hint.active := true;
			}
		} else if (!self.greeted) {
			"\^<<self.thedesc>> shrinks back from you. ";
		} else if (v = followVerb) {
			"Iona bites her lip and looks at you for a moment, then takes your hand. ";
			parserGetMe().myfollower := iona;
		} else if (v = swearVerb)
			self.disavow;
		else if (v = yesVerb)
			"Iona smiles up at you. ";
		else if (v = noVerb)
			"Iona frowns up at you. ";
		else if (isclass(v, passwordverb))
			"Iona looks confused. ";
		else
			"Iona shakes her head. ";
		exit;
	}
	sayArriving = {
		self.location.dispParagraph;
		"\^<<self.thedesc>> follows close behind. ";
		if (self.location = northvillage) {
			"She seems to position herself so that you stand between her and the skeletons. ";
		}
	}
	sayLeaving = {}
	// Iona won't follow the PC to all locations
	follow(actor, room) = {
		if ((room = firstfloorhall) || (room = closet) || (room = basement)) {
			actor.myfollower := nil;
			return;
		}
		self.travelTo(room);
		if (room = crumblingbuilding) {
			"<<global.dispParagraph>>Iona looks down into the basement.  \"I'm not going down there,\" she whispers, her face pale with fear. ";
		}
		if (room = baseentrance) {
			"<<global.dispParagraph>>Iona looks down into the base.  \"I'm not going down there,\" she whispers, her face pale with fear. ";
		}
		if (room = hiddenpath) {
			"<<global.dispParagraph>>Iona looks down into the air vent.  \"I'm not going down there,\" she whispers, her face pale with fear. ";
		}
	}
	verDoTellAbout(actor, io) ={}
	// Random things to be cute
	randomaction = {
		if (self.location = parserGetMe().location) {
			switch (rand(8)) {
				case 1:
					"<<global.dispParagraph>>Iona sniffles and wipes her arm across her nose. ";
					break;
				case 2:
					"<<global.dispParagraph>>Iona pokes you in the leg and grins when you look down at her. ";
					break;
				case 3:
					"<<global.dispParagraph>>Crouching down, Iona scrawls some random designs in the dirt, then wipes them away. ";
					break;
				case 4:
					"<<global.dispParagraph>>Iona scratches her head. ";
					break;
				case 5:
					"<<global.dispParagraph>>Iona hums tunelessly. ";
					break;
				case 6:
					"<<global.dispParagraph>>Iona sighs and stares down at her feet. ";
					break;
				case 7:
					"<<global.dispParagraph>>Grinning, Iona hugs your leg tightly. ";
					break;
				case 8:
					"<<global.dispParagraph>>Iona rubs the side of her nose.  \"Are we there yet?\" ";
					break;
			}
		}
		notify(iona, &randomaction, rand(8)+5);
	}
	verIoTradeWith(actor) = {
		local dobj := parserGetObj(PO_DOBJ);
		if (dobj = doll)
			"\^A huge crocodile tear wells up in her eye.  \"B-b-but I don't have anything to trade!\" ";
		else
			"\^<<self.thedesc>> doesn't seem interested. ";
	}
	verDoTouch(actor) = {}
	doTouch(actor) = {
		if (self.greeted) {
			"You ruffle Iona's hair and she smiles up at you. ";
		} else {
			"As you move in towards the little girl, her eyes widen and she screams.  With surprising agility, she dashes between your legs and through the shack door. ";
			self.moveInto(nil);
		}
	}
	verDoTake(actor) = {}
	doTake(actor) = {
		if (self.greeted) {
			"Thin as she is, you doubt you'd be able to carry her around for long. ";
		} else {
			"As you move in towards the little girl, her eyes widen and she screams.  With surprising agility, she dashes between your legs and through the shack door. ";
			self.moveInto(nil);
		}
	}
	doHug(actor) = {
		if (self.greeted) {
			"You lean down and embrace Iona.  She hugs you back tightly. ";
		} else {
			"As you move in towards the little girl, her eyes widen and she screams.  With surprising agility, she dashes between your legs and through the shack door. ";
			self.moveInto(nil);
		}
	}
	doTalkto(actor) = {
		if (self.greeted) {
			"Iona doesn't seem too talkative, so you tell her a little about life in Arcadia.  You're not sure she believes you. ";
		} else {
			"\^<<self.thedesc>> seems too afraid of you to respond. ";
		}
	}
	verGrab(item) = {
		"Since <<self.thedesc>> is carrying <<item.thedesc>>, you decide to let her keep <<item.itobjdesc>>. ";
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"You feel like you could drown in her pure and innocent nature.  Fragments of violent images dredge up from her deepest subconscious. ";
		"She cocks her head and looks at you with a quizzical look. ";
	}
	smelldesc = "She doesn't smell all too clean. "
;

ionacorpse: fixeditem
	noun = 'girl' 'iona' 'body' 'corpse'
	adjective = 'small' 'little' 'iona\'s' 'girl\'s'
	location = nil
	isHer = true
	sdesc = {
		if (iona.greeted)
			"Iona's body";
		else
			"little girl's body";
	}
	adesc = {
		if (iona.greeted)
			"Iona's body";
		else
			"a little girl's body";
	}
	thedesc = {
		if (iona.greeted)
			"Iona's body";
		else
			"the little girl's body";
	}
	ldesc = "This is the body of a little girl, probably about five or six years old.  She has dirty brown hair which would probably reach to her shoulders if it wasn't so tangled.  She wears a plain brown dress of a coarse (and very bloody) fabric.  Her face is also covered in grime and streaked with blood. "
	isListed = true
	smelldesc = "She doesn't smell any better now that she's dead. "
	verDoTake(actor) = "You can't see any possible reason to do that. "
;

tub: openable, readable
	noun = 'tub' 'bucket'
	adjective = 'small' 'plastic'
	sdesc = "small plastic tub"
	location = shack
	maxbulk = 3		// maximum bulk the container can contain
	bulk = 3
	weight = 3
	isopen = nil
	ldesc = {
		"It appears to be a small tub, maybe three inches in diameter, made of yellowish-orange plastic.  Whatever lettering was on the side or lid has long since worn away.  The tub is currently";
		if (self.isopen) {
			" open ";
			if (self.contentsVisible and itemcnt(self.contents) <> 0) {
				"and contains "; listcont(self); ". ";
			} else {
				"and empty. ";
			}
		} else {
			" closed. ";
		}
	}
	smelldesc = "It has a faint, earthy smell. "
	readdesc = "The writing is too faded to read. "
;

clay: item
	noun = 'clay'
	adjective = 'squishy' 'lump' 'of'
	sdesc = "lump of clay"
	location = tub
	weight = 2
	bulk = 2
	ldesc = "A lump of clay, probably the kind used in sealing gaps between wooden planks.  Kinda squishy. "
	smelldesc = "It has a strong earthy smell. "
	touchdesc = "It feels kind of squishy. "
	verDoTake(actor) = {
		if (location.isGun) {
			"The clay is too far down in the barrel of <<location.thedesc>>, and the barrel is simply too narrow for you to work with. ";
		}
	}
;

crumblingbuilding: darkroom
	sdesc = "Crumbling Building"
	ldesc = "You find yourself inside one of the largest structures in this quiet village.  It may have been some kind of town hall or meeting area, as it consists of a single large room.  Huge gaping holes not only in the floor, ceiling, and south wall indicate the quality of workmanship.  Time and the elements will eventually bring the entire building down.  It appears to have been cleared of everything except for rotting furniture and various other pieces of debris.  A single crooked doorway leads east. "
	south = edgeofvillage
	east = northvillage
	down = { "You carefully lower yourself down into the basement.\b"; return basement; }
	collapsed = nil
	listendesc = "The building creaks ominously. "
	exitlist = { "South and east are the only reasonable exits.  You might be able to travel down as well. "; return nil; }
	smelldesc = "You catch a whiff of decay. "
	handleExplosion(actor, playerloc, heard) = {
		self.collapsed := true;
	}
	floordesc = "A large section of the floor has collapsed down into the basement.  There's no stairs or ladder in sight. "
	ceilingdesc = "The holes above are like inky blackness as you sweep your flashlight across the ceiling. "
;

basement: darkroom
	sdesc = "Collapsed Basement"
	noun = 'basement' 'cellar'
	adjective = 'collapsed'
	ldesc = {
		"This wouldn't be your first choice for a final resting place.  But clearly not everyone has the luxury of making that decision for themselves.  Among the split and crumbling timbers that litter the floor of this basement lie perhaps a dozen dead bodies.  It's hard to tell for certain because the corpses are stacked in a large pile in a corner of the room, the body parts jumbled together like a macabre jigsaw puzzle.  The scent of decay is nearly overwhelming.  The splinters of wood scattered across the ground only enhance the aura of past violence. ";
		if (!sitchair.grenadeblast)
			"Only a single chair seems to have survived the mayhem. ";
	}
	specialdesc = {
		if ((doll.location = self) && (itemcnt(self.contents) < 1)) {
			return 'A discarded rag doll lies among the carnage. ';
		}
		return '';
	}
	enterRoom(actor) = {
		basementHint.active := true;
		pass enterRoom;
	}
	up = doorup
	// Special ladder text.  This is one of 2 spots where the ladder can be used.
	laddermessage = "You manage to grab one of the timbers above and pull yourself out of the basement.\b"
	ladderup = {
		basementHint.active := nil;
		return crumblingbuilding;
	}
	listendesc = "The dead lie quiet. "
	// Capture jump and up verbs
	roomAction(a, v, d, p, i) = {
		if (v = jumpVerb) {
			if (a.location = standchair) {
				uVerb.action(a);
				exit;
			} else {
				"You leap upwards and your fingers just scrape the edge of the ceiling above.  However, you can't get any grip at all, and you land back on the floor. ";
				exit;
			}
		}
		pass roomAction;
	}
	exitlist = { "There are no exits save up. "; return nil; }
	listContents = {
		if (itemcnt(self.contents) = 1) 
			"There is ";
		else
			"There are ";
		listcont(self); " on the ground, surrounded by the wooden splinters. ";
		if (doll.location = self) {
			"A discarded rag doll also lies among the carnage. ";
		}
	}
	smelldesc = "The stink of death is unavoidable down here. "
	ceilingdesc = "There's a gaping hole in the ceiling that appears to be the room's only exit."
;

doorup: doorway
	location = basement
	doordest = crumblingbuilding
	destination = {
		if (!location.islit) {
			darkTravel();
			return nil;
		}
		"You can't quite reach the next floor. ";
		return nil;
	}
;

// This chair is for sitting!  TADS2 designated nested rooms like this as 
// applying to only one state (sitting on, standing on, lying down on), I
// made 2 different objects for whether the player is sitting or standing
// the chair.  If the player stands on the chair, this chair shuffles off
// to the other chair object (which has no nouns and cannot be referred to
// directly).
sitchair: chairitem
	reachable = ([standchair] + self)
	canReachContents = true
	noun = 'chair' 'leg'
	adjective = 'wooden' 'single'
	sdesc = "chair"
	ldesc = "A rickety wooden chair.  One leg is plainly shorter than the others. "
	location = basement
	verDoStandon(actor) = {
		if (actor.location = standchair) {
			"%You're% already on "; self.thedesc; "! ";
		}
	}
	doStandon(actor) = {
		"Okay, %you're% now standing on <<self.thedesc>>.  It wobbles slightly under your weight. ";
		actor.travelTo(standchair);
	}
	up = { "The next floor is well out of reach from a sitting position."; return nil; }
	verDoUse(actor) = { self.verDoSiton(actor); }
	doUse(actor) = { self.doSiton(actor); }
	verDoClimb(actor) = { self.verDoStandon(actor); }
	doClimb(actor) = { self.doStandon(actor); }
	verDoTake(actor) = "\^<<self.thedesc>> is too heavy to be carrying around. "
	smelldesc = "The stink of death is unavoidable down here. "
	handleExplosion(actor, playerloc, heard) = {
		sitchair.moveInto(nil);
		standchair.moveInto(nil);
	}
	verDoMove(actor) = {
		"You slide the chair around pieces of debris. ";
	}
;

// This is the chair for standing on.
standchair: chairitem
	reachable = ([sitchair] + self)
	canReachContents = true
	sdesc = "chair"
	location = basement
	outOfPrep = "off of"
	statusPrep = "on"
	up = {
		"You manage to grab one of the timbers above and pull yourself out of the basement.\b";
		basementHint.active := nil;
		return crumblingbuilding;
	}
	smelldesc = "The stink of death is unavoidable down here. "
;

doll: item, knownTopic
	location = basement
	sdesc = "rag doll"
	noun = 'doll' 'ragdoll' 'toy' 'dolly'
	adjective = 'rag' 'moth-eaten'
	bulk = 2
	weight = 1
	ldesc = "A moth-eaten rag doll with only the slightest resemblance to a human shape. "
	isListed = {
		return (self.location != basement);
	}
	doBurn(actor) = {
		"The doll burns quickly in the campfire. ";
		self.moveInto(nil);
	}
	smelldesc = "You detect an odor that is not entirely pleasant. "
	doDrop(actor) = {
		inherited.doDrop(actor);
		if (self.location = shack && iona.location = shack) {
			iona.ioGiveTo(actor, self);
		}
	}
;

debris: qcontainer, fixeditem
	noun = 'timbers' 'splinters' 'wood' 'debris' 'mayhem'
	adjective = 'split' 'crumbling' 'shattered'
	location = basement
	sdesc = "debris"
	isopenable = true
	isopen = nil
	contentsReachable = true
	contentsVisible = true
	ldesc =  {
		"The split pieces of wood are scattered over the entire floor, concealing most of the room.  The destruction is so complete that you can't even hazard a guess as to what they originally belonged to.  Some of the debris must have come from the ceiling above, but not all of it. ";
		if (screwdriver.location = self)
			"You think you notice a glint of metal within the splinters of wood. ";
	}
	doSearch(actor) = {
		local num, i;
		if (itemcnt(self.contents) <> 0) {
			"\^%you% root around the splintered wood and discover%s% <<listcont(self)>>, which you take. ";
			num := itemcnt(self.contents);
			for (i:=1; i<=num; i++) {
				self.contents[1].moveInto(actor);
			}
			isopen := true;
		} else {
			"There's nothing in <<self.thedesc>>. ";
		}
	}
	verDoTake(actor) = "\^<<self.thedesc>> would probably give you a very painful bunch of splinters. "
	verIoPutIn(actor) = "A waste of time. "
	smelldesc = "The stink of death is unavoidable down here. "
;

screwdriver: item, iron
	noun = 'screwdriver' 'driver' 'glint' 'metal'
	adjective = 'screw'
	sdesc = "screwdriver"
	location = debris
	ldesc = {
		if (self.location = debris) {
			"You root around the splintered wood and discover that the glint of metal is a screwdriver, which you take. ";
			self.moveInto(parserGetMe());
		} else {
			"Much of the metal on the screwdriver shaft is pitted and rusted.  The handle, what there is of it, is wrapped in a ragged black tape. ";
		}
	}
	bulk = 2
	weight = 2
	verIoTurnWith(actor) = {}
	ioTurnWith(actor, dobj) = {
		if ((dobj = screws1) || (dobj = panel1)) {
			if (!panel1.islocked) {
				self.ioScrewWith(actor, dobj);
			} else {
				self.ioUnscrewWith(actor, dobj);
			}
		} else {
			dobj.doTurn(actor);
		}
	}
	verIoScrewWith(actor) = {}
	verIoUnscrewWith(actor) = {}
	ioUnscrewWith(actor, dobj) = {
		if ((dobj = screws1) || (dobj = panel1)) {
			"The screws each resist at first, but with some effort, you manage to loosen them all. ";
			panel1.islocked := nil;
		}
	}
	ioScrewWith(actor, dobj) = {
		if ((dobj = screws1) || (dobj = panel1)) {
			"You tighten the screws. ";
			panel1.islocked := true;
		}
	}
	verIoAttackWith(actor) = {}
	ioAttackWith(actor, dobj) = {
		dobj.doAttack(self);
	}
	verDoUseOn(actor, io) = {
		if (io = screws1) {
			return;
		}
		if (io = screws2) {
			screws2.verDoScrew(actor);
			return;
		}
		self.usemessage;
	}
	moveInto(room) = {
		delword(self, &noun, 'glint');
		delword(self, &noun, 'metal');
		pass moveInto;
	}
	verDoDrink(actor) = {}
	doDrink(actor) = {
		"Mmm, a delicious blend of orange juice and vodka.  And there's still a lot left too. ";
	}
;

northvillage: darkroom
	sdesc = "North End of Village"
	ldesc = {
		"The main north-south road of this gloomy hamlet ends here at a tangle of underbrush to the north.  The tiny houses here look downright eerie in the cascading glow of your flashlight.  To your west ";
		if (crumblingbuilding.collapsed) {
			"was once the largest structure of the village, now just a pile of debris.";
		} else {
			"is the largest structure of the village, a wide one-story lodge that looks to be collapsing under its own weight.";
		}
	}
	specialdesc = {
		if (!self.grenadeblast)
			return 'Lying in the middle of the road are a pair of human skeletons, their bones scattered across the ground.  The ground around them appears nearly black in your electric light. ';
	}
	bOutdoor = true
	west = doortocrumblingbuilding2
	south = centerofvillage
	east = { return self.badexit('You peek your head inside one of the buildings and find nothing but ghosts. '); }
	north = { return self.badexit('Your path is blocked by the underbrush. '); }
	listendesc = "The area seems deathly silent. "
	exitlist = { "The heavy vegetation to the north means the only available pathways are east, west, and south. "; return nil; }
	listContents = {
		"\^%you% see%s% "; listcont(self); " dropped on the road. ";
	}
	smelldesc = "There is a faint smell of decay. "
	scorchdesc = {
		inherited.scorchdesc;
		"Bones are scattered across the area. ";
	}
	handleExplosion(actor, playerloc, heard) = {
		if (necklace.location = skeleton1)
			necklace.moveInto(nil);
	}
;

doortocrumblingbuilding2: doortocrumblingbuilding1
	location = northvillage
;

skeleton1: qcontainer, fixeditem, knownTopic
	noun = 'skeleton' 'skeletons' 'bones' 'body' 'bodies' 'sinew' 'flesh' 'skull' 'hole' 'gouges' 'scratches' 'bites' 'bite' 'marks' 'carnage' 'skull' 'remnants' 'bone'
	adjective = 'pair' 'human' 'strips' 'strips of' 'scattered'
	location = northvillage
	sdesc = {
		if (northvillage.grenadeblast)
			"scattered bones";
		else
			"pair of skeletons";
	}
	isThem = {
		return northvillage.grenadeblast;
	}
	isopenable = nil
	isopen = true
	ldesc = {
		if (northvillage.grenadeblast) {
			"The bones are scattered across the road.  Many of them are blackened, splintered, and broken. ";
		} else {
			"There are strips of sinew and flesh hanging from some of the bones.  One of the skeletons is visibly larger in stature than the other, and the larger body has a near perfect hole in the front of its skull.  A closer look at the bones reveals that most are covered with gouges and scratches that look suspiciously like bite marks.  Several of the bones have been dragged away from the main cluster. ";
			if (necklace.location = skeleton1)
				"As you move your flashlight among the carnage, something glints in the illumination. ";
		}
	}
	doSearch(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0) {
			"You search among the bones and discover a simple necklace, which you take. ";
			necklace.moveInto(actor);
		} else {
			"You can't find anything else of value amidst <<self.thedesc>>. ";
		}
	}
	verDoTake(actor) = "Even if you did want to disturb these poor people's remains, there's no way you could juggle all those bones. "
	smelldesc = "The smell of decomposition still lingers. "
	verDoMove(actor) = "You decide to let the dead rest in peace. "
	verDoTouch(actor) = "You decide to let the dead rest in peace. "
	verDoPush(actor) = "You decide to let the dead rest in peace. "
	verDoPull(actor) = "You decide to let the dead rest in peace. "
	verDoPutIn(actor) = "You decide to let the dead rest in peace. "
;

necklace: clothingItem, knownTopic
	noun = 'necklace' 'jewelry' 'something' 'cord' 'crystal' 'quartz' 'glint' 'glints'
	verb = 'leather'
	location = skeleton1
	sdesc = "necklace"
	bulk = 3
	weight = 2
	seen = nil
	ldesc = {
		if (!self.seen) {
			"Turns out it's nothing more than a quartz crystal hanging from a leather cord.  It sparkles nicely, but you wouldn't get more than a couple bucks for it at the Arcadia market. ";
			self.seen := true;
		} else {
			"Worthless, really.  It's nothing more than a quartz crystal hanging from leather cord.  It sparkles nicely, but you wouldn't more than a couple bucks for it at the Arcadia market. ";
		}
	}
	takeMessage = {
		if (!self.seen) {
			"You pull out the sparkling object and discover it to be a simple necklace, a crystal hanging from a leather cord. ";
			self.seen := true;
		} else
			pass takeMessage;
	}
	moveInto(room) = {
		delword(self, &noun, 'something');
		delword(self, &noun, 'glint');
		delword(self, &noun, 'glints');
		pass moveInto;
	}
;

// This room is designed to be somewhat useful both lit and dark.  It allows
// some verbs to be used even when not properly lit.
fadedpath: darkroom
	sdesc = "Faded Path"
	ldesc = {
		"This grassy walkway winds through the thick dry underbrush and trees, coming in from the north and changing directions several times before straightening out and continuing to the southeast.  This area too is eerily quiet, without even the usual chirping of nocturnal insects.  Large bushes line the path, struggling to survive in this harsh landscape.  To the southeast, you can make out a bright, flickering light from what appears to be a campfire. ";
		if (bush.contentsVisible and itemcnt(bush.contents) <> 0) {
			"<<global.dispParagraph>>You can just make out <<listcont(bush)>> partially concealed behind the bush.";
		}
	}
	bOutdoor = true
	north = clearing
	se = pathtobase
	darkdesc = {
		"It's too dark to see anything nearby.  To the southeast, you can make out a bright, flickering light from what appears to be a campfire. ";
	}
	lookAround(verbosity) = {
		if (self.islit)
			inherited.lookAround(verbosity);
		else
			"It's too dark to see.  To the southeast, you can make out a bright, flickering light from what appears to be a campfire. ";
//		helmet.detect(self);
	}
	roomAction(a, v, d, p, i) = {
		if (((v = replyVerb) || (v = yellVerb)) && (guard.location = baseentrance)) {
			guard.hearplayer;
			exit;
		}
		if ((v=attackVerb || v=shootVerb) && (d=phantomguard)) {
		} else {
			pass roomAction;
		}
	}
	roomCheck(v) = {
		if (self.islit or v.isDarkVerb or v.isKindaDarkVerb or v=attackVerb or v=shootVerb)
			return true;
		else {
			if (v = lookVerb)
				self.darkdesc;
			else
				"<<self.darkmessage>>\n";
			return nil;
		}
	}
	listendesc = {
		switch (guard.mark) {
			case 0:
				"You hear nothing beyond the wind in the underbrush. ";
				break;
			case 1:
				"You hear a man's voice to the southeast. ";
				break;
			case 2:
				"The still air has been shattered by the sound of gunshots. ";
				break;
			case 3:
				"There are footsteps approaching from the southeast. ";
				break;
			case 4:
			case 5:
				"You can hear the man walking the path and grumbling quietly to himself. ";
				break;
			case 6:
				"You can just make out the fading sounds of footfalls as the man disappears into the night. ";
				break;
		}
	}
	exitlist = { "This path only heads to the north and southeast. "; return nil; }
	listContents = {
		"\^%you% see%s% "; listcont(self); " lying among the weeds. ";
	}
;

pathtobase: doorway
	location = fadedpath
	isopen = true

	doordest = baseentrance
	destination = {
		if (guard.mark = 3) {
			"You move down the path and come face-to-face with the surprised face of an unkempt man carrying a large rifle.  Without a word he fires.  The world spins and goes dark. ";
			lose(true);
		} if (guard.mark > 3) {
			"You dash out from behind the bush, sprinting down the path.  \"Ah ha!\" shouts the guard from behind you.  You may be fast, but his bullets are much faster. ";
			lose(true);
		} else
			return doordest;
	}
;

bush: hideitem
	noun = 'bush' 'bushes'
	adjective = 'large'
	sdesc = "bush"
	ldesc = "The large bushes have few leaves, but their tightly packed branches make them difficult to see through. "
	location = fadedpath
	reachable = [] + self + guard + phantomguard
	isqcontainer = true
	isListed = nil
	north = clearing
	se = pathtobase
	roomAction(a, v, d, p, i) = {
		if ((v = yellVerb) && (guard.location = fadedpath)) {
			"Hearing your shout, the guard turns in your direction and fires a burst into the bush with his rifle.  You have only seconds to regret your action before your life comes to an end. ";
			lose(true);
		}
		pass roomAction;
	}
	listendesc = {
		if (parserGetMe().location = self)
			return self.location.listendesc;
		else
			"It rustles slightly in the faint breeze. ";
	}
	smelldesc = "The bush has a pleasant, flowery scent. "
;

// This lets the player refer to the guard even when the guard is out
// of scope (in the next room).  But since the guard can be heard from
// the next room, it makes sense to be able to refer to him at a
// distance.  The phantomguard and regular guard should never be in
// the same room, and there is special disambiguation to always prefer
// the real guard to the phantom on when possible.
phantomguard: Actor
	noun = 'raider' 'bandit' 'guard' 'brigand' 'man' 'voice' 'himself'
	adjective = 'bandit' 'raider' 'brigand' 'scruffy' 'looking' 'scruffy-looking' 'gruff' 'distant'
	sdesc = "distant guard"
	thedesc = "the distant guard"
	isHim = true
	ldesc = "You can't see him from here. "
	location = fadedpath
	actorDesc = nil
	showActorDesc = nil
	dobjGen(a, v, i, p) = {
		if (v <> inspectVerb && v <> attackVerb && v <> shootVerb && v <> tellVerb && v <> askVerb && v <> talkVerb) {
			"He's too far away.";
			exit;
		}
	}
	iobjGen(a, v, d, p) = {
		if (v <> inspectVerb && v <> attackVerb && v <> shootVerb && v <> tellVerb && v <> askVerb && v <> talkVerb) {
			"He's too far away.";
			exit;
		}
	}
	actorAction(v, d, p, i) = {
		guard.hearplayer;
		exit;
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if ((io != pistol) && (io != rifle))
			"You're too far away. ";
	}
	hesitatemsg = "You hesitate for a moment, being unable to clearly see your target.  You know the chances of hitting are extraordinarily poor unless you can see your target. "
	doAttack(weapon) = {
		if (weapon != grenade)
			inherited.doAttack(weapon);
		if ((weapon = pistol) || (weapon = rifle)) {
			"You blindly discharge your weapon in the direction of the campfire.  The return burst of automatic fire, unfortunately, actually manages to score several hits. ";
			lose(true);
		}
	}
	verDoAskAbout(actor, iobj) = {}
	doAskAbout(actor, iobj) = {
		self.doTalkto(actor);
	}
	verDoTellAbout(actor, iobj) = {}
	doTellAbout(actor, iobj) = {
		self.doTalkto(actor);
	}
	doTalkto(actor) = {
		guard.hearplayer;
		exit;
	}
	validActor = {
		if (findRootRoom(guard) = findRootRoom(parserGetMe()))
			return nil;
		if (guard.location = nil)
			return nil;
		if (self.location = nil)
			return nil;
		if (findRootRoom(parserGetMe()) = fadedpath)
			return true;
		return nil;
	}
;

baseentrance: room
	sdesc = "Base Entrance"
	ldesc = "At first glance, this might look like a normal clearing in the brush.  Tall grass fills in the open space between the taller vegetation, broken only by a path leading northwest.  However, the wide concrete-lined chasm leading down into the ground is a clear indication that this clearing was, at least initially, the result of human labor.  The opening is square, with a series of cement steps leading down to the south, ending at a metal-framed doorway.  The concrete surface extends for several feet around the entryway in a wide rectangle.  On the edge of this short plateau, directly in front of the stairs, is a small campfire and, beside it, a bedroll. "
	bOutdoor = true
	nw = fadedpath
	down = firstfloorhall
	south = firstfloorhall
	listendesc = "The campfire crackles and spits as it burns. "
	exitlist = { "You can head along the path to the northwest or down the steps to the south. "; return nil; }
	listContents = {
		"On the ground %you% can see%s% "; listcont(self); ". ";
	}
	smelldesc = "You can smell the smoke of the fire. "
;

// The real outdoor guard
guard: Actor
	noun = 'raider' 'bandit' 'guard' 'brigand' 'man' 'himself' 'jeans' 'boots'
	adjective = 'bandit' 'raider' 'brigand' 'scruffy' 'looking' 'scruffy-looking' 'dingy'
	sdesc = "raider guard"
	thedesc = "the raider guard"
	location = baseentrance
	isHim = true
	suspicion = 0
	threshold = 3
	fired = nil
	ldesc = {
		if (location = fadedpath) {
			"The illumination from his lantern shows an unshaved man of about average height.  He appears to be wearing a pair of raggedy jeans, boots, and a patchwork coat of alternating colors.  You think you recognize them as raider markings.  His left hand helps support the large rifle he carries while also holding onto the handle of his electric lamp. ";
		} else {
			"A scruffy-looking man of about average height sitting beside the fire.   He's dressed in a ragged pair of jeans with a pair of worn boots.  His coat has alternating bands of red and green that you recognize as the wasteland equivalent of gang colors.  Raider groups use such \"uniforms\" to distinguish themselves from each other.  Cradled in his lap is an automatic rifle.  He watches you keenly. ";
		}
	}
	islamp = true
	islit = true
	spotted = 0
	heard = nil
	mark = 0
	searching = nil
	reassured = nil
	actorDesc = "A dingy guard scans the area attentively. "
	doAskAbout(actor, iobj) = {
		if (self.location = fadedpath) {
			"Hearing you, the guard spins in your direction and fires a burst into the bush with his rifle.  You have only seconds to regret your action before your life comes to an end. ";
			lose(true);
		}
		pass doAskAbout;
	}
	verDoTake(actor) = "You find yourself overestimating your physique. "
	verDoTellAbout(actor, iobj) = {
	}
	doTellAbout(actor, iobj) = {
		if (self.location = fadedpath) {
			"Hearing you, the guard spins in your direction and fires a burst into the bush with his rifle.  You have only seconds to regret your action before your life comes to an end. ";
			lose(true);
		}
		if (iobj = Me) {
			"If you blow your cover, it could cost you your life.. ";
			return;
		}
		"You start to speak and the guard cuts you off.  ";
		self.disavow;
	}
	doTalkto(actor) = {
		if (self.location = fadedpath) {
			"Hearing you, the guard spins in your direction and fires a burst into the bush with his rifle.  You have only seconds to regret your action before your life comes to an end. ";
			lose(true);
		}
		self.disavow;
	}
	disavow = "\"I ain't got time for jibber-jabber.  I'm on guard duty tonight.\" "
	sayLeaving = {
		"<<global.dispParagraph>>The man leaves to the southeast, muttering under his breath. ";
	}
	sayArriving = {
		"<<global.dispParagraph>>A scruffy-looking man enters the area, searching the area with an electric lantern and a large rifle. ";
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if (!io.isGun)
			"That wouldn't be advisable. ";
	}
	shootplayer = {
		if (parserGetMe().location = baseentrance) {
			"<<global.dispParagraph>>\"Shit!\" the guard shouts as he sees you in the light of the campfire, bringing up his automatic rifle and firing in your direction.  The bullets tear through you and you're dead before you hit the ground. ";
		} else if (flashlight.islit) {
			"<<global.dispParagraph>>\"Shit!\" the guard shouts as he sees your light, bringing up his automatic rifle and firing in your direction.  The bullets tear through you and you're dead before you hit the ground. ";
		} else {
			"<<global.dispParagraph>>\"Shit!\" the guard shouts as he sees you in the light of his lantern, bringing up his automatic rifle and firing in your direction.  The bullets tear through you and you're dead before you hit the ground. ";
		}
		lose(true);
	}
	greetplayer = {
		if (!self.reassured) {
			"<<global.dispParagraph>>\^<<self.thedesc>> nods in your direction, but eyes you warily. ";
			reassured := true;
		}
		guard1Hint.active := nil;
	}
	doAttack(weapon) = {
		if (weapon = pistol) {
			if (self.location = fadedpath) {
				"Using the night vision sights on your pistol, you carefully take aim at the man from behind the bush.  The gun coughs quietly and the man gurgles, clutching at his throat.  He flails violently, twisting and staggering across the path.  The rifle, flung aside by his thrashing limbs, goes  spinning deep into the brush and disappears from sight.  The man's lamp is caught around his wrist and swings wildly through the air as he thrashes, smashing against his body several times before dropping to the ground and going out.  The night is plunged into darkness once more.  After what  seems like an eternity, you can hear the man finally collapse to the ground.  The air becomes eerily still.<<global.dispParagraph>>You can't help but shudder at the gruesome display.<<global.dispParagraph>>It's now pitch black. ";
				lantern.moveInto(fadedpath);
				lantern.islit := nil;
			} else {
				if (weapon != grenade)
					inherited.doAttack(weapon);
				"You quickly draw up your <<weapon.sdesc>>, but the guard reacts too quickly.  The rifle, already pointed in your direction, spits out a lethal fusillade of lead.  Several of the projectiles find their mark. ";
				lose(true);
			}
			remdaemon(guardDaemon, guard);
			phantomrifle.moveInto(self.location);
			guardrifle.moveInto(nil);
			guardcorpse.moveInto(self.location);
			guard1Hint.active := nil;
			guard.moveInto(nil);
			phantomguard.moveInto(nil);
			guard.mark := 0;
			guard.searching := nil;
		}
		if (weapon = rifle) {
			if (self.location = fadedpath) {
				if (rifle.rounds > 0) {
					"You aim as best you can in the darkness.  Without being able to see your sights you pull the trigger and hope for the best.  \"Wha--?\" the guard shouts, turning and firing wildly into the woods.  A second shot from your rifle seems to hit the guard dead in the chest.  He flails violently, twisting and staggering across the path.  The rifle, flung aside by his thrashing limbs, goes  spinning deep into the brush and disappears from sight.  The man's lamp is caught around his wrist and swings wildly through the air as he thrashes, smashing against his body several times before dropping to the ground and going out.  The night is plunged into darkness once more.  After what  seems like an eternity, you can hear the man finally collapse to the ground.  The air becomes eerily still.<<global.dispParagraph>>You can't help but shudder at the gruesome display.<<global.dispParagraph>>It's now pitch black. ";
					lantern.moveInto(fadedpath);
					lantern.islit := nil;
					rifle.rounds--;
				} else {
					"You aim as best you can in the darkness.  Without being able to see your sights you pull the trigger and hope for the best.  \"Wha--?\" the guard shouts, turning and firing wildly into the woods.  A second pull of the trigger... and nothing.  Your rifle is empty.  The guard's, however, is not.  Your shadowy form falls inside the beam of the lantern and the guard fires a burst into your unprotected body. ";
					lose(true);
				}
			} else {
				"He won't take his eyes off of you and that rifle is already pointed in your direction.  You seriously doubt it would be a good idea. ";
				return;
			}
			remdaemon(guardDaemon, guard);
			phantomrifle.moveInto(self.location);
			guardcorpse.moveInto(self.location);
			guardrifle.moveInto(nil);
			guard1Hint.active := nil;
			guard.moveInto(nil);
			phantomguard.moveInto(nil);
			guard.mark := 0;
			guard.searching := nil;
		}
		if (weapon = grenade) {
			remdaemon(guardDaemon, guard);
			guardcorpse.moveInto(self.location);
			guardrifle.moveInto(nil);
			guard1Hint.active := nil;
			guard.moveInto(nil);
			phantomguard.moveInto(nil);
			guard.mark := 0;
			guard.searching := nil;
		}
		if ((weapon = flashlight) ||(weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"You are gunned down before you even get close. ";
			lose(true);
		}
	}
	announce(actor, obj, val) = {
		if ((val != 4) && (findRootRoom(actor) = fadedpath)) {
			return;
		}
		if (obj = grenade) {
			switch (val) {
				case 1:
					"The guard gives you a cold stare.  \"You'd better put that pin back in, buddy.\" ";
					break;
				case 2:
					"The man by the campfire snorts angrily. ";
					break;
				case 3:
					"With barely a motion, the guard fires a burst from his rifle and you are quickly deceased. ";
					lose(true);
					break;
				case 4:
					"Almost immediately, a burst of gun fire originating from that direction tears into your unprotected body. ";
					lose(true);
					break;
			}
		}
	}
	verDoRape(actor) = {
		"Not advisable. ";
	}
	verDoScrew(actor) = { self.verDoRape(actor); }
	doScrew(actor) = { self.doRape(actor); }
	actorAction(v, d, p, i) = {
		if (parserGetMe().location = bush) {
			"Hearing your voice, the guard turns in your direction and fires a burst into the bush with his rifle.  You have only seconds to regret your action before your life comes to an end. ";
			lose(true);
		} else {
			"\^<<self.thedesc>> simply stares at you. ";
		}
		exit;
	}
	smelldesc = "A delicate aroma of lavender and rotting meat. "
	verGrab(item) = {
		"You doubt it would be that easy to wrestle it away from him. ";
	}
	doHug(actor) = {
		"He doesn't look like he'd appreciate it. ";
	}
	hearplayer = {
		if (!self.reassured) {
			"You shout out to the distant man... and are greeted with a hail of gunfire!  Bullets slam into the ground and trees around you, narrowly missing your fragile self.  Whoever's out there doesn't appear too friendly. ";
			if (guard.mark = 0) {
				guard.searching := true;
				guard.mark := 1;
				guard.spotted++;
			}
			guard.heard := true;
			guard.fired := true;
		} else {
			"There's no response from the guard. ";
		}
	}
	validActor = {
		if (findRootRoom(self) = findRootRoom(parserGetMe()))
			return true;
		return nil;
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"You can't help but blanch at the waves of cold hatred emanating from the man before you. ";
		"The guard snarls at you. ";
	}
;

// This controls the guard's pathing and behavior when responding to an 
// unexpected light or noise.
guardDaemon: function(actor) {
	local playeronpath := ((parserGetMe().location = fadedpath) || (parserGetMe().location.location = fadedpath));
	local playerinclearing := (parserGetMe().location = clearing);
	local playervisible := ((parserGetMe().location = fadedpath) || (playeronpath && flashlight.islit));
	if ((parserGetMe().location = baseentrance) && (actor.location = baseentrance)) {
		if (wearingUniform(parserGetMe())) {
			actor.greetplayer;
		} else {
			actor.shootplayer;
		}
		return;
	}
	if (actor.reassured)
		return;
	if (!actor.searching && (playeronpath && flashlight.islit))
		actor.searching := true;
	if (!actor.searching)
		return;
	if (!guard.reassured) {
		guard1Hint.active := true;
	}
	switch(actor.mark) {
		case 0:
			if (!fadedpath.islit)
				return;
			actor.spotted++;
			if (actor.spotted = 1) {
				"<<global.dispParagraph>>From the direction of the campfire, you hear a man's gruff voice shout out, \"Who's out there?  I can see your light.  I'll start shooting if you don't answer me!\" ";
			} else if (actor.spotted = 2) {
				"<<global.dispParagraph>>From the direction of the campfire, you hear a man's gruff voice shout out, \"I said who's out there?!\" ";
			} else {
				"<<global.dispParagraph>>From the direction of the campfire, you hear a man's gruff voice shout out, \"Don't make me come out there again!\" ";
			}
			if (iona.location = fadedpath) {
				"Iona lets out a tiny squeal and runs off to the north. ";
				parserGetMe().myfollower := nil;
				iona.travelTo(clearing);
			}
			break;
		case 1:
			if (playeronpath) {
				"<<global.dispParagraph>>";
				if (actor.fired) {
					"Another";
				} else {
					"A";
				}
				" series of gunshots erupts from the southeast.  Bullets whiz by you, throwing up showers of dirt and chips of bark. ";
			}
			if (playerinclearing) {
				"<<global.dispParagraph>>You hear a series of gunshots ring out far to the south. ";
			}
			break;
		case 2:
			if (playeronpath)
				"<<global.dispParagraph>>You hear footsteps approaching from the southeast!";
			break;
		case 3:
			actor.travelTo(fadedpath);
			phantomguard.moveInto(nil);
			lantern.moveInto(actor);
			lantern.islit := true;
			if (playervisible) {
				actor.shootplayer;
			}
			if (playerinclearing) {
				"<<global.dispParagraph>>A bobbing light appears further down the southern path.  You can hear faint sounds of movement. ";
			}
			break;
		case 4:
			if (playervisible) {
				actor.shootplayer;
			}
			if (playeronpath) {
				switch (actor.spotted) {
					case 1:
						"<<global.dispParagraph>>The man frowns and scratches his head as he peers around the area.  \"I could've sworn...\" ";
						break;
					case 2:
						if (actor.heard)
							"<<global.dispParagraph>>The man frowns and scratches his head as he peers around the area.  \"Bastard must've run off.\" ";
						else
							"<<global.dispParagraph>>The man makes a slow circle around the path.  \"What the hell is going on here?\" ";
						break;
					case 3:
						"<<global.dispParagraph>>The man on the path prods the underbrush with his rifle as he searches.  \"I know you're out here somewhere, vermin.  I'll find you eventually.\" ";
						break;
					default:
						"<<global.dispParagraph>>The guard shakes his head.  \"I must be seeing things.\" ";
						break;
				}
			}
			if (playerinclearing) {
				"<<global.dispParagraph>>The light down the southern path waves and dances about. ";
			}
			break;
		case 5:
			if (playervisible) {
				actor.shootplayer;
			}
			if (playerinclearing) {
				"<<global.dispParagraph>>The bobbing light to the south dims and finally disappears. ";
			}
			actor.travelTo(baseentrance);
			phantomguard.moveInto(fadedpath);
			lantern.moveInto(baseentrance);
			lantern.islit := nil;
			actor.mark := -1;
			actor.searching := nil;
			break;
	}
	actor.mark++;
}

guardrifle: item, gun
	noun = 'gun' 'rifle'
	adjective = 'guard\'s' 'man\'s' 'large' 'automatic'
	sdesc = "automatic rifle"
	ldesc = "A mean-looking automatic rifle. "
	location = guard
;

lantern: switchLightItemNotFixed, lightsource
	noun = 'lantern' 'lamp'
	adjective = 'electric'
	sdesc = "lantern"
	location = baseentrance
	ldesc = {
		"An electric lantern of green metal and clear glass. ";
		if (guard.location = nil) {
			"The glass is cracked, the base badly scuffed, and the bulb inside is quite dark. ";
		} else {
			if (self.islit)
				"It is currently on. ";
			else
				"It is currently off. ";
		}
	}
	brokenmsg = {
		if (guard.location != nil)
			"The guard growls.  \"That's mine.  Hands off.\" ";
		else
			"Nothing happens. ";
	}
	takemsg = {
		if (guard.location != nil)
			"The guard growls.  \"That's mine.  Hands off.\" ";
	}
	verDoSwitch(actor) = self.brokenmsg
	verDoTurnon(actor) = self.brokenmsg
	verDoTurnoff(actor) = self.brokenmsg
	verDoTake(actor) = self.takemsg
	verDoPutIn(actor, io) = self.takemsg
	verDoPutOn(actor, io) = self.takemsg
	verDoPull(actor) = self.takemsg
	verDoPush(actor) = self.takemsg
	verDoTouch(actor) = self.takemsg
;

guardcorpse: fixeditem, qcontainer
	noun = 'raider' 'bandit' 'guard' 'brigand' 'man' 'body' 'corpse' 'jeans' 'boots'
	adjective = 'bandit' 'raider' 'brigand' 'guard' 'man\'s'
	sdesc = "man's body"
	ldesc = {
		"A scruffy-looking dead man of about average height.  He's dressed in a ragged pair of jeans with a pair of worn boots, all streaked with blood. ";
		if (uniform1.location = self)
			"His coat has alternating bands of red and green that appear to be the wasteland equivalent of gang colors. ";
	}
	location = nil
	isHim = true
	isListed = true
	smelldesc = "He smells... well, awful. "
	doSearch(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0)
			"The only thing you can find worth taking is <<listcont(self)>>. ";
		else
			"Rummaging through <<self.thedesc>> turns up nothing else. ";
	}
	verDoTake(actor) = "\^<<self.thedesc>> is far too heavy to carry around. "
	verDoMove(actor) = {}
	doMove(actor) = {
		if (self.location = bush) {
			"You drag the body out from behind the bush. ";
			self.moveInto(fadedpath);
		} else {
			"You drag the body behind the bush, partially concealing it. ";
			self.moveInto(bush);
		}
	}
;

// I probably shouldn't have, but I decided that even if you kill the guard,
// you won't be able to get his gun.  Maybe I should have just let the player
// get it, since it wouldn't really change anything anyway.  In any event,
// this phantom rifle lets the player refer to it (try to pick it up) even
// after it flies into the brush.  It's there to let the player know not to
// worry about it.
phantomrifle: fixeditem
	sdesc = "rifle"
	noun = 'rifle' 'gun'
	ldesc = ""
	adjective = 'automatic' 'large'
	location = nil
	dobjGen(a, v, i, p) = {
		"The guard's rifle disappeared deep into the brush.  You doubt you'll ever be able to find it. ";
		abort;
	}
;

class uniform: clothingItem
	isUniform = true
	verIoAttachTo(actor) = {}
	ioAttachTo(actor, dobj) = {
		dobj.doWear(actor);
	}
;

wearingUniform: function(actor) {
	if (((uniform1.location = actor) && uniform1.isworn) ||
		((uniform2.location = actor) && uniform2.isworn))
		return true;
	return nil;
}

uniform1: uniform, knownTopic
	noun = 'uniform' 'coat'
	adjective = 'shabby'
	sdesc = "shabby coat"
	location = guardcorpse
	bulk = 10
	weight = 8
	ldesc = "A shabby coat with alternating bands of red and green that you recognize as the wasteland equivalent of gang colors. "
	takeMessage = {
		if (self.location = guardcorpse)
			"You strip the coat off of the body and try to wipe off as much blood as you can. ";
		else
			pass takeMessage;
	}
	verDoWear(actor) = {
		if (uniform2.isworn)
			"\^<<self.thedesc>> won't fit over <<uniform2.thedesc>>. ";
		else if (radsuit.isworn)
			"\^<<self.thedesc>> won't fit over <<radsuit.thedesc>>. ";
		else
			pass verDoWear;
	}
	smelldesc = "It doesn't smell all that pleasant, actually. "
	doBurn(actor) = {
		if (self.isworn) {
			"Not while you're wearing it. ";
			return;
		}
		"You toss the ratty piece of clothing onto the fire.  It quickly burns away. ";
		self.moveInto(nil);
	}
;

hiddenpath: darkroom
	sdesc = "Hidden Path"
	ldesc = "This is the end of a very convoluted and twisted path through several hundred feet of frighteningly thorny briar patch.  The track is extremely difficult to see, especially at night, and you know you never would have found your way without help.  It heads back into the thicket to your northeast, the only footpath leading from this small open field among the brambles.  In the middle of the small clearing is a square metal structure, about two feet long on each side and protruding a couple of inches from the ground.  This box is topped by a metal grating held on by rusty latches. "
	bOutdoor = true
	ne = clearing
	down = grating
	listendesc = "You hear nothing beyond the wind in the underbrush. "
	exitlist = { "The path twists off to the northeast. "; return nil; }
	smelldesc = "The faint scent of the forest fills your nostrils. "
;

grating: doorway, iron
	noun = 'box' 'grating' 'structure' 'latches' 'chimney' 'grate' 'hinges' 'latch'
	adjective = 'metal' 'rusty'
	sdesc = "metal grating"
	location = hiddenpath
	islockable = nil
	noAutoOpen = true
	ldesc = {
		"It appears to be the top end of some sort of chimney or air vent.  About two feet long on each side, it's constructed out of sheet metal.  A metal grating is attached to the top by a set of rusty hinges.  If there was originally a lock on it, it's long since disintegrated. ";
		if (self.isopen)
			"The metal grating is open, making the shaft below accessible.  Your flashlight reveals only that the shaft bends several feet down. ";
	}
	doordest = closet
	destination = {
		if (!location.islit) {
			darkTravel();
			return nil;
		}
		if (self.isopen) {
			"You lower yourself carefully into the shaft and squirm along as it turns horizontal.  After several feet of crawling, the duct turns vertical again.  Shimmying along its length, you drop down into the room beneath.\b ";
			return self.doordest;
		}
		else if (not self.islocked and not self.noAutoOpen) {
			self.setIsopen(true);
			"(Opening << self.thedesc >>)";
			return self.doordest;
		} else {
			"%You%'ll have to open << self.thedesc >> first. ";
			setit(self);
			return nil;
		}
	}
	doOpen(actor) = {
		"With neglected hinges screeching in protest, the grating swings open, revealing a wide shaft that disappears down into darkness. ";
		self.setIsopen(true);
	}

;

firstfloorhall: room
	sdesc = "First Floor Hallway"
	ldesc = {
		"Once pristine stainless steel has become soiled by the dirty boots of countless pedestrians.  Before the war, this hall was almost certainly kept immaculately clean as a necessary process of this research facility.  However, new occupants have been less than fastidious in keeping it in its original condition.  The metal grating beneath your feet is discolored and encrusted with dried mud.  White walls sport numerous scuff marks, gouges and, in several places, lewd and explicit graffiti.  The hallway continues east and west, with an exit north and a door to the south.  That door, you notice, has a number of variations on \"out of order\" scribbled across its surface.  On the wall beside it is a small panel. ";
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if (firstfloordoor.forced) {
			"The door to the south appears to have been blasted open.  The thick steel is buckled and scorched. ";
		} else if (firstfloordoor.isblocked) {
			"The door to the south has been blocked open by a metal pole in its track. ";
		}
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			if (firstfloordoor.forced || firstfloordoor.isblocked) {
				"<<global.dispParagraph>>";
			}
			"Whirling red lights and warning alarms fill the corridor.  Confused and directionless raiders run through the halls, searching vainly for the cause.  You seem to go completely unnoticed. ";
		}
		str := outcapture(stat);
		return str;
	}
	up = baseentrance
	north = baseentrance
	east = firstflooreast
	west = firstfloorwest
	south = firstfloordoor
	down = firstfloordoor
	enterRoom(actor) = {
		inherited.enterRoom(actor);
		if (!global.inbase) {
			global.inbase := true;
			incscore(1);
			global.scEnterBase := true;
			if (!global.insecond) {
				door1Hint.active := true;
			}
		}
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
		} else {
			"The place is eerily quiet... though you can make out a faint murmur to the east. ";
		}
	}
	exitlist = { "Passages lead north, south, east and west. "; return nil; }
;

// The big bad security door.  Requires badge and security code.  Plus
// it needs to be fixed (a power connection worked itself loose... or was
// deliberately disconnected).  Once successfully opened, the door will 
// self-unlock whenever approached or the player tries to 'open' it.  Can
// be braced open with the metal pole.
firstfloordoor: doorway, floatingItem, qcontainer, knownTopic
	noun = 'door' 'doorway'
	adjective = 'south' 'southern' 'security' 'north' 'northern'
	sdesc = "security door"
	plural = 'doors'
	location = {
		if ((parserGetMe().location = firstfloorhall) || (parserGetMe().location = firststairs)) {
			return parserGetMe().location;
		}
		return firstfloorhall;
	}
	doordest = firststairs
	islocked = true
	isblocked = nil
	isqcontainer = true
	isopen = nil
	destination = {
		if (self.isopen) {
			return self.doordest;
		} else if (global.opendoor) {
			if (panel1.badgeread) {
				"You enter the security code and the door slides open.\b";
			} else if (isinside(securitybadge, parserGetMe())) {
				"You scan the security badge and enter the code and the door slides open.\b";
			} else {
				"You can't get the door open without the security badge. ";
				return nil;
			}
			self.isopen := true;
			panel1.badgeread := nil;
			panel1.codetyped := nil;
			notify(panel1, &closedoor, 2);
			return self.doordest;
		} else {
			"%You%'ll have to open << self.thedesc >> first. ";
			setit(self);
			return nil;
		}
	}
	ldesc = {
		local first := (parserGetMe().location = firstfloorhall);
		if (first) {
			"It looks like a fairly solid security door.  Built out of brushed stainless steel, but even this wasn't able to withstand the determined new inhabitants and their need to desecrate everything they found.  As you look closer, you see faded letters that read \"SUBLEVEL 2\" through the random messages and graffiti.  Built into the wall right beside the door is a control panel. ";
		} else {
			"It looks like a fairly solid security door.  Built out of brushed stainless steel, much tidier on this side than the other.  As you look closer, you see faded letters that read \"SUBLEVEL 1\".  Built into the wall right beside the door is a large red button. ";
		}
		if (self.forced) {
			"<<global.dispParagraph>>The door appears to have been blasted open.  The thick steel is buckled and scorched. ";
		} else {
			if (self.isopen)
				"The door is currently open. ";
			else
				"The door is closed. ";
			if (self.isblocked) {
				"You notice there's a metal pole wedged in its track, keeping it open. ";
			}
		}
	}
	verDoUnlock(actor) = {
		"You're convinced you'll need to use the panel to unlock this door. ";
	}
	verIoPutIn(actor) = {
		if (self.forced) {
			"Not much point in it now.  That door doesn't look like it will ever close. ";
		} else if (!self.isopen) {
			"Not while the door is closed. ";
		}
	}
	ioPutIn(actor, dobj) = {
		if (dobj = ladder) {
			"That's too large to brace the door open. ";
		} else if (dobj != pole) {
			"Something a little longer might brace the door open. ";
		} else {
			"You wedge the metal pole into the door, preventing it from closing. ";
			pole.moveInto(self);
			self.isblocked := true;
		}
	}
	Grab(obj) = {
		if (obj = pole) {
			"As you pull the pole out of the door track, the door slides shut. ";
			self.isblocked := nil;
			self.isopen := nil;
		}
	}
	verDoBraceWith(actor, iobj) = {
		if (self.forced) {
			"Not much point in it now.  That door doesn't look like it will ever close. ";
		} else if (!self.isopen) {
			"Not while the door is closed. ";
		}
	}
	doBraceWith(actor, iobj) = {
		self.ioPutIn(actor, iobj);
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
	verDoOpen(actor) =
	{
		if (actor.location = firststairs)
			return;
		if (self.isopen) {
			"\^<<self.itisdesc>> already open. ";
		} else if (global.opendoor) {
			// Nothing
		} else if (self.islocked) {
			"\^<<self.itisdesc>> locked. ";
		}
	}
	doOpen(actor) = {
		if (actor.location = firststairs) {
			"You press the button and the door opens. ";
		} else if (panel1.badgeread) {
			"You enter the security code and the door slides open.";
		} else if (isinside(securitybadge, parserGetMe())) {
			"You scan the security badge and enter the code and the door slides open.";
		} else {
			"You can't get the door open without the security badge. ";
			return nil;
		}
		firstfloordoor.isopen := true;
		self.badgeread := nil;
		self.codetyped := nil;
		notify(panel1, &closedoor, 2);
	}
;

panel1: lockable, qcontainer, fixeditem, knownTopic
	noun = 'panel' 'hinge' 'hinges'
	adjective = 'small'
	sdesc = "panel"
	location = firstfloorhall
	isopen = nil
	islocked = true
	isbroken = true
	badgeread = nil
	codetyped = nil
	ldesc = {
		"The panel is built into the southern wall, right beside the security door.  It's a smooth plastic rectangle that juts out from the surface of the wall about a half inch.  The majority of its area is taken up by ";
		if (!self.isbroken)
			"a brightly illuminated ";
		else
			"an oddly dark ";
		"scanner grid.  You've seen these before, they're designed to react with security badges with embedded microchips.  When the badge is touched to the grid, the chip is activated and the user's identity is read.  Below the grid is a numeric keypad";
		if (!self.isbroken && self.badgeread)
			" with glowing number keys";
		else if (!self.badgeread)
			" which is dark";
		else
			", also dark";
		".  In each of the right two corners is a single screw, and you see a pair of hinges along the left side. ";
		if (self.isopen) {
			"The panel is currently open.  On the inside back of the panel, there is a metal-lined socket.  A bundle of wires ending in a metal prong runs out of the wall ";
			if (!self.isbroken) {
				"and into the socket. ";
			} else {
				"and dangles loosely in the air. ";
			}
		}
	}
	closedoor = {
		if (firstfloordoor.forced)
			return;
		if (firstfloordoor.isblocked) {
			if (parserGetMe().location = firstfloorhall)
				"<<global.dispParagraph>>The door to the south bangs against the metal pole laying in its track, but stays open. ";
			else if (parserGetMe().location = firststairs)
				"<<global.dispParagraph>>The door to the north bangs against the metal pole laying in its track, but stays open. ";
		} else {
			if (parserGetMe().location = firstfloorhall)
				"<<global.dispParagraph>>The door to the south slides shut. ";
			else if (parserGetMe().location = firststairs)
				"<<global.dispParagraph>>The door slides shut. ";
			firstfloordoor.isopen := nil;
		}
	}
	typecode = {
		"The door slides open, revealing a stairway to the south. ";
		firstfloordoor.isopen := true;
		global.opendoor := true;
		self.badgeread := nil;
		self.codetyped := nil;
		notify(self, &closedoor, 2);
	}
	readcard = {
		self.badgeread := true;
		"You touch the security badge to the scanning grid.  The keys on the keypad become lit. ";
		return;
	}
	doOpen(actor) = {
		"The panel swings open on tiny hinges.  On the inside back of the panel, there is a metal-lined socket.  A bundle of wires ending in a metal prong runs out of the wall ";
		if (!self.isbroken) {
			"and into the socket. ";
		} else {
			"and dangles loosely in the air. ";
		}
		self.isopen := true;
	}
	doClose(actor) = {
		"Closed. ";
		self.isopen := nil;
	}
	verDoOpen(actor) = {
		if (self.islocked)
			"\^<<self.thedesc>> won't budge. ";
		else
			pass verDoOpen;
	}
	verDoLock(actor) = {
		"\^<<self.thedesc>> isn't something that you can lock. ";
	}
	verDoUnlock(actor) = {
		"\^<<self.thedesc>> isn't something that you can unlock. ";
	}
	verDoScrew(actor) = {
		screws1.verDoScrew(actor);
	}
	verDoScrewWith(actor, iobj) = {
		screws1.verDoScrewWith(actor, iobj);
	}
	doScrew(actor) = {
		screws1.doScrew(actor);
	}
	verDoUnscrew(actor) = {
		screws1.verDoUnscrew(actor);
	}
	verDoUnscrewWith(actor, iobj) = {
		screws1.verDoUnscrewWith(actor, iobj);
	}
	doUnscrew(actor) = {
		screws1.doUnscrew(actor);
	}
	verIoPutIn(actor) = {
		"You can't put anything inside the panel. ";
	}
	verIoPutOn(actor) = {}
	ioPutOn(actor, dobj) = {
		grid1.ioPutOn(actor, dobj);
	}
	verDoTake(actor) = "\^<<self.thedesc>> is attached to the wall. "
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	ioTouchTo(actor, dobj) = {
		grid1.ioTouchTo(actor, dobj);
	}
	ioScanOn -> grid1
	ioScanWith -> grid1
;

screws1: fixeditem
	noun = 'screw' 'screws'
	sdesc = "screws"
	location = firstfloorhall
	ldesc = {
		"Two simple steel screws that are currently ";
		if (!panel1.islocked)
			"unscrewed.";
		else
			"screwed in tight.";
	}
	isThem = true
	verDoScrew(actor) = {
		if (panel1.isopen) {
			"Not while <<panel1.thedesc>> is open. ";
		}
		if (panel1.islocked) {
			"They're already tightened as far as they'll go. ";
		}
	}
	verDoScrewWith(actor, iobj) = {
		if (panel1.isopen) {
			"Not while <<panel1.thedesc>> is open. ";
		}
		if (panel1.islocked) {
			"They're already tightened as far as they'll go. ";
		}
	}
	verDoTurn(actor) = {}
	verDoTurnWith(actor, io) = {
		if (panel1.isopen) {
			"Not while <<panel1.thedesc>> is open. ";
		}
		if (io != screwdriver) {
			"You can't turn the screws with <<io.thedesc>>. ";
		}
	}
	doTurn(actor) = {
		askio(withPrep);
	}
	doScrew(actor) = {
		askio(withPrep);
	}
	verDoUnscrew(actor) = {
		if (!panel1.islocked) {
			"They've already been unscrewed. ";
		}
	}
	verDoUnscrewWith(actor, iobj) = {
		if (!panel1.islocked) {
			"They've already been unscrewed. ";
		}
	}
	doUnscrew(actor) = {
		askio(withPrep); 
	}
	verIoUseOn(actor) = {}
	ioUseOn(actor, dobj) = {
		if (panel1.islocked) {
			dobj.ioUnscrewWith(actor, self);
		} else {
			dobj.ioScrewWith(actor, self);
		}
	}
	verDoTake(actor) = "\^<<self.thedesc>> seem to be designed to remain attached even when unscrewed. "
;

keypad1: fixeditem
	noun = 'keypad' 'pad' 'keys'
	adjective = 'numeric' 'number'
	sdesc = "keypad"
	ldesc = {
		"You can see it's a pretty standard numeric keypad.  There are ten off-white plastic buttons, numbered 0 through 9. ";
		if ((!panel1.isbroken) && (panel1.badgeread)) {
			"The keys on the panel are currently glowing faintly. ";
		}
	}
	location = firstfloorhall
	verIoTypeOn(actor) = {
	}
	ioTypeOn(actor, dobj) = {
		local val;
		if (isclass(dobj, basicStrObj)) {
			if (cvtstr(cvtnum(dobj.value)) = dobj.value) {
				val := cvtnum(dobj.value);
			} else {
				"You can only type numbers on this keypad. ";
				return;
			}
		} else if (!isclass(dobj, basicNumObj)) {
			"You can only type numbers on this keypad. ";
			return;
		} else {
			val := dobj.value;
		}
		if ((panel1.isbroken) || (!panel1.badgeread)) {
			"Nothing happens. ";
			return;
		}
		if (val != personDB.getCode(securitybadge.person)) {
			"The panel emits a loud buzz and the keypad goes dark. ";
			panel1.badgeread := nil;
			return;
		}
		panel1.typecode;
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

grid1: fixeditem, surface, knownTopic
	noun = 'grid'
	adjective = 'scanning' 'brightly' 'illuminated' 'oddly' 'dark' 'scanner' 'sensor'
	sdesc = "scanning grid"
	location = firstfloorhall
	ldesc = {
		"The scanning grid is a ";
		if (panel1.isbroken)
			"dark";
		else
			"softly glowing";
		" piece of glass, four inches square.  There are a number of white lines etched into the glass's surface, forming a pattern of squares. ";
	}
	ioPutOn(actor, dobj) = {
		if (firstfloordoor.forced || panel1.isbroken || (dobj != securitybadge)) {
			"\^%You% touch <<dobj.thedesc>> to <<self.thedesc>> but nothing happens. ";
			panelHint.active := true;
			return;
		}
		panel1.readcard;
	}
	ioTouchTo(actor, dobj) = {
		if (dobj != securitybadge)
			"Nothing happens. ";
		else
			self.ioPutOn(actor, dobj);
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	verIoScanOn(actor) = {}
	verIoScanWith(actor) = {}
	ioScanOn(actor, dobj) = {
		self.ioPutOn(actor, dobj);
	}
	ioScanWith(actor, dobj) = {
		self.ioPutOn(actor, dobj);
	}
;

prong: fixeditem
	noun = 'prong' 'wires'
	adjective = 'metal' 'bundle' 'of'
	sdesc = "prong"
	location = panel1
	ldesc = "The prong is a piece of metal about a half-inch long and an eighth-inch in diameter.  One end is attached to a bundle of thin wires that connect to the inside of the panel. "
	verDoPutIn(actor, io) = {
		if (io != socket) {
			"You can't get the wires to stretch that far. ";
		}
	}
	verDoPlugIn(actor, io) = { self.verDoPutIn(actor, io); }
	verDoTake(actor) = "\^<<self.thedesc>> is attached to the panel by a bundle of wires. "
	verDoUnplug(actor) = "That seems unnecessary. "
;

socket: fixeditem
	noun = 'socket'
	sdesc = "socket"
	location = panel1
	ldesc = "The socket is nothing more than a metal-encircled hole in the panel's interior.  It is about an eighth of an inch in diameter. "
	isclogged = nil
	verIoPutIn(actor) = {}
	ioPutIn(actor, dobj) = {
		if (dobj = prong) {
			if (!panel1.isbroken) {
				"Really, you only need to do it once. ";
				return;
			}
			if (self.isclogged) {
				"You try to slide the prong into the socket, but the mass of clay in the socket prevents you. ";
				return;
			}
			"You plug the metal prong into the socket.  The scanner grid on the face of the panel immediately lights up. ";
			panel1.isbroken := nil;
			if (!global.scFixPanel) {
				incscore(1);
				global.scFixPanel := true;
			}
		} else if (dobj = clay) {
			if (!panel1.isbroken) {
				"The prong is in the way. ";
				return;
			}
			"You pack a lot of the clay into the socket. ";
			clay.moveInto(socket);
			self.isclogged := true;
		} else
			"That won't fit. ";
	}
	verGrab(obj) = {
		if (obj = clay) {
			"No matter how hard you try, you can't get the clay out of the socket. ";
		}
	}
	verIoPlugIn(actor) = { self.verIoPutIn(actor); }
	ioPlugIn(actor, dobj) = { self.ioPutIn(actor, dobj); }
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

firstfloorwest: room
	sdesc = "First Floor Hallway West"
	ldesc = {
		"The metal grating that serves for a floor is mostly undisturbed, with only the occasional muddy footprint heading in from the east.  Messages are scrawled on the wall in ink, paint, and a few other unidentified substances.  Three doors are available here, one in each wall. ";
	}
	specialdesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			return 'Whirling red lights and warning alarms fill the corridor.  Confused and directionless raiders run through the halls, searching vainly for the cause.  You seem to go completely unnoticed. ';
		}
		return '';
	}
	west = {
		"The automated door opens to let you pass.\b";
		return closet;
	}
	north = { "That door appears to have been jammed shut for years. "; return nil; }
	south = {
		"The automated door opens to let you pass.\b";
		return abandonedoffice;
	}
	east = firstfloorhall
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
		} else {
			"The hall is eerily quiet. ";
		}
	}
	exitlist = { "The hall continues to the east and there are doors to the north, south and west. "; return nil; }
	listContents = {
		"%You% see%s% "; listcont(self); " on the floor. ";
	}
;

closet: room
	sdesc = "Closet"
	ldesc = {
		"Your standard storage closet, complete with your standard ugly government regulation metal shelving units.  This one appears to see little use, as the few worn boxes sitting on the shelves are pretty much empty.  In the high ceiling above, you can see the dark shape of a vent shaft. ";
		if (pole.location = closetshelves) {
			"Hanging across the room, suspended by shelves, is a metal pole. ";
 		}
		"A single door leads east into the hall. ";
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if ((pole.location = closetshelves) && (itemcnt(pole.contents) <> 0)) {
			"Hanging from the pole is ";
			listcont(pole);
			". ";
			global.spacer := nil;
		}
		str := outcapture(stat);
		return str;
	}
	east = {
		"The automated door opens to let you pass.\b";
		return firstfloorwest;
	}
	enterRoom(actor) = {
		inherited.enterRoom(actor);
		if (!global.inbase) {
			global.inbase := true;
			incscore(1);
			global.scEnterBase := true;
			if (!global.insecond) {
				door1Hint.active := true;
			}
		}
	}
	up = { "Even if you could reach it, there aren't any handholds in that vent shaft. "; return nil; }
	exitlist = { "There is a single door to the east. "; return nil; }
	listContents = {
		"\^%you% see%s% "; listcont(self); " lying on the floor. ";
	}
	smelldesc = "A stale odor of paper. "
	scorchdesc = {
		if (closetshelves.grenadeblast)
			"Some of the shelves have been destroyed by the grenade's explosion. ";
		else
			pass scorchdesc;
	}

;

closetshelves: fixeditem, surface
	noun = 'shelves' 'shelf'
	adjective = 'metal' 'regulation' 'government'
	sdesc = "shelves"
	location = closet
	ldesc = {
		if (self.grenadeblast)
			"A large section of the shelves has been destroyed.  The adjacent edges of the remaining shelves are bent and scorched. ";
		if (pole.location = self) {
			"Stretching across the corner of the room, suspended by the shelves, is a metal pole. ";
			if (itemcnt(self.contents)) {
				"Also on "; self.thedesc; ", in addition to the boxes, %you% see%s% "; listcont(self); ". ";
			} else {
				"There's nothing else on "; self.thedesc; " except for the boxes. ";
			}
		} else {
			if (itemcnt(self.contents)) {
				"On "; self.thedesc; ", in addition to the boxes, %you% see%s% "; listcont(self); ". ";
			} else {
				"There's nothing on "; self.thedesc; " except for the boxes. ";
			}
		}
	}
	verDoClimb(actor) = {
		"You'd still be unable to reach the vent shaft. ";
	}
	verDoStandon(actor) = {
		"You'd still be unable to reach the vent shaft. ";
	}
	handleExplosion(actor, playerloc, heard) = {
		self.grenadeblast := true;
		if (pole.location = closetshelves) {
			pole.moveInto(closet);
			if (uniform1.location = pole)
				uniform1.moveInto(closet);
			if (uniform2.location = pole)
				uniform2.moveInto(closet);
		}
	}
;

pole: item, surface, iron, knownTopic
	noun = 'pole' 'rod' 'tube' 'end' 'ends'
	adjective = 'metal' 'iron'
	sdesc = "metal pole"
	location = closetshelves
	bulk = 15
	weight = 8
	ldesc = {
		"An iron pole that stretches about three feet long.  Feels like it's hollow, though the ends are closed. ";
		if (self.location = closetshelves) {
			"The pole is hanging across the room, suspended by two shelves. ";
			if ((uniform2.location = self) && (uniform1.location = self)) {
				"There are a frayed jacket and shabby coat hanging from it. ";
			} else {
				if (uniform2.location = self) {
					"There is a frayed jacket dangling from it. ";
				}
				if (uniform1.location = self) {
					"There is a shabby coat dangling from it. ";
				}
			}
		}
		if (magnet.location = self)
			"Attached to the end of <<self.thedesc>> is <<magnet.adesc>>. ";
	}
	isListed = {
		if (self.location = closetshelves)
			return nil;
		return true;
	}
	isVisible(vantage) = {
		pass isVisible;
	}
	moveInto(actor) = {
		if ((uniform2.location = self) && (uniform1.location = self)) {
			if (findRootRoom(self) = findRootRoom(parserGetMe()))
				"The frayed jacket and shabby coat slide off the pole onto the floor. ";
			uniform1.moveInto(closet);
			uniform2.moveInto(closet);
		} else {
			if (uniform2.location = self) {
				if (findRootRoom(self) = findRootRoom(parserGetMe()))
					"The frayed jacket slides off the pole onto the floor. ";
				uniform2.moveInto(closet);
			}
			if (uniform1.location = self) {
				if (findRootRoom(self) = findRootRoom(parserGetMe()))
					"The shabby coat slides off the pole onto the floor. ";
				uniform1.moveInto(closet);
			}
		}
		pass moveInto;
	}
	verIoTakeWith(actor) = {}
	verDoPutUnder(actor, io) = {}
	ioTakeWith(actor, dobj) = {
		if ((dobj = securitybadge) && (!desk1.gotbadge)) {
			desk1.ioPutUnder(actor, self);
		}
	}
	verIoPutOn(actor) = {
		local dobj := parserGetObj(PO_DOBJ);
		if (dobj = magnet)
			return;
		if (dobj = clay) {
			"You attempt to stick the lump of clay to the end of the pole, but it keeps sliding off. ";
			return;
		}
		if ((dobj != uniform1) && (dobj != uniform2)) {
			"There's no good surface on "; self.thedesc; ". ";
			return;
		}
		if (self.location != closetshelves)
			"\^<<dobj.thedesc>> would just slide off <<self.thedesc>>. ";
	}
	verDoAttachTo(actor, iobj) = {
		if (iobj != magnet)
			"There's no obvious way to do that. ";
		self.verDoPutOn(actor, iobj);
	}
	verIoAttachTo(actor) = {}
	ioAttachTo(actor, dobj) = {
		dobj.doPutOn(actor, self);
	}
	ioTouchTo(actor, dobj) = {
		if (dobj != magnet)
			"Nothing happens. ";
		else
			dobj.doPutOn(actor, self);
	}
	verDoClimb(actor) = {
		if (location = closetshelves) {
			"You'd still be unable to reach the vent shaft. ";
		} else {
			"That would be futile. ";
		}
	}
	verDoStandon(actor) = {
		if (location = closetshelves) {
			"That would be too precarious a perch. ";
		} else {
			"That would be futile. ";
		}
	}
	verIoPryWith(actor) = {}
	verIoBraceWith(actor) = {}
	ioBraceWith(actor, dobj) = {
		dobj.doBraceWith(actor, self);
	}
	verIoAttackWith(actor) = {}
	ioAttackWith(actor, dobj) = {
		dobj.doAttack(self);
	}
	doThrowAt(actor, io) =
	{
		if (io = gastank) {
			"Your aim goes a little high, knocking the regulator off of the top of the gas tank and releasing a jet of pressurized gas.  \^<<self.thedesc>> must have sparked as it ricocheted off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame. ";
			gastank.onfire := true;
			if ((leader.location = lab) && leader.isalive) {
				if (gastank.position = 1)
					"<<global.dispParagraph>>\"You missed--\" begins the raider, then seems to notice the burning gas tank beside him and turns towards it.  \"What the?\" ";
				else
					"<<global.dispParagraph>>The raider boss, across the room from the gas canister, seems to pause and takes a step back.  \"And what do you expect <I>that</I> to get you?\" ";
			}
			notify(gastank, &explode, 2);
			return;
		}
		"\^<<self.thedesc>> bounce<<self.verbs>> off of <<io.thedesc>> and fall<<self.verbs>> to the ground. ";
		self.moveInto(actor.location);
	}
;

uniform2: uniform, knownTopic
	noun = 'uniform' 'jacket'
	adjective = 'frayed'
	sdesc = "frayed jacket"
	location = pole
	bulk = 10
	weight = 8
	ldesc = {
		"A frayed jacket with alternating bands of red and green that you recognize as the wasteland equivalent of gang colors. ";
		if ((pole.location = pole) && (self.location = pole))
			"It's currently hanging from a metal pole. ";
	}
	isListed = {
		if ((pole.location = pole) && (self.location = pole))
			return nil;
		return true;
	}
	verDoWear(actor) = {
		if (uniform1.isworn)
			"\^<<self.thedesc>> won't fit over <<uniform1.thedesc>>. ";
		else if (radsuit.isworn)
			"\^<<self.thedesc>> won't fit over <<radsuit.thedesc>>. ";
		else
			pass verDoWear;
	}
	smelldesc = "You doubt that it has been washed any time in the recent past. "
	doBurn(actor) = {
		if (self.isworn) {
			"Not while you're wearing it. ";
			return;
		}
		"You toss the ratty piece of clothing onto the fire.  It quickly burns away. ";
		self.moveInto(nil);
	}
;

closetboxes: fixeditem
	sdesc = "boxes"
	noun = 'box' 'boxes'
	adjective = 'few' 'worn' 'cardboard'
	location = closet
	isThem = true
	contentsReachable = nil
	contentsVisible = nil
	ldesc = "A bunch of warped and water-damaged cardboard boxes.  Most have only a scattering of broken electronics, dead batteries, corroded wires, and other completely useless junk. "
	verDoSearch(actor) = {}
	doSearch(actor) = {
		if (circuitbreaker.location = nil) {
			"Searching among all the useless trash in the boxes, you come across a circuit breaker in reasonable condition, which you take. ";
			circuitbreaker.moveInto(actor);
		} else {
			"You can't find anything else useful in <<self.thedesc>>. ";
		}
	}
	verDoLookin(actor) = {}
	doLookin(actor) = {
		"A quick glance reveals nothing beyond a scattering of broken electronics, dead batteries, corroded wires, and other completely useless junk. ";
	}
	verDoTake(actor) = "\^<<self.thedesc>> would probably fall apart if you moved them. "
	smelldesc = "A stale odor of paper. "
;

circuitbreaker: switchItemNotFixed, hiddenItem
	sdesc = "circuit breaker"
	noun = 'breaker'
	adjective = 'circuit'
	bulk = 5
	ldesc = {
		"A small, thin block of plastic with a switch on the front and metal tabs on the back.  A circuit breaker, you know, fits into an electrical breaker box in order to complete the electrical circuit and allow power to flow.  In the event of an electrical overload, it switches itself off. ";
		if (self.location = breakerbox1 || self.location = breakerbox2) {
			"The breaker is currently in the breaker box and ";
			if (self.isActive) {
				"turned on. ";
			} else {
				"switched off. ";
			}
		} else {
			if (self.isActive) {
				"The breaker is currently on. ";
			} else {
				"The breaker is currently switched off. ";
			}
		}
	}
//	location = closetboxes
	doTurnon(actor) = {
		inherited.doTurnon(actor);
		if (location = breakerbox1) {
			"Almost immediately, the overhead lights begin to glow.  In a moment, they are at full strength. ";
			circuitbreaker.isActive := true;
			global.secondpower := true;
			if (!global.scSecondFloor) {
				incscore(1);
				global.scSecondFloor := true;
			}
			if (commissaryconsole.isbroken) {
				commissarydoor1.setIsopen(true);
				door2Hint.active := nil;
			}
			if (!secondfloordoor.isblocked)
				secondfloordoor.isopen := nil;
		}
		if (location = breakerbox2) {
			"Almost immediately, the overhead lights begin to glow.  In a moment, they are at full strength. ";
			powerHint.active := nil;
			global.thirdpower := true;
			circuitbreaker.isActive := true;
			if (!global.scThirdFloor) {
				incscore(1);
				global.scThirdFloor := true;
			}
		}
	}
	doTurnoff(actor) = {
		inherited.doTurnoff(actor);
		if (location = breakerbox1) {
			"The lights above go dark. ";
			global.secondpower := nil;
			circuitbreaker.isActive := nil;
		}
		if (location = breakerbox2) {
			"The lights above go dark. ";
			global.thirdpower := nil;
			circuitbreaker.isActive := nil;
		}
	}
	takeMessage = {
		if ((location != breakerbox1) && (location != breakerbox2))
			"Taken. ";
	}
;

abandonedoffice: room
	sdesc = "Abandoned Office"
	ldesc = {
		if (self.grenadeblast) {
			if (desk1.grenadeblast) {
				"The only feature in the room, the massive wooden desk, has been reduced to kindling by a violent explosion.  There are splinters and fragments scattered all over the room, visible through the fading haze and lingering smoke.  The door to the north is the only exit. ";
			} else {
				"Very little remains in this dimly lit office beyond the massive wooden desk in the corner.  In the center of the room is a small crater, surrounded by blackened scorch marks that radiate outwards.  The edge of the desk facing this crater is mangled and charred.  North is the only exit. ";
			}
		} else {
			"Very little remains in this dimly lit office and the dust you've stirred up suggests it hasn't seen any real use in quite some time.  The massive wooden desk in the corner, except for a few dust-free outlines, is layered in a thick coating of the stuff. ";
			if (itemcnt(self.contents) < 1) {
				"Beyond it, nothing has been left in the room: not even a chair. ";
			}
			"The door to the north is the only exit. ";
		}
	}
	north = {
		"The automated door opens to let you pass.\b";
		return firstfloorwest;
	}
	exitlist = { "There is a single door to the north. "; return nil; }
	smelldesc = {
		if (self.grenadeblast)
			"The tang of smoke is inescapable. ";
		else
			"You sneeze violently. ";
	}
	handleExplosion(actor, playerloc, heard) = {
		scorchmark.moveInto(nil);
	}
	listContents = {
		if (desk1.grenadeblast) {
			"You can see "; listcont(self); " among the scattered bits of desk. ";
			return;
		}
		pass listContents;
	}
;

desk1: fixeditem, chairitem
	noun = 'desk' 'leg' 'legs'
	adjective = 'wood' 'bulky' 'wooden'
	location = abandonedoffice
	sdesc = "wooden desk"
	isopen = nil
	gotbadge = nil
	bulkthresh = 40
	maxbulk = 40
	ldesc = {
		if (self.grenadeblast) {
			"There's not much left of the desk.  It's been completely blown apart.  The largest piece left is no bigger than your head. ";
		} else {
			"This is a bulky office desk built from some kind of dark hardwood.  Its appearance is particularly ugly and utilitarian.  The top is covered with dust except for a few outlines of items that have probably been recently removed.  The left side consists of a large sliding drawer and the desk's very short legs keep it only about an inch off the floor. ";
			if (location.grenadeblast)
				"Both visible sides are pockmarked and have been blackened by a terrible heat. ";
			if (itemcnt(self.contents) <> 0)
				"Looking the top of the desk, you can see <<listcont(self)>>. ";
		}
	}
	nomovemsg = {
		if (self.grenadeblast)
			"Too many splinters; too little use. ";
		else
			"There's no way you're moving this monster of a desk. ";
	}
	verDoMove(actor) = { self.nomovemsg; }
	verDoMoveWith(actor, io) = { self.nomovemsg; }
	verDoPush(actor) = { self.nomovemsg; }
	verDoTake(actor) = { self.nomovemsg; }
	genMoveDir = { self.nomovemsg; }
	verIoPutOn(actor) = {
		if (self.grenadeblast)
			"The desk is little more than splinters now. ";
	}
	verDoLookunder(actor) = {
		if (self.grenadeblast) {
			"You look under a few pieces of the desk, but find nothing. ";
			return;
		}
		if (self.gotbadge)
			pass verDoLookunder;
	}
	doLookunder(actor) = {
		if (!self.gotbadge) {
			"You peer under the desk and see the glint of something metallic.  Unfortunately, the desk is too low to the ground to reach it. ";
			objectHint.active := true;
		}
	}
	verIoPutUnder(actor) = {
		if (self.grenadeblast)
			"There's nothing left of the desk worth dealing with. ";
	}
	ioPutUnder(actor, dobj) = {
		if (dobj = pole) {
			if (magnet.location = dobj) {
				"You crouch down and slide the pole under the desk towards the metallic object beneath.  Whatever it is sticks to the magnet at the end of the pole, and you pull the pole out to find a security badge dangling from the end, its belt clip stuck to the magnet. ";
				securitybadge.moveInto(actor);
				self.gotbadge := true;
				photo.moveInto(securitybadge);
				incscore(1);
				global.scFoundBadge := true;
				objectHint.active := nil;
			} else {
				"You crouch down and try to snag the metallic object under the desk with the metal pole.  You can't seem to catch it, though, and after a minute you give up. ";
			}
		} else if (dobj = magnet) {
			if (magnet.location = pole) {
				"You crouch down and slide the pole under the desk towards the metallic object beneath.  Whatever it is sticks to the magnet at the end of the pole, and you pull the pole out to find a security badge dangling from the end, its belt clip stuck to the magnet. ";
				securitybadge.moveInto(actor);
				self.gotbadge := true;
				photo.moveInto(securitybadge);
				incscore(1);
				global.scFoundBadge := true;
				objectHint.active := nil;
			} else {
				"You can't reach the object with the magnet.  It's too far underneath the desk. ";
			}
		} else if (dobj.ispaper) {
			"You slide <<dobj.thedesc>> under the desk but you can't reach the metallic object, so you pull it back out. ";
		} else {
			"There isn't room. ";
		}
	}
	doLookin -> deskdrawer
	verDoSearch(actor) = {
		if (self.grenadeblast)
			"You pick through the remains of the desk but find nothing. ";
	}
	doSearch(actor) = {
		if (itemcnt(self.contents) <> 0)
			"Searching the top of the desk, you find <<listcont(self)>>. ";
		else
			"Searching the top of the desk, you find nothing except dust. ";
		if (!deskdrawer.isopen) {
			"The desk drawer is currently closed. ";
		} else {
			if (itemcnt(deskdrawer.contents) <> 0)
				"In the desk drawer %you% see%s% <<listcont(self)>>. ";
			else
				"The drawer is empty. ";
		}
		if (!self.gotbadge) {
			"You peer under the desk and see the glint of something metallic.  Unfortunately, the desk is too low to the ground to reach it. ";
			objectHint.active := true;
		}
	}
	doOpen -> deskdrawer
	doClose -> deskdrawer
	ioPutIn -> deskdrawer
	/*
	verDoSiton(actor) = {
		if (self.grenadeblast)
			"There's nothing left of the desk worth dealing with. ";
		else
			"And disturb all that dust?  You'd be sneezing for hours. ";
	}
	*/
	verDoStandon(actor) = {}
	doStandon(actor) = {
	        "Okay, %you're% now standing on "; self.thedesc; ". ";
	        actor.travelTo(self);
	}
	smelldesc = {
		if (self.grenadeblast)
			"The tang of smoke is inescapable. ";
		else
			"You sneeze violently. ";
	}
	handleExplosion(actor, playerloc, heard) = {
		local o;
		self.gotbadge := true;
		photo.moveInto(securitybadge);
		self.location.grenadeblast := true;
		scorchmark.moveInto(nil);
		while (length(self.contents) > 0) {
			o := car(self.contents);
			o.moveInto(nil);
		}
		while (length(deskdrawer.contents) > 0) {
			o := car(deskdrawer.contents);
			o.moveInto(abandonedoffice);
		}
		deskdrawer.moveInto(nil);
	}
;

deskdrawer: fixeditem, openable
	sdesc = "desk drawer"
	noun = 'drawer' 'drawers'
	adjective = 'desk' 'large' 'sliding'
	location = abandonedoffice
	ldesc = {
		"The drawer is a simple deep sliding box on a track.  It is currently ";
		if (self.isopen) {
			if (self.contentsVisible and itemcnt(self.contents) <> 0) {
				"open.  Inside %you% can see "; listcont(self); ". ";
			} else {
				"open and empty. ";
			}
		} else {
			" closed. ";
		}
	}
	isopen = nil
	bulkthresh = 10
	maxbulk = 10
	doSearch -> desk1
	verDoLookin(actor) = {
		if (self.grenadeblast) {
			"You pick through the remains of the desk but find nothing. ";
			return;
		}
		/* we can look in it if either it's open or it's transparent */
		if (not self.isopen and not isclass(self, transparentItem))
			"The desk drawer is closed. ";
	}
	doLookin(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0)
			"In <<self.thedesc>> %you% see%s% <<listcont(self)>>. ";
		else
			"There's nothing in <<self.thedesc>>. ";
	}
	verDoOpen(actor) = {
		if (self.grenadeblast)
			"There's nothing left of the desk worth dealing with. ";
	}
	doOpen(actor) = {
		self.isopen := true;
		"You slide the desk drawer open";
		if (self.contentsVisible and itemcnt(self.contents) <> 0) {
			", revealing <<listcont(self)>>";
		} else {
			" and find it empty";
		}
		". ";
	}
	verDoClose(actor) = {
		if (self.grenadeblast)
			"There's nothing left of the desk worth dealing with. ";
	}
	doClose(actor) = {
		"You slide the desk drawer closed. ";
		self.isopen := nil;
	}
	handleExplosion(actor, playerloc, heard) = {
		local o;
		desk1.gotbadge := true;
		photo.moveInto(securitybadge);
		desk1.location.grenadeblast := true;
		desk1.grenadeblast := true;
		scorchmark.moveInto(nil);
		while (length(desk1.contents) > 0) {
			o := car(desk1.contents);
			o.moveInto(abandonedoffice);
		}
		while (length(deskdrawer.contents) > 0) {
			o := car(deskdrawer.contents);
			o.moveInto(nil);
		}
		deskdrawer.moveInto(nil);
	}
;

securitybadge: item, clothingItem, iron, readable, knownTopic
	noun = 'badge' 'something' 'object' 'card'
	adjective = 'security' 'identification' 'id' 'metallic' 'glint'
	location = abandonedoffice
	person = 0
	bulk = 2
	weight = 1
	sdesc = { 
		if (desk1.gotbadge)
			"security badge";
		else
			"metallic object";
	}
	ldesc = {
		if (desk1.gotbadge) {
			"A white plastic security badge with a clip at the top designed to attach to a belt or shirt.  It lacks any kind of magnetic strip.  It's likely the kind with the microchip inside that you put next to a sensor in order to activate.  The name on the badge reads: <<personDB.getFullNameAndRank(self.person)>>.  Below that, there's a grainy photo. ";
		} else {
			"It's difficult to tell what it is, even with your flashlight.  All you can see is the glinting of light off of some metallic object. ";
		}
	}
	readdesc = {
		"The name on the badge reads: <<personDB.getFullNameAndRank(self.person)>>. ";
	}
	initialize = {
		person:=rand(6);
	}
	isListed = {
		return desk1.gotbadge;
	}
	moveInto(room) = {
		delword(self, &noun, 'something');
		delword(self, &adjective, 'glint');
		delword(self, &adjective, 'metallic');
		pass moveInto;
	}
	verDoTake(actor) = {
		if ((actor.location = self.location) && !desk1.gotbadge) {
			"Your arm is too large to fit under the desk. ";
		} else {
			pass verDoTake;
		}
	}
	verifyRemove(actor) = {
		if (desk1.gotbadge)
			pass verifyRemove;
		"Your arm is too large to fit under the desk. ";
	}
	verDoPush(actor) = {
		if (desk1.gotbadge)
			pass verDoPush;
		"Your arm is too large to fit under the desk. ";
	}
	verDoPushWith(actor, iobj) = {
		if (desk1.gotbadge)
			pass verDoPushWith;
		if (iobj != pole) {
			"\^<<iobj.thedesc>> can't reach it. ";
			return;
		}
		"That would just push it closer to the wall. ";
	}
	verDoPull(actor) = {
		if (desk1.gotbadge)
			pass verDoPull;
		"Your arm is too large to fit under the desk. ";
	}
	verIoThrowAt(actor) = {
		if (!desk1.gotbadge)
			"The desk is far too close to the ground. ";
		else
			pass verIoThrowAt;
	}
	verDoTouch(actor) = {
		if (desk1.gotbadge)
			pass verDoTouch;
		"Your arm is too large to fit under the desk. ";
	}
	verDoTurn(actor) = {
		if (desk1.gotbadge)
			pass verDoTurn;
		"Your arm is too large to fit under the desk. ";
	}
	verDoMove(actor) = {
		if (desk1.gotbadge)
			pass verDoMove;
		"Your arm is too large to fit under the desk. ";
	}
	verDoLookbehind(actor) = {
		if (desk1.gotbadge)
			pass verDoLookbehind;
		"You can't see it well enough. ";
	}
	verDoLookunder(actor) = {
		if (desk1.gotbadge)
			pass verDoLookunder;
		"You can't see it well enough. ";
	}
	verDoPoke(actor) = {
		if (desk1.gotbadge)
			pass verDoPoke;
		"Your arm is too large to fit under the desk. ";
	}
	genMoveDir = {
		if (desk1.gotbadge)
			pass genMoveDir;
		"Your arm is too large to fit under the desk. ";
	}
	ioTouchTo(actor, dobj) = {
		if (dobj = panel1)
			panel1.ioPutOn(actor, self);
		else if (dobj = panel2)
			panel2.ioPutOn(actor, self);
		else
			"Nothing happens. ";
	}
	doScan(actor) = {
		if (actor.location = firstfloorhall) {
			panel1.ioPutOn(actor, self);
		} else if (actor.location = secondfloorhall) {
			panel2.ioPutOn(actor, self);
		} else {
			askio(withPrep);
		}
	}
	verDoAttachTo(actor, iobj) = {
		if (!isclass(iobj, clothingItem)) {
			"It won't clip on. ";
		} else if (!iobj.isworn) {
			"But you aren't wearing <<iobj.thedesc>>. ";
		}
	}
	verIoScrewWith(actor) = {
		"The plastic edges of the badge just won't engage the screws. ";
	}
	verIoUnscrewWith(actor) = {
		"The plastic edges of the badge just won't engage the screws. ";
	}
;

photo: fixeditem
	sdesc = "photo"
	noun = 'photo' 'photograph' 'picture'
	adjective = 'grainy'
	location = nil
	ldesc = {
		"A ";
		if (personDB.gender[securitybadge.person] = 'M')
			"man";
		else
			"woman";
		"'s face looks back at you. ";
	}
;

firstflooreast: room
	sdesc = "First Floor Hallway East"
	step = 0
	ldesc = {
		"The hallway widens out slightly, expanding several feet to create alcoves to the north and south before constricting again after a span of several strides.  At the far end of each alcove is smooth, featureless door.  Muddy prints indicates that a regular stream of foot traffic passes along this east-west corridor.  More graffiti adorns these walls";
		if (!self.grenadeblast)
			", and there's a tall wooden bookcase against one wall, partially concealing the northern alcove";
		". ";
		if (crowd.isAlive) {
			"From the east comes a din of numerous voices. ";
		}
	}
	specialdesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			return 'Whirling red lights and warning alarms fill the corridor.  Confused and directionless raiders run through the halls, searching vainly for the cause.  You seem to go completely unnoticed. ';
		}
		return '';
	}
	west = firstfloorhall
	east = barracksdoorway
	north = {
		if (firstflooreast.grenadeblast) {
			"You hear a grinding noise coming from the automatic door.  Apparently its mechanism has decayed beyond repair. ";
		} else {
			"You slip past the bookcase into the northern alcove.  From this location, you are almost completely hidden from the hallway. ";
			parserGetMe().moveInto(northalcove);
		}
		return nil;
	}
	south = { "You approach the automatic door but nothing happens.  It doesn't seem to be operational anymore. "; return nil; }
	enterRoom(actor) = {
		local count;
		inherited.enterRoom(actor);
		if (!global.inbarracks) {
			raidersHint.active := true;
		}
		if (global.heardpassword)
			return;
		step := 0;
		count := getfuse(self, &activate);
		if (count = nil && firstflooreast.step = 0)
			notify(self, &activate, rand(2)+1);
	}
	activate = {
		if (firstflooreast.step = 0)
			setdaemon(hallDaemon, 0);
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		if (firstflooreast.step > 1) {
			"It sounds like there are two men in the hall. ";
		} else {
			if (crowd.isAlive)
				"There are a lot of voices coming from the room to your east. ";
			else
				"There is eerie silence. ";
		}
	}
	exitlist = { "The hall continues west and there are doors to the north, south and east. "; return nil; }
	handleExplosion(actor, playerloc, heard) = {
		bookcase.moveInto(nil);
	}
;

barracksdoorway: doorway
	location = firstflooreast
	isopen = true
	lasttried = -5
	doordest = barracks
	destination = {
		if (!wearingUniform(parserGetMe())) {
			if (global.turnsofar - self.lasttried>5) {
				"You pause for a moment, the voices giving you a sense of dread. ";
				self.lasttried := global.turnsofar;
				return nil;
			}
		}
		pass destination;
	}
;

// The east hall has north and south alcoves.  The player can enter either
// one, but only the north alcove is concealed (by the bookcase).
class alcove: nestedroom, fixeditem 
	plural = 'alcoves'
	reachable = ([] + self)
	canReachContents = true
	verDoEnter(actor) = {
		if (actor.location = self) {
			"%You're% already <<self.statusPrep>> <<self.thedesc>>! ";
		} else if (actor.isCarrying(self)) {
			"%You%'ll have to drop <<self.thedesc>> first!";
		}
	}
	doEnter(actor) = {
		"Okay, %you're% now <<self.statusPrep>> <<self.thedesc>>. ";
		actor.moveInto(self);
	}
;

northalcove: alcove
	noun = 'alcove' 'north' 'n' 'northern'
	adjective = 'north' 'n' 'northern'
	sdesc = "north alcove"
	location = firstflooreast
	reachable = ([] + self + bookcase + scenerydoor9)
	ldesc = {
		if (firstflooreast.grenadeblast) {
			"The northern wall of the corridor is recessed, forming a small alcove.  On the north end of the alcove is a smooth, featureless door. ";
		} else {
			"The northern wall of the corridor is recessed, forming a small alcove.  A wooden bookcase set against that wall sticks out into the alcove, partially obscuring it from the hall.  On the north end of the alcove is a smooth, featureless door. ";
		}
	}
	statusPrep = "in"
	outOfPrep = "out from"
	north = { "You hear a grinding noise coming from the automatic door.  Apparently its mechanism has decayed beyond repair. "; return nil; }
	south = {
		if (firstflooreast.grenadeblast) {
			"You step out of the alcove, back into the hallway. ";
		} else {
			"You step out from behind the bookcase, back into the hallway. ";
		}
		parserGetMe().moveInto(firstflooreast);
		return nil;
	}
	out = {
		if (firstflooreast.grenadeblast) {
			"You step back into the hallway. ";
		} else {
			"You step out from behind the bookcase. ";
		}
		parserGetMe().moveInto(firstflooreast);
		return nil;
	}
	roomAction(a, v, d, p, i) = {
		if ((v = yellVerb) && ((firstflooreast.step > 1) && (firstflooreast.step < 5))){
			"You shout loudly.  However, the gunshots that follow are even louder.  Apparently the men in the hall are a bit twitchy.  The dry rotted wood of the bookcase does little to slow the bullets and they have more than enough speed to prove fatal. ";
			lose(true);
		}
		pass roomAction;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		if (firstflooreast.step > 1) {
			"It sounds like there are two men in the hall. ";
		} else {
			if (crowd.isAlive)
				"There are a lot of voices coming from the room to your east. ";
			else
				"There is eerie silence. ";
		}
	}
	verDoHidebehind(actor) = {
		if (actor.location = self)
			"You're already in <<self.thedesc>>. ";
	}
	verDoHidein(actor) = {
		self.verDoHidebehind(actor);
	}
	doHidebehind(actor) = {
		self.doEnter(actor);
	}
	doHidein(actor) = {
		self.doEnter(actor);
	}
	hideMessage = {
		if (firstflooreast.grenadeblast)
			"Okay, %you're% now <<self.statusPrep>> <<self.thedesc>>. ";
		else
			"You slip past the bookcase into the northern alcove.  From this location, you are almost completely hidden from the hallway. ";
	}
	doEnter(actor) = {
		"<<self.hideMessage>>";
		actor.moveInto(self);
	}
	noexit = "You can move south back into the hallway and there is a door to the north. ";
;

southalcove: alcove
	noun = 'alcove' 'south' 's' 'southern'
	adjective = 'south' 's' 'southern'
	sdesc = "south alcove"
	location = firstflooreast
	reachable = ([] + self + scenerydoor10)
	ldesc = "The southern wall of the corridor is recessed, forming a small alcove.  On the south end of the alcove is a smooth, featureless door. "
	noexit = "You can move north back into the hallway and there is a door to the south. "
	north = {
		"You step back into the hallway. ";
		parserGetMe().moveInto(firstflooreast);
		return nil;
	}
	out = {
		"You step back into the hallway. ";
		parserGetMe().moveInto(firstflooreast);
		return nil;
	}
	verDoHidebehind(actor) = {
		if (actor.location = self)
			"You're already in <<self.thedesc>>. ";
	}
	verDoHidein(actor) = {
		self.verDoHidebehind(actor);
	}
	doHidebehind(actor) = {
		"You step into the south alcove, though you aren't really hidden. ";
		actor.moveInto(self);
	}
	doHidein(actor) = {
		self.doHidebehind(actor);
	}
	verDoExit(actor) = {}
	doExit(actor) = {
		"You step back into the hallway. ";
		actor.moveInto(firstflooreast);
		return nil;
	}
;

// Hiding behind the bookcase shuffles the player into the north alcove.
bookcase: hideitem
	noun = 'bookshelf' 'shelf' 'case' 'bookcase' 'shelves'
	adjective = 'wooden' 'tall'
	sdesc = "bookcase"
	ldesc = "Reaches about six feet tall and spans around three feet across.  The wood is extremely old, and probably has only survived this long because it's been in the dry, sterile air of the base.  Most of the shelves are missing, possibly used for firewood.  The majority of the backing still remains, however, obscuring the view of the northern alcove from the hall. "
	location = firstflooreast
	isListed = nil
	hideMessage = {
		"You slip past the bookcase into the northern alcove.  From this location, you are almost completely hidden from the hallway. ";
	}
	verDoMove(actor) = {
		"As you start to push on it, the wood cracks under your fingers.  It looks like the rotted wood might completely fall apart if you tried to move it. ";
	}
	verDoHidebehind(actor) = {
		if (actor.location = northalcove)
			"You're already behind <<self.thedesc>>. ";
	}
	doHidebehind(actor) =
	{
		"<<self.hideMessage>> ";
		actor.moveInto(northalcove);
	}
	ioHideBehind(actor, dobj) = {
		if (addbulk(self.contents) + dobj.bulk > self.maxbulk)
		{
			"There's not enough room behind "; self.thedesc; ". ";
		} else {
			dobj.doPutIn(actor, northalcove);
		}
	}
;

// Controls the sounds and messages of the two wandering raiders.  They
// are not represented by actual objects.  This is all smoke and mirrors.
hallDaemon: function(val) {
	local hidden;
	hidden := (parserGetMe().location = northalcove);
	if (firstflooreast.grenadeblast)
		hidden := nil;
	if (!hidden && (findRootRoom(parserGetMe()) != firstflooreast)) {
		firstflooreast.step := 0;
		remdaemon(hallDaemon, 0);
		return;
	}
	if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
		return;
	}
	if (!crowd.isAlive)
		return;
	switch (firstflooreast.step) {
		case 0:
			"<<global.dispParagraph>>Over the sound of voices to the east, you can just make out what sounds like footsteps and voices approaching the hall. ";
			break;
		case 1:
			if (hidden) {
				"<<global.dispParagraph>>From behind the bookcase, you hear what sounds like two men enter the room. ";
			} else {
				if (!wearingUniform(parserGetMe())) {
					"<<global.dispParagraph>>Two armed men enter the room and their eyes widen as they see you.  Immediately, their guns are thundering and you fall in a hail of bullets. ";
					lose(true);
				} else {
					"<<global.dispParagraph>>Two armed men enter the room, engrossed in a quiet conversation that you cannot quite make out.  One of them sees you and immediately stops talking while elbowing his companion.  That man quickly falls silent as well, and the two hurry past you. ";
				}
				firstflooreast.step := 0;
				remdaemon(hallDaemon, 0);
			}
			break;
		case 2:
			if (hidden) {
				"<<global.dispParagraph>>\"Boss is pissed,\" you hear a voice say.<<global.dispParagraph>>\"Yeah?\" a second voice replies.  \"Why's that?\" ";
			} else {
				if (!wearingUniform(parserGetMe())) {
					"<<global.dispParagraph>>The two armed men in the hall widen their eyes as they see you.  Immediately, their guns are thundering and you fall in a hail of bullets. ";
					lose(true);
				} else {
					"<<global.dispParagraph>>As you emerge from behind the bookcase, you find yourself face to face with two armed and angry-looking men.  \"Stupid recruits,\" one of them mutters, giving you a dark look.  The two men hastily depart. ";
				}
				firstflooreast.step := 0;
				remdaemon(hallDaemon, 0);
			}
			break;
		case 3:
			if (hidden) {
				"<<global.dispParagraph>>The first voice says, \"He thinks Jones is working to take over.  Boss changed the password early.\"<<global.dispParagraph>>\"Oh?  What is it now?\"<<global.dispParagraph>>\"";
				"\^<<confguard.getpassword>>.";
				"\" ";
				global.heardpassword := true;
			} else {
				if (!wearingUniform(parserGetMe())) {
					"<<global.dispParagraph>>As you emerge from behind the bookcase, you find yourself face to face with two armed and angry-looking men.  Unfortunately for you, they don't take kindly to intruders. ";
					lose(true);
				} else {
					"<<global.dispParagraph>>As you emerge from behind the bookcase, you find yourself face to face with two armed and angry-looking men.  \"Stupid recruits,\" one of them mutters, giving you a dark look.  The two men hastily depart. ";
				}
				firstflooreast.step := 0;
				remdaemon(hallDaemon, 0);
			}
			break;
		case 4:
			if (hidden) {
				"<<global.dispParagraph>>There's a pause.  \"Is it true?\" the second voice asks.  \"Is Jones?\"<<global.dispParagraph>>\"I doubt it,\" says the first voice.  \"He's just been....\"  The voices trail off as the men move out of earshot. ";
			} else {
				if (!wearingUniform(parserGetMe())) {
					"<<global.dispParagraph>>As you emerge from behind the bookcase, you find yourself face to face with two armed and angry-looking men.  Unfortunately for you, they don't take kindly to intruders. ";
					lose(true);
				} else {
					"<<global.dispParagraph>>As you emerge from behind the bookcase, you find yourself face to face with two armed and angry-looking men.  \"Stupid recruits,\" one of them mutters, giving you a dark look.  The two men hastily depart. ";
				}
				remdaemon(hallDaemon, 0);
			}
			remdaemon(hallDaemon, 0);
			firstflooreast.step := 0;
			break;
	}
	firstflooreast.step++;
}

barracks: room
	sdesc = "Barracks"
	ldesc = {
		if (!crowd.isAlive) {
			"Whatever purpose this room originally served, it's now been reduced to a gruesome battlefield.  There are about three dozen bloody beds, cots, and mattresses spread across the red-streaked tile floor, and any number of personal possessions leaning against or hanging on the bullet-ridden walls.  The corpses of some two dozen men lie in various locations around the room.  Some clutch recently-fired weapons, others lie in pools of blood with surprise looks on their faces. ";
		} else {
			"Whatever purpose this room originally served, it now functions as a massive brigand encampment.  There are about three dozen makeshift beds spread across the blue tile floor, and jumbled piles of personal possessions scattered about. ";
			if (global.venttime = 0) {
				"The raiders, a large crowd of dirty, unpleasant-looking men, are each dressed in similar colors of red and green and all appear to be armed.  The men are engaged in all manner of activities, from games of skill to maintaining weapons to eating to gambling.  ";
			}
		}
		"Embedded in the western wall, beside one of the bunks, is a lone metal panel of brushed stainless steel.  The doorway west leads back into the hallway, while a second exit lies to the south. ";
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"Whirling red lights and warning alarms fill the room.  Confused and directionless raiders run through the halls, searching vainly for the cause.  You seem to go completely unnoticed. ";
			if ((itemcnt(board.contents) > 0) || (magnet.location = board)) {
				"<<global.dispParagraph>>";
			}
		}
		if (itemcnt(board.contents)) {
			"Hanging from "; board.thedesc;" is "; listcont(board);
			" being held on by a kitchen magnet. ";
		} else if (magnet.location = board) {
			"Hanging from "; board.thedesc;" is <<magnet.adesc>>. ";
		}
		str := outcapture(stat);
		return str;
	}
	enterRoom(actor) = {
		inherited.enterRoom(actor);
		if ((!wearingUniform(parserGetMe()) || !leader.isalive) && (global.venttime = 0)) {
			"<<global.dispParagraph>>Several of the men shout \"Intruder!\" as you enter the room.  From all around the room, men draw their weapons and fire.  The fusillade of bullets is impossible to avoid, and your blood soon stains the tile floor. ";
			lose(true);
		}
		global.inbarracks := true;
		raidersHint.active := nil;
		crowd.skip := true;
		if (crowd.count>45)
			crowd.count := crowd.count - 5;
	}
	west = firstflooreast
	south = conference
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		if (crowd.isAlive)
			"You can barely hear yourself think over the noise of this crowd. ";
		else
			"There is eerie silence. ";
	}
	exitlist = { "The only exits are west and south. "; return nil; }
	listContents = {
		"On the floor %you% can see%s% "; listcont(self); ". ";
	}
	smelldesc = {
		if (crowd.isAlive) {
			"You try to hide your grimace. ";
		} else {
			"Gunpowder and blood.  In copious amounts. ";
		}
	}
	roomAction(a, v, d, p, i) = {
		if (crowd.isAlive) {
			if (v = yellVerb || v = jumpVerb) {
				"It's probably best not to draw attention to yourself unless you really have to. ";
				exit;
			}
		}
		pass roomAction;
	}
;

crowd: Actor, knownTopic
	noun = 'man' 'raider' 'bandit'
	plural = 'crowd' 'men' 'raiders' 'bandits' 'group'
	adjective = 'large' 'dirty' 'unpleasant-looking' 'unpleasant'
	sdesc = "raiders"
	location = barracks
	isListed = nil
	showActorDesc = nil
	actorDesc = nil
	isHim = true
	isThem = true
	isAlive = true
	ldesc = {
		if (self.isAlive)
			"Villains of the waste.  You've encountered raiders before, but never this up close and personal.  At a glance you'd say there are a couple dozen men here, maybe more.  Most are dirty, unshaven, with unkempt hair and ratty clothes.  All are armed in one manner or another.  Each one has a jacket, shirt, or armband with alternating bands of red and green.  You've seen the like before.  Wasteland raiders, like Arcadian gangs, use these colors to announce their allegiance as well as identify their companions. ";
		else
			"Over two dozen, all dead.  Which makes you the luckiest son of a bitch in the wastelands. ";
	}
	verDoTake(actor) = "You find yourself overestimating your physique. "
	badidea = {
		if (crowd.isAlive)
			"It's probably best not to draw attention to yourself unless you really have to. ";
		else
			"They're all dead.  What would be the point? ";
	}
	actorAction(v, d, p, i) = {
		self.badidea;
		exit;
	}
	verDoFollow(actor) = self.badidea
	verDoTellAbout(actor, iobj) = self.badidea
	verDoAskAbout(actor, iobj) = self.badidea
	verIoGiveTo(actor, dobj) = self.badidea
	verIoShowTo(actor, dobj) = self.badidea
	verDoPoke(actor) = self.badidea
	doHug(actor) = self.badidea
	doTalkto(actor) = self.badidea
	disavow = self.badidea
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		local num;
		if (!self.isAlive) {
			"They're already dead.  They really can't get any deader. ";
			return;
		}
		if (weapon = pistol) {
			inherited.doAttack(weapon);
			"Selecting a raider at random, you raise your pistol and fire off a shot.  He cries out and tumbles to the floor.  Even though the pistol is silenced, it is quite clear where the shot came from.  The rest of the raiders spare no time in avenging their fallen comrade. ";
			lose(true);
		}
		if (weapon = rifle) {
			inherited.doAttack(weapon);
			num := rifle.rounds;
			if (num=0) {
				"The gun thunders in your hands and a nearby raider falls.  Grinning wildly, you point the rifle at another shocked man and pull the tripper.  The fall of the hammer on an empty chamber is nearly deafening in the room's silence.  The return fire, though, seems oddly distant. ";
				lose(true);
			} else if (num=1) {
				"The gun thunders in your hands and a nearby raider falls.  Grinning wildly, you drop another man to the ground with a second -- and final -- shot.  The empty rifle is still clutched tightly in your hands when the raiders stop shooting into your mutilated corpse. ";
				lose(true);
			} else if (num=2) {
				"The gun thunders in your hands.  Three shots and two raiders fall.  Not too shabby.  You soon, however, join them on the blood-soaked floor. ";
				lose(true);
			} else if (num=19) {
				"Time seems to slow down as you raise the rifle and fire at your target: one of the men holding what appears to be a ready weapon.  As he goes down and the shocked raiders look on, you choose your next victim.  The rifle booms and he falls as well.  The second shot seems to break the others from their paralysis and they grab their weapons.  Boom, boom.  Two more bandits fall before the return fire begins.  You dive to the side, sliding along slick tile floor, firing your rifle at the panicking men.  Their wild shots whiz by your ear, slam into the wall, and crash into the floor around you.  Miraculously, though, you remain unhurt and you pump bullet after bullet into these wasteland ruffians.<<global.dispParagraph>>\"Click\"<<global.dispParagraph>>Your hand is trembling as it falls away from the empty rifle.  Somehow... somehow, despite all logic and reason telling you it is impossible, your twenty round weapon managed to kill over two dozen armed men. ";
				rifle.rounds := 0;
				self.isAlive := nil;
				confguard.moveInto(nil);
				phantomguard.moveInto(nil);
				confguard.gone := true;
				grenadelauncher.moveInto(nil);
				raiderweapons.moveInto(barracks);
			} else if (num>2) {
				num := num - 1;
				"Time seems to slow down as you raise the rifle and fire at your target: one of the men holding what appears to be a ready weapon.  As he goes down and the shocked raiders look on, you choose your next victim.  The rifle booms and he falls as well.  The second shot seems to break the others from their paralysis and they grab their weapons.  Boom, boom.  Two more bandits fall before the return fire begins.  You dive to the side, sliding along slick tile floor, firing your rifle at the panicking men.  Their wild shots whiz by your ear, slam into the wall, and crash into the floor around you.  Miraculously, though, you remain unhurt and you pump bullet after bullet into these wasteland ruffians.<<global.dispParagraph>>\"Click\"<<global.dispParagraph>>Your hand is trembling as it falls away from the empty rifle.  Over a dozen men lie dead on the blood-soaked floor.  About a dozen more, however, leap towards you, weapons blazing. ";
				lose(true);
			}
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			inherited.doAttack(weapon);
			"Selecting a raider at random, you lunge forward and make your attack.  He cries out and tumbles to the floor, but the rest of the raiders spare no time in avenging their fallen comrade. ";
			lose(true);
		}
	}
	announce(actor, obj, val) = {
		if (!self.isAlive)
			return;
		if (obj = grenade) {
			switch (val) {
				case 1:
					"\"Grenade!\" one of the raider shouts as he sees you.  The room falls quiet and every eye turns in your direction. ";
					break;
				case 2:
					"The raiders return to their conversations.  Several of them cast occasional dark looks in your direction. ";
					break;
				case 3:
					"The men closest to you dive behind the nearest available cover.  However, the ones further away take a moment to eliminate you before ducking through an open doorway. ";
					lose(true);
					break;
				case 4:
					"Almost immediately, a burst of gun fire originating from that direction tears into your unprotected body. ";
					lose(true);
					break;
			}
		}
	}
	listendesc = {
		if (self.isAlive)
			"You can barely hear yourself think over the noise of this crowd. ";
		else
			"The room is as quiet as a morgue. ";
	}
	smelldesc = "A delightful blend of spiced plum and vomit.  Mmmm. "
	lastnum = -1
	lastshown = []
	skip = nil
	count = 0
;

// Random actions to develop atmosphere.  Also contains certain milestones
// (and death) if the player spends A LOT of time in the barracks.
crowdDaemon: function(actor) {
	local choice, choicelist;
	if ((actor.location != barracks) && (actor.location.location != barracks)) {
		return;
	}
	if (global.venttime > 0)
		return;
	crowd.count++;
	switch (crowd.count) {
		case 20:
			"<<global.dispParagraph>>Out of the corner of your eye you catch one of the men looking at you suspiciously. ";
			return;
			break;
		case 30:
			"<<global.dispParagraph>>The raider you noticed watching you earlier is now talking quietly with a cluster of other men.  You can't help but notice them looking in your direction. ";
			return;
			break;
		case 40:
			"<<global.dispParagraph>>A raider attempts to eat his knife. I don't think it would agree with him. ";
			return;
			break;
		case 50:
			"<<global.dispParagraph>>Suddenly several men rush you from behind and grab your arms.  A number of other raiders encircle you.  Their weapons are raised and pointed in your direction.  \"I don't know who you are,\" an authoritative figure says, stepping into view, \"but you have a lot of explaining to do.\"  The tall, shrewd man steps forward and tugs on the collar of your ";
			if (uniform2.isworn)
				"jacket";
			else
				"coat";
			".  \"Did you really think that just wearing one of our coats would fool us forever?\"<<global.dispParagraph>>The following several hours are exceedingly unpleasant as you are slowly tortured to death. ";
			lose(true);
			return;
			break;
	}
	if (crowd.skip) {
		crowd.skip := nil;
		return;
	} else {
		if (rand(2) = 1)
			crowd.skip := true;
	}
	choicelist := [ 1 2 3 4 5 6 7 8 9 10 11 12 13 ] - crowd.lastshown;
	choice := choicelist[rand(length(choicelist))];
	if (length(crowd.lastshown) >= 7)
		crowd.lastshown := cdr(crowd.lastshown);
	crowd.lastshown += [choice];

	"<<global.dispParagraph>>";
	switch (choice) {
		case 1:
			"Without warning, a fist-fight breaks out among the raiders.  The two brawlers trade punches and kicks for a few moments until the rest of the crowd breaks up the fight. ";
			break;
		case 2:
			"Several of the men begin playing a raucous game of cards.   ";
			break;
		case 3:
			"A group of raiders clear off a makeshift table.  Two of them sit at opposite ends and begin arm wrestling.  The rest of the men crowd around the two challengers, yelling and cheering. ";
			break;
		case 4:
			"One of the men begins bragging loudly about how many villagers he killed.  The rest of the raiders appear unimpressed by his boasts. ";
			break;
		case 5:
			"A raider hurls a small metal dish across the room, grumbling about the food.  Several others nearby mutter their agreement. ";
			break;
		case 6:
			"A handful of rifle cartridges drops to the floor and the rounds scatter across the room.  The man who dropped them swears as he chases after them.  A couple of the other raiders snort and laugh. ";
			break;
		case 7:
			"One of the raiders tells a lewd joke, followed by shouts of laughter from the rest of the crowd.  ";
			break;
		case 8:
			"With a knife, a raider tries to slice a cut in the wall.  He examines the wall with a puzzled look as his weapon fails to make even a mark. ";
			break;
		case 9:
			"A small crowd forms around a raider holding up a yellowed girly magazine.  The men hoot and grunt at the centerfold. ";
			break;
		case 10:
			"A member of the crowd begins telling the story of how his great-great-grandfather was once the prince of a place called Ohio.  The other men seem a little awed by this revelation. ";
			break;
		case 11:
			"You are roughly jostled by a careless raider as he walks by.  \"Watch where you're going,\" he snarls at you. ";
			break;
		case 12:
			"A group of the men begin arguing with several others about the proper way to scalp a person: some of the men insist that the best way is to start at the front and the others disagree. ";
			break;
		case 13:
			"Some of the crowd begin to debate whether dogs or cats make a tastier meal. ";
			break;
	}
}

raiderweapons: iron, fixeditem
	sdesc = "raider weapons"
	noun = 'weapons' 'guns'
	adjective = 'recently' 'recently-fired'
	isThem = true
	ldesc = "Guns of all sizes, colors, and calibers. "
	location = nil
	verDoTake(actor) = {
		"You check the various weapons and find them all empty. ";
	}
	smelldesc = "The acrid scent of gunpowder. "
;

board: fixeditem, qsurface
	noun = 'panel'
	adjective = 'brushed' 'stainless' 'steel' 'metal'
	sdesc = "stainless steel panel"
	location = barracks
	ldesc = {
		"The metal panel is a stainless steel rectangle, about a foot wide and a foot-and-a-half tall.  It's set into the wall, completely flush against the surface, about four feet off the floor. ";
		if (itemcnt(self.contents)) {
			"On "; self.thedesc; " %you% see%s% "; listcont(self); " being held on by a simple kitchen magnet. ";
		} else {
			if (magnet.location = self)
				"The only thing on <<self.thedesc>> is a small kitchen magnet. ";
			else
				"There's nothing on <<self.thedesc>>. ";
		}
	}
	isqcontainer = true
	ioPutOn(actor, dobj) = {
		if (dobj = magnet) {
			"You stick the magnet onto the <<self.thedesc>>. ";
			dobj.moveInto(self);
			return;
		}
		if (!dobj.ispaper) {
			"\^<<dobj.thedesc>> would just slide right off. ";
			return;
		}
		if (!magnet.isReachable(actor)) {
			"\^<<dobj.thedesc>> will just fall right off without something to hold it up. ";
			return;
		}
		"\^%You% use the magnet to attach <<dobj.thedesc>> to the <<self.thedesc>>. ";
		dobj.moveInto(self);
		magnet.moveInto(self);
	}
	verDoOpen(actor) = "If <<self.thedesc>> opens, it's not clear how. "
	verDoPull(actor) = "There's no way to get a grip on the smooth panel, since it's flush with the wall. "
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

// Red herring, basically
map: item, paper, readable, knownTopic
	noun = 'map'
	sdesc = "map"
	location = board
	bulk = 0
	weight = 0
	ldesc = "It appears to be an old, faded map of the countryside.  After all your travels, you do have a certain familiarity with the terrain.  The location of Arcadia is marked in red, as is the location of this base. "
	doBurn(actor) = {
		"You toss the map into the fire and watch as it is quickly reduced to ash. ";
		self.moveInto(nil);
	}
;

note: item, paper, readable
	noun = 'note'
	adjective = 'paper' 'ragged' 'yellow'
	sdesc = "note"
	location = board
	bulk = 0
	weight = 0
	ldesc = "A note written on ragged, yellow paper. "
	readdesc = "The note reads:\b
	Okay, maggots, this is the last time I'm gonna tell you.  You don't bother me unless it is something important.  Otherwise you talk to Ertz or Randall.  The next person to bother me without a very good reason ends up in a shallow grave next to Sulz.\b
	Those of you that can read, make sure the rest of the troops understand this.\b
	Randall managed to scrounge up a pack of explosives on his last trip out.  If we can't get that damn door open any other way, we'll blast the fucking thing.\b
	Remember: the wasteland belongs to <B>us</B>.\b
	Boss"
	doBurn(actor) = {
		"You toss the note into the fire and watch as it is quickly reduced to ash. ";
		self.moveInto(nil);
	}
;

magnet: item
	noun = 'magnet'
	adjective = 'kitchen'
	sdesc = "kitchen magnet"
	location = board
	bulk = 1
	weight = 0
	ldesc = "A small black disc, about an inch in diameter and a quarter of an inch thick. "
	isListed = {
		if (self.location = board)
			return nil;
		return true;
	}
	verDoAttachTo(actor, iobj) = {
		if (iobj != pole)
			"There's no obvious way to do that. ";
		if (iobj != nil)
			iobj.verDoPutOn(actor, self);
	}
	verIoAttachTo(actor) = {}
	ioAttachTo(actor, dobj) = {
		if (dobj != nil)
			dobj.ioAttachTo(actor, self);
	}
	ioTouchTo(actor, dobj) = {
		if (dobj != pole)
			"Nothing happens. ";
		else
			self.doPutOn(actor, dobj);
	}
	doPutOn(actor, io) = {
		self.moveInto(io);
		if (io = pole) {
			"The magnet is now firmly attached to the end of the metal pole. ";
		} else {
			"Done. ";
		}
	}
	moveInto(obj) = {
		local i, num, onboard;
		onboard := (self.location = board);
		inherited.moveInto(obj);
		if (obj = board)
			return;
		if (onboard && (itemcnt(board.contents))) {
			"The papers being held by the magnet flutter to the floor. ";
			num := itemcnt(board.contents);
			for (i:=1; i<=num; i++) {
				board.contents[1].moveInto(barracks);
			}
		}
	}
	verIoTakeWith(actor) = {}
	ioTakeWith(actor, dobj) = {
		if ((dobj = securitybadge) && (!desk1.gotbadge)) {
			"You can't reach the object with the magnet.  It's too far underneath the desk. ";
		}
	}
	verDoPutUnder(actor, io) = {}
;

conference: room
	sdesc = "Conference Room"
	ldesc = {
		"From the looks of it, the original purpose of this large room was to hold meetings and briefings.  However, the large table and office chairs are in desperate need of maintenance and the once white walls now sport an impressive array of scuffs, gouges, and puncture marks.  The doorway to the north leads back into the barracks and the western door has one of the more intact chairs sitting beside it, almost blocking the way. ";
		submachinegun.known := true;
	}
	specialdesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			return 'Whirling red lights and warning alarms fill the room.  Confused and directionless raiders run through the halls, searching vainly for the cause.  You seem to go completely unnoticed. ';
		}
		return '';
	}
	north = barracks
	west = confguard
	enterRoom(actor) = {
		inherited.enterRoom(actor);
		if (isinside(grenadelauncher, actor) && (global.venttime = 0)) {
			"<<global.dispParagraph>>\"Fucking thief!\" the raider guard shouts as he turns his gun toward you and pulls the trigger.  As you fall to the ground, the grenade launcher slips from your grasp and clatters to the floor. ";
			lose(true);
		}
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"To the north you can hear the dull roar of the barracks. ";
	}
	exitlist = { "Exits lie to the north and west. "; return nil; }
	listContents = {
		"\^%you% notice%s% "; listcont(self); " here. ";
	}
;

confguard: Actor, doorway, knownTopic
	noun = 'raider' 'bandit' 'guard' 'brigand' 'man' 'himself'
	adjective = 'bandit' 'raider' 'brigand' 'scruffy' 'looking' 'brown' 'leather' 'jacket'
	sdesc = "raider guard"
	thedesc = "the raider guard"
	location = conference
	isHim = true
	seennecklace = nil
	seengun = nil
	askedaboutcube = nil
	gotpassword = nil
	gone = nil
	doordest = securitystation
	stockroom = confguard
	suspicion = 0
	threshold = 3
	passwordlist = ['boulder' 'mutant' 'pinhole' 'monkey']
	passwordnum = 0
	isMerchant = true
	getpassword = {
		return self.passwordlist[self.passwordnum];
	}
	ldesc = {
		"A little less scruffy than the rest of the raiders, but not by much.  He's wearing a brown leather jacket, torn in a couple of places, with the same green and red bands as the rest of the raiders.  The raider is sitting in the chair beside the door, a mean looking black submachine gun cradled in his hands.  His feet are propped up on the table in the center of the room. ";
		if (datacube1.location = self) {
			"Dangling from a cord around the man's neck is what appears to be a yellow-tinted glass prism that refracts and reflects the light in the room. ";
		}
		if (pistol.location = self) {
			"On his pistol, he's wearing a silenced pistol. ";
		}
	}
	destination = {
		if (self.gone)
			return self.doordest;
		if (self.gotpassword) {
			"The raider guard waves you in.\b";
			guard2Hint.active := nil;
			return self.doordest;
		} else {
			"\"Nah, nah, nah,\" the guard says, holding up his hand.  \"No one gets through that door without this week's password.  If you got permission to see da boss, then you got da password.  Otherwise, beat it.\" ";
			guard2Hint.active := true;
			return nil;
		}
	}
	initialize = {
		passwordnum:=rand(4);
	}
	actorDesc = "Sitting in the one good chair is a raider guard whose attention is focused on you. "
	verDoTake(actor) = "You find yourself overestimating your physique. "
	verGrab(item) = {
		"\^<<thedesc>> sticks his gun in your face.  \"Don't be so grabby,\" he growls. ";
	}
	actorAction(v, d, p, i) = {
		if (v = helloVerb) {
			"\"Yo.\" ";
		} else if (v = swearVerb) {
			"The guard barks a laugh.  \"Well aren't you a tough guy.\" ";
		} else if (v.txt = self.getpassword) {
			"You say, \"The password is '<<self.getpassword>>.'\"<<global.dispParagraph>>The guard nods.  \"That's it.  Go on in.  Just don't mess with anything in there.  Understand?  If you do, I got to kill you.\" ";
			self.gotpassword := true;
		} else if ((v.txt != nil) && (v.txt != '')) {
			"The guard shakes his head.  \"Nope.\" ";
		} else if (v = noVerb || v = yesVerb) {
			"The guard says, \"Actions speak louder than words.\" ";
		} else {
			"\^<<self.thedesc>> glares at you.  \"Look, I only take orders from one guy, and you ain't him.\" ";
		}
		exit;
	}

	askTopics(topic, words) = {
		return askGuard(self, topic);
	}
	tellTopics(topic, words) = {
		switch (topic) {
			case metopic:
				"If you blow your cover, it could cost you your life.. ";
				break;
			case scrapofpaper:
				"It's doubtful that the guard would find the scrap of paper interesting, considering where he is. ";
				break;
			case arcadiacity:
				"And give away your identity?  No, you decide against it. ";
				break;
			case iona:
				if (global.scKillGirl) {
					"The guard guffaws as you tell him the story of killing the little girl that the other raiders missed. ";
					guard.suspicion := 0;
				} else {
					"\"You found a survivor?\" he asks in surprise.  \"If I wasn't on guard duty, I'd go help you take care of her.  But since I am, you'll have to do it yourself.\" ";
				}
				break;
			case crumpledpaper:
				"The guard shrugs.  \"We've found bits of useless paper all over this place.\" ";
				break;
			case grenadelauncher:
				if (clay.location = grenadelauncher) {
					"\"The grenade launcher is full of clay?\" he asks in shock.  He rushes into the security station and, after a few moments, returns.  \"Don't know who did that, but it's a good thing you found it.  If the grenade launcher had been fired with a blockage in the barrel, it probably would have blown the whole thing to bits.\" ";
				} else {
					"The guard would not be interested in what little you have to say on that subject. ";
				}
			case scavengers:
				"\"Don't try to defend those thieving scum.\" ";
				break;
			case bouldertopic: case monkeytopic: case mutanttopic: case pinholetopic:
				if (topic.txt = self.getpassword) {
					"You say, \"The password is '<<self.getpassword>>.'\"<<global.dispParagraph>>The guard nods.  \"That's it.  Go on in.  Just don't mess with anything in there.  Understand?  If you do, I got to kill you.\" ";
					self.gotpassword := true;
				} else {
					"The guard shakes his head.  \"Nope.\" ";
				}
				break;
			default:
				return nil;
		}
		return true;
	}
	disavow = "\^<<self.thedesc>> says, \"I don't know whatcher talking about.\" "
	telldisavow = "The guard would not be interested in what little you have to say on that subject. "
	verDoTellAbout(actor, iobj) = {
	}
	verIoShowTo(actor) = {}
	ioGiveTo(actor, dobj) = {
		if (dobj = cash) {
			"The bandit looks at your money and laughs.  \"Where did you get that junk?  Did one of the other guys fool you into accepting that stuff?  It's worthless out here.  You might be able to use that stuff in one of the towns, but out here in the wastes it'd only be good for wiping your ass.\" ";
			return;
		}
		if (datacube1.location != self) {
			"\^<<self.thedesc>> shrugs.  \"Thanks, I guess.\" ";
			dobj.moveInto(self);
			return;
		}
		if (!self.askedaboutcube) {
			"\^<<self.thedesc>> shrugs.  \"Thanks, I guess.\" ";
			dobj.moveInto(self);
		} else {
			if (dobj = pistol) {
				"The guard nods and hands you the prism in exchange for the pistol.  \"Heh, can't believe you'd want to make a trade like this.\" ";
				pistol.moveInto(self);
				datacube1.moveInto(actor);
			} else if (dobj = necklace) {
				"The guard nods and hands you the prism in exchange for the quartz crystal necklace.  He loops the leather strap around his neck and holds the crystal in front of his face, marveling at its sparkle. ";
				necklace.moveInto(self);
				datacube1.moveInto(actor);
			} else {
				"He shakes his head.  \"Nah, not interested in that.\" ";
			}
		}

	}
	ioShowTo(actor, dobj) = {
		if (dobj = cash) {
			"The bandit looks at your money and laughs.  \"Where did you get that junk?  Did one of the other guys fool you into accepting that stuff?  It's worthless out here.  You might be able to use that stuff in one of the towns, but out here in the wastes it'd only be good for wiping your ass.\" ";
			return;
		}
		if (dobj = necklace) {
			if (necklace.location = self) {
				"You're not carrying it. ";
			} else {
				self.seenecklace;
			}
		} else if (dobj = pistol) {
			"\^<<self.thedesc>> nods approvingly.  \"Nice looking piece.  Wouldn't mind having one of those.\" ";
			self.seengun := true;
		} else {
			"\^<<self.thedesc>> says, \"Yeah?  So?\" ";
		}
	}
 	askWord(word, lst) = {
		local iobj := parserGetObj(PO_IOBJ);
 		askGuard(self, iobj);
 	}
	doTalkto(actor) = {
		"\"Sorry, pal, I ain't interested in small talk.  If you have a question, then ask.";
		if (!self.gotpassword)
			"  If you're here to see the boss, you'd better know the password.";
		if (self.gotpassword)
			"  Otherwise you can leave.";
		"\" ";
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if (weapon != grenade)
			inherited.doAttack(weapon);
		if (weapon = pistol) {
			"The guard's eyes widen as you lift your pistol and aim it at him.  Your shot hits him in the shoulder and he cries out in pain.  Gritting his teeth, he mashes the trigger on his submachine gun, spraying wildly.  Unfortunately for you, several of those bullets hit their mark. ";
			lose(true);
		}
		if (weapon = rifle) {
			"The guard's eyes widen as you heft your rifle and aim it at him.  A single shot sends him sprawling across the floor.  Unfortunately, the noise quickly attracts the rest of the raiders, who quickly cut you down. ";
			lose(true);
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"Your attack achieves only a glancing blow.  The enraged raider empties his magazine into you at point-blank range.  You are very, very dead. ";
			lose(true);
		}
	}
	announce(actor, obj, val) = {
		if (obj = grenade) {
			switch (val) {
				case 1:
					"The guard seated before you lifts his firearm up, aiming it directly at your head.  \"You'd best put that pin back in.\" ";
					raiseSuspicion(self);
					break;
				case 3:
					"Too bad the delay on the grenade is too long to prevent the raider guard from emptying a generous portion of lead into your torso.  Which he does, of course. ";
					lose(true);
					break;
				case 4:
					"Almost immediately, a burst of gun fire originating from that direction tears into your unprotected body. ";
					lose(true);
					break;
			}
		}
	}
	turnin = {
		"The guard frowns at you and grips his firearm tightly.  \"Come to think of it, I don't think I've ever seen you before.\"  The gun raises up.  \"Now if you really belong here, you'll be able to tell me the boss's name.\"  Perspiration begins to drip down your face.  Though you sputter and make excuses, you can tell by the look on the guard's face that you won't be able to talk your way out of this one.... ";
		lose(true);
	}
	seenecklace = {
		"You lift the quartz crystal necklace up and show it to the guard.  He raises an eyebrow, then fingers the prism hanging around his neck.  \"Hey, wow, that's pretty nice.  A lot better than this old thing here.  Let me know if you're interested in a trade.\"  He raises an eyebrow. ";
		self.seennecklace := true;
		self.askedaboutcube := true;
	}
	smelldesc = "Just a hint of vanilla.  And dank sweat. "
	verIoTradeWith(actor) = {
		if (datacube1.location != self) {
			"\^<<self.thedesc>> shakes his head.  \"I ain't got nothing more I want to trade.\" ";
			return;
		}
	}
	ioTradeWith(actor, dobj) = {	// actor is offering dobj to this NPC
		if ((dobj = necklace) || (dobj = pistol)) {
			self.askedaboutcube := true;
			self.ioGiveTo(actor, dobj);
			return;
		}
		"\^<<self.thedesc>> snorts.  \"What would I want with that?\" ";
	}
	ioTradeTo(actor, dobj) = {	// actor is offering dobj to this NPC
		self.ioTradeWith(actor, dobj);
	}
	makeTrade(actor, offerObj, forObj) = {	// actor is offering offerObj to this NPC in exchange for forObj
		if (forObj = submachinegun) {
			"The guard laughs.  \"Yeah, I'll just bet you'd like to do that.  Think again.\" ";
			return;
		}
		if (forObj = datacube1) {
			if ((offerObj = necklace) || (offerObj = pistol)) {
				self.askedaboutcube := true;
				self.ioGiveTo(actor, offerObj);
				return;
			}
			"\^<<self.thedesc>> snorts.  \"What would I want with that?\" ";
			return;
		}
		if (forObj = pistol) {
			"The guard frowns.  \"You just gave the damn thing to me.  Forget it.  No deal.\" ";
			return;
		}
	}
	verIoBuyFrom(actor) = {
	}
	ioBuyFrom(actor, dobj) = {
		"The bandit looks at your money and laughs.  \"Where did you get that junk?  Did one of the other guys fool you into accepting that stuff?  It's worthless out here.  You might be able to use that stuff in one of the towns, but out here in the wastes it'd only be good for wiping your ass.\" ";
	}
	doHug(actor) = {
		"He doesn't look like he'd appreciate it. ";
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"You experience a mixture of irritation and tedium.  Scattered images of bloodshed whip through your mind. ";
		"The guard gives you an odd look. ";
	}
;

// Another gun the player will never get
submachinegun: gun, iron, topic
	sdesc = "submachine gun"
	noun = 'submachinegun' 'submachine' 'gun' 'subgun'
	adjective = 'semiautomatic' 'silenced' 'submachine'
	location = confguard
	rounds = 30
	bulk = 15
	weight = 15
	ldesc = "Large, black, and very scary.  The guard clutches it tightly. "
	ioAttackWith(actor, dobj) = {
		"You're not carrying it! ";
	}
	smelldesc = "It has the faint scent of gun oil. "
;

// Password "verbs"
class passwordverb: darkVerb
	action(actor) = {
		if (actor.location != conference)
			"There's no verb in that sentence! ";
		else
			confguard.actorAction(self, nil, nil, nil);
	}
;

boulderpwd: passwordverb
	verb = 'boulder'
	txt = 'boulder'
	sdesc = "boulder"
;

mutantpwd: passwordverb
	verb = 'mutant'
	txt = 'mutant'
	sdesc = "mutant"
;

pinholepwd: passwordverb
	verb = 'pinhole'
	txt = 'pinhole'
	sdesc = "pinhole"
;

monkeypwd: passwordverb
	verb = 'monkey'
	txt = 'monkey'
	sdesc = "monkey"
;

datacube1: item, clothingItem, knownTopic
	sdesc = "yellow datacube"
	noun = 'cube' 'datacube' 'prism'
	adjective = 'glass' 'yellow' 'data' 'leather' 'cord'
	plural = 'cubes' 'datacubes' 'prisms'
	location = confguard
	datacube = true
	bulk = 2
	weight = 1
	cost = 1
	ldesc = "It very much resembles a yellow-tinted glass prism that refracts and reflects the light in the room.  Datacubes like this were the standard portable digital storage medium before the war.  This one has leather cord fastened to it, so it can be worn around the neck. "
	data = [self.line1 self.line2 self.line3 self.line4 self.line5 self.line6]
	line1='You hear a trembling female voice come from the log player: \"The personal log of Doctor Colleen Hill, cube twelve.\b\"October twenty-first.  Colonel Miles ordered the top two floors evacuated during all of this... insanity.  Lord Christ, I can\'t believe it came to this.  Major Tuco said he believed it was probably all over already, since nuclear war isn\'t something that can really drag on.  Is it?  How long will we be safe down here?  No one seems to know.  Some of the soldiers say that all military facilities like this are designed to provide years of comfortable living in the event of such a \'contingency\'.  This is not a contingency!  It\'s the death of the whole damn world!\"  The speaker pauses and takes a breath.  \"In any event, the problem is that no one was exactly expecting this.  Maybe we could stay down here for years, if the base was fully stocked.  Problem is, no one was anticipating a nuclear war this month.\"'
	line2='\bThe speaker continues: \"October twenty-eighth.  No contact on the radio for over a week.  Everyone believes the bombs have stopped, but does it matter?  The world is poison now.  Still, the base life support systems seem up to the task and we\'ve all returned to the first two floors.  No one\'s gone outside though.  We were told that this new self-maintaining reactor design might well last over a hundred years without maintenance.  As long as no one screws up and shunts the plasma.\"'
	line3='\bThe female voice goes on: \"November second.  Not much of a celebration.  Supplies are low and rationed, so a party was out of the question.  Bobby stuck a candle in a dinner roll and wished me a happy birthday.  It was a sweet gesture, though didn\'t really do much to cheer me up for long.\"'
	line4='\bThe voice begins again: \"November thirteenth.  Radio\'s still got nothing but static.  Fredericks, Lee, and James suited up and went above ground.  Fredericks says all of the vehicles are gone and that the soldiers who were trapped outside when the base went on lock down must have taken them and fled.  Can they still be alive up there?  Lee says the radiation level is dangerously high, but can\'t be sure how long it\'ll stay that way.  Maybe for the next ten thousand years?\"'
	line5='\bThe female voice continues, the strain clearly evident: \"December eighth.  We\'re all falling apart.  There\'s no food left and Tonya finally went over the edge today.  My God, she shot the Colonel right in front of everyone!  There were just these deafening booms and Colonel Miles just collapsed and there was all this blood, he was just bleeding so much.\"  She draws a shaky breath.  \"Then before I think even Tonya realized what she\'d done, Bobby and Walsh shot her dead too.  The Major went absolutely white, though I don\'t know if it was because of the murder or that he\'s now in charge.\"'
	line6='\bThe speaker again, this time in a flat and emotionless voice: \"December ninth.  We\'re leaving.  The Captain decided, since the Major is gone.  No one knows where he went.  Lince thinks Tuco went topside and shot himself with his pistol, though so far no one has gone to check.  I guess we\'ll find out soon enough.  We\'ve emptied the commissary of nearly everything.  Everything except for Tuco\'s rad suit.  Corporal Wilson did some radiation checks this morning and found that levels were low enough to allow travel.  To where?  I don\'t know.  Maybe Tuco had the right idea.\"'
;

securitystation: room, knownTopic
	sdesc = "Security Station"
	noun = 'station'
	adjective = 'security'
	ldesc = {
		"You are standing in a small square room, about ten feet across.  You notice a distinct lack of the grime and squalor that has been so evident in the rest of the facility.  The large metal desk in the southeast corner appears to be reasonably clean and in good condition, as does the office chair before it.  Mounted on the south wall are a number of small screens, each labeled with a number.  All of them are dark.  Attached to the desk is a small computer console, consisting of monitor and keyboard.  Two doors lead out: the east door towards the conference room and a second one heading to the west.  Both are closed. ";
		launcherHint.active := true;
	}
	specialdesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			return 'Whirling red lights and warning alarms fill the room. ';
		}
		return '';
	}
	east = conference
	west = "The door is locked. "
	exitlist = { "There are two doors available: east and west. "; return nil; }
	listContents = {
		"Resting in the corner %you% spot%s% "; listcont(self); ". ";
	}
	handleExplosion(actor, playerloc, heard) = {
		if (crowd.isAlive && (global.venttime = 0)) {
			if (playerloc = barracks) {
				"<<global.dispParagraph>>The guard from the conference rushes into the room.  \"Get him!\" he shouts, and several raiders immediately tackle you to the floor.  \"He tried to blow up the boss!\" the guard growls.  The following several hours are exceedingly unpleasant as you are slowly tortured to death. ";
				lose(true);
			} else if (playerloc = conference) {
				"<<global.dispParagraph>>The guard leaps to his feet upon hearing the noise.  \"What did you do in there?!\" he shouts, but he isn't quite patient enough to wait for a response.  His weapon thunders and everything goes dark. ";
				lose(true);
			}
			
		}
	}
;

desk2: fixeditem, iron, surface
	noun = 'desk'
	adjective = 'metal'
	location = securitystation
	sdesc = "metal desk"
	moved = nil
	ldesc = {
		"A bulky office desk built from stamped steel.  Most of the surfaces have been painted a rather hideous shade of beige. ";
		if (!self.moved)
			"The desk is pushed up against the walls in the southeastern corner of the room. ";
		if (self.contentsVisible and itemcnt(self.contents) <> 0) {
			"On the top of the desk, in addition to the computer console, you can see <<listcont(self)>>.";
		} else {
			"There is a computer console attached to it. ";
		}
	}
	verDoMove(actor) = {
		if (self.moved)
			"That hardly seems necessary now. ";
	}
	doMove(actor) = {
		"You struggle to move the weighty desk away from the wall.  As you do, a crumpled piece of paper flutters from behind it and settles on the ground. ";
		crumpledpaper.moveInto(self.location);
		self.moved := true;
	}
	verDoPush(actor) = {}
	doPush(actor) = {
		self.doMove(actor);
	}
	verDoLookbehind(actor) = {
		if (!self.moved)
			"The desk is pushed right up against the wall, making it impossible to look behind. ";
	}
	doLookbehind(actor) = {
		"There's nothing behind the desk. ";
	}
	verDoSearch(actor) = {}
	doSearch(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0)
			"On the top of the desk, in addition to the computer console, you can see <<listcont(self)>>. ";
		else
			"The top of the desk is clear, except for the computer console. ";
		if (!self.moved)
			"The desk is pushed right up against the wall, making it impossible to look behind.  Peering under it, you find nothing. ";
		else
			"There's nothing behind it or under it. ";
	}
	verDoLookin(actor) = {
		if (self.contentsVisible and itemcnt(self.contents) <> 0)
			"On the top of the desk, in addition to the computer console, you can see <<listcont(self)>>. ";
		else
			"The top of the desk is clear, except for the computer console. ";
	}
	verDoTake(actor) = "It's far too heavy for you to lift. "
	verDoSiton(actor) = "There's not much room to sit with the computer console in the way. "
;

crumpledpaper: readable, paper
	noun = 'paper'
	adjective = 'crumpled' 'piece of' 'old'
	sdesc = "crumpled paper"
	location = nil
	bulk = 0
	weight = 0
	ldesc = "A crumpled piece of old paper.  Appears to have some sort of memo on it. "
	readdesc = {
		"TO: Lieutenant Walsh\n
		FROM: Colonel Miles\n
		DATE: Sep 2, 2060\n
		RE: Base security\b
		Lieutenant,\b
		The current performance of the overall base security is unacceptable.  As you know, last week a civilian was found within the base perimeter while Lieutenant Leach's squad was on guard duty.  She has been reassigned to an Alaskan supply depot where the effects of her incompetence will be of less importance.  Yesterday I was informed by Dr. Hill that our network had been accessed by an unauthorized person from off site.  This finally gave me the justification I needed to sever our network from the rest of the outside world.  The general wasn't happy, but he couldn't deny the necessity.  Please have your people search their respective project files for any signs of unauthorized access or tampering.\b
		And Dr. Rock informed me that you're again using your last name as your password.  This practice is inexcusably insecure and if it happens again, you'll be digging trenches in Alaska.\b
		Colonel Perry Miles";
	}
	doBurn(actor) = {
		"You toss the paper into the fire and watch as it is quickly reduced to ash. ";
		self.moveInto(nil);
	}
;

// This object acts as a database object for base personnel.  The security
// codes are generated randomly, and so is the name on the security badge.
personDB: item
	location = nil
	firstnames = ['Lince' 'Colleen' 'Robert' 'Kevin' 'Tonya' 'James']
	lastnames = ['Herbert' 'Hill' 'Jacob' 'Rock' 'Robinson' 'Walsh']
	ranks = ['Sergeant' 'Dr.' 'Corporal' 'Dr.' 'Private' 'Lieutenant']
	gender = ['M' 'F' 'M' 'M' 'F' 'M']
//	codes = [99413 84149 54321 44544 21995 46661]
	codes = []
	initialize = {
		local i;
		for (i:=0; i<length(firstnames); i++) {
			codes += (rand(89999)+10000);
		}
		if (securitybadge.person = 1)
			codes[2] := 69105;
		else
			codes[1] := 69105;
	}
	getFullName(i) = {
		"<<self.firstnames[i]>> <<self.lastnames[i]>>";
	}
	getFullNameAndRank(i) = {
		"<<self.ranks[i]>> <<self.firstnames[i]>> <<self.lastnames[i]>>";
	}
	getNameAndRank(i) = {
		"<<self.ranks[i]>> <<self.lastnames[i]>>";
	}
	getRank(i) = {
		"<<self.ranks[i]>>";
	}
	getCode(i) = {
		return self.codes[i];
	}
;

securityconsole: switchItem, readable, knownTopic
	noun = 'console' 'keyboard' 'monitor' 'key' 'board'
	adjective = 'security' 'computer'
	sdesc = "security console"
	state = 0
	ldesc = {
		"This appears to be a simple single-purpose computer.  It's a keyboard and monitor combined into a single unit, mounted onto a tabletop. ";
		if (isActive) {
			"The security console is on and displaying text. ";
		} else {
			"The security console is off. ";
		}
		"(You can type on the keyboard by entering a command like: \(TYPE \"RUTABAGA\" ON KEYBOARD\))";
	}
	location = securitystation
	verDoUse(actor) = { self.verDoSwitch(actor); }
	doUse(actor) = { self.doSwitch(actor); }
	doTurnon(actor) = {
		self.isActive := true;
		"Okay, <<self.itisdesc>> now turned on.  Some text appears on the monitor:\n";
		self.getText;
	}
	verIoTypeOn(actor) = {
	}
	ioTypeOn(actor, dobj) = {
		local text;
		if (!isActive) {
			"Nothing happens.  It might help to turn it on. ";
			return;
		}
		if (isclass(dobj, basicStrObj)) {
			text := dobj.value;
		} else if (isclass(dobj, basicNumObj)) {
			text := cvtstr(dobj.value);
		} else {
			self.error;
			return;
		}
		text := lower(text);
		switch (state) {
			case 0:
				switch (text) {
					case 'walsh':
						self.state := 2;
						"Some new text appears:\n";
						self.getText;
						break;
					case 'herbert': case 'hill': case 'jacob': case 'rock': case 'robinson':
						self.state := 1;
						"Some new text appears:\n";
						self.getText;
						break;
					default:
						self.bottomText(&baduser);
				}
				break;
			case 1:
				self.bottomText(&badpassword);
				break;
			case 2:
				switch (text) {
					case 'walsh':
						self.state := 3;
						"The screen clears and a menu appears:";
						loginHint.active := nil;
						self.getText;
						break;
					case 'cancel':
						self.state := 0;
						"The screen clears and then shows:";
						self.getText;
						break;
					default:
						self.bottomText(&badpassword);
				}
				break;
			default:
				displayMenu(consolemenu);
				break;
		}
	}
	readdesc = {
		if (!isActive) {
			"The console is dark.  It might help to turn it on. ";
			return;
		}
		self.getText;
	}
	getText = {
		local i;
		switch (state) {
			case 0:
				"<CENTER><TT>LOGIN\bENTER LAST NAME:</TT></CENTER>";
				loginHint.active := true;
				break;
			case 1: case 2:
				"<CENTER><TT>ENTER PASSWORD:\n(ENTER \"CANCEL\" TO CANCEL LOGIN)</TT></CENTER>";
				break;
			case 3:
				displayMenu(consolemenu);
				return;
		}
	}
	bottomText(str) = {
		"The text \"<TT>";
		say(self.(str));
		"</TT>\" appears at the bottom of the screen. ";
	}
	error = 'UNRECOGNIZED INPUT'
	baduser = 'UNRECOGNIZED USER ACCOUNT'
	badpassword = 'INCORRECT PASSWORD'
;

// These logins are recognized, but no password is available
badlogins: item
	noun = 'Lince' 'Colleen' 'Robert' 'Kevin' 'Tonya' 'James' 'Herbert' 'Hill' 'Jacob' 'Rock' 'Robinson'
;

goodlogin: item
 	noun = 'Walsh'
;

// Menus for the security system
consolemenu: menu
	sdesc  = "<TT>MAIN MENU"
	ldesc  = "MAKE YOUR SELECTION: "
	choices = [menulogs menuduty menuaccess menusecurity menulogout]
	enterMsg = "Enter choice:</TT>"
	illegalMsg = "<TT>UNRECOGNIZED INPUT</TT>"
	quitMsg = "[Stop using console]"
;

// Security logs
menulogs: menuChoice
	sdesc = "REVIEW SECURITY LOGS"
	Chosen = {
		"\bThe logs go on for page after page.  After skimming through the dry, boring text, you reach the last several entries:\b
		<TT>\n\tSECURITY LOGS\b
		<I>2060-08-25\t\tCorporal Jacob</I>\n\tA civilian penetrated the base perimeter before being spotted by a passing tech and reported to security.  Colonel Miles has had Lieutenant Leach reassigned for this failure in security.\b
		<I>2060-09-20\t\tSergeant Herbert</I>\n\tA company of renegade troops passing nearby caught sight of the base perimeter and advanced on the base.  The 83rd Armored engaged them and drove them back, but took heavy casualties.\b
		<I>2060-10-11\t\tSergeant Herbert</I>\n\tOfficial word's come in that another base in NM has fallen to the rebels.  Rumors have it that their base commander went over to the other side.  The Colonel has ordered now that speech even somewhat sympathetic to the rebel cause is grounds for court martial and immediate execution for treason.\b
		<I>2060-10-20\t\tMajor Tuco</I>\n\tThe owrd just came ocver the radio.  Edgar gave THE ORDER.  oh god I camn't bleieve this oh god oh god oh god\b
		<I>2060-12-09\t\tCaptain Ramirez</I>\n\tWe're out of food and supplies.  Major Tuco has vanished.  Colonel Miles is dead.  Private Robinson went crazy when the Colonel refused to discuss the idea of leaving the base.  Shot him right in front of everyone.  The others killed her right after she did it.  Now we have no choice.  We hve to leave and see if there's anything left.  Can there be?  Can there be anything left?  Anyone alive?  The radio's been dead since it all happened.  We will be too if we don't leave soon.  Mother of God.\b
		</TT>";
		morePrompt();
//		clearscreen();
	}
;

// Duty shifts
menuduty: menuChoice
	sdesc = "DUTY SHIFTS"
	Chosen = {
		"<TT>\b\n\tDUTY SHIFTS\b
		\n\tMONDAY: Corporal Wilson, Private Paris, Private Grant
		\n\tTUESDAY: Sergeant Herbert, Corporal Jacob, Private Robinson
		\n\tWEDNESDAY: Corporal Fredericks, Corporal Lee, Private James
		\n\tTHURSDAY: Sergeant Goldburg, Private LaBarbara, Private Roy
		\n\tFRIDAY: Corporal Carson, Private Charles, Private Boyne
		\n\tSATURDAY: Corporal Steele, Corporal Greene, Private Nagagta
		\n\tSUNDAY: Sergeant Hale, Private Ivanova, Private Morris</TT>";
		morePrompt();
//		clearscreen();
	}
;

// View security codes
menuaccess: menuChoice
	sdesc = "UPDATE SQUAD ACCESS CODES"
	Chosen = {
		local i;
		"<TT>\b\n\tACCESS CODES:\b";
		for (i:=1; i<=6; i++) {
			"\n\t<<personDB.getFullNameAndRank(i)>>: <<personDB.getCode(i)>>";
		}
		"\b\n\tSECURITY LEVEL INSUFFICIENT TO UPDATE CODES</TT>";
		if (!global.scFoundCode) {
			incscore(1);
			global.scFoundCode := true;
		}
		codeHint.active := nil;
		morePrompt();
//		clearscreen();
	}
;

// Base security levels
menusecurity: menuChoice
	sdesc = "SET BASE SECURITY LEVELS"
	Chosen = {
		"<TT>\b\n\tBASE SECURITY LEVELS\b
		\n\tLEVEL 4: STANDARD
		\n\tLEVEL 3: DUTY SHIFTS DOUBLED
		\n\tLEVEL 2: DUTY SHIFTS TRIPLED
		\n\tLEVEL 1: DUTY SHIFTS TRIPLED, AUTOMATED DEFENSES ACTIVE\b
		\n\tCURRENT LEVEL: 2\b
		\n\tSECURITY LEVEL INSUFFICIENT TO SET SECURITY LEVEL</TT>";
		morePrompt();
//		clearscreen();
	}
;

menulogout: menuChoice
	sdesc = "LOGOUT"
	Chosen = {
		securityconsole.state := 0;
		"<<global.dispParagraph>>The screen clears and then shows:";
		securityconsole.getText;
	}
	exitAfter = true
;

// Another gun that can't be used.  Can be sabotaged, though.
grenadelauncher: gun, knownTopic
	noun = 'launcher' 'muzzle' 'barrel' 'weapon'
	adjective = 'grenade' 'black' 'launcher'
	sdesc = "grenade launcher"
	location = securitystation
	ldesc = {
		if (leader.isalive)
			"A formidable looking weapon.  Built mostly from black plastic and steel.  The muzzle seems so wide you could practically fall into it.  You notice that the grip seems to have some kind of electronic sensor on it. ";
		else
			"It's nothing more than a charred twisted lump of slag now. ";
	}
	isqcontainer = true
	bulk = 21
	weight = 12
	verIoPutIn(actor) = {}
	ioPutIn(actor, dobj) = {
		if (dobj != clay) {
			"You can't figure out how to get <<dobj.thedesc>> into <<self.thedesc>>. ";
			return;
		}
		if (self.location = leader) {
			"There's no way you can get near enough while the raider boss is holding it. ";
			return;
		}
		"You cram the glob of clay into the cavernous barrel of the grenade launcher.  The barrel is now completely blocked, but far enough in that you wouldn't notice unless you were looking carefully. ";
		clay.moveInto(self);
	}
	ioAttackWith(actor, dobj) = {
		"You pull the trigger, but the only thing that happens is that the grenade launcher emits a loud buzz. ";
	}
	verIoShootWith(actor) = {
		"You pull the trigger, but the only thing that happens is that the grenade launcher emits a loud buzz. ";
	}
	doTake(actor) = {
		inherited.doTake(actor);
		if (self.location = actor)
			"When you pick up the grenade launcher, the weapon lets out a loud, angry buzz.  A grid on the weapon's grip flashes red. ";
	}
	smelldesc = "It has the faint scent of gun oil. "
	verDoLoad(actor) = {}
	doLoad(actor) = {
		"You don't have any extra ammunition to load. ";
	}
	verDoLoadWith(actor, iobj) = {
		if (iobj = grenade) {
			"As far as you can tell, the grenade launcher fires a specialized type of self-launching grenade.  Your hand grenade won't fit. ";
		}
	}
	verDoEmpty(actor) = "You can't figure out how to eject the magazine.  It may be locked somehow. "
	verDoUnload(actor) = "You can't figure out how to eject the magazine.  It may be locked somehow. "
 	doLookin(actor) = {
		"You can't figure out how to eject the magazine.  It may be locked somehow. ";
		if (clay.location = self) {
			"Peering down the barrel, you can see a lump of clay deep within. ";
		}
	}
;

glgrip: gun, floatingItem, knownTopic
	noun = 'grip' 'sensor' 'grid'
	adjective = 'grenade' 'launcher'
	sdesc = "grenade launcher grip"
	location = grenadelauncher
	ldesc = {
		if (leader.isalive)
			"There is some kind of faintly glowing grid along the backstrap of the grenade launcher's grip.  You're not sure exactly what it is.  Some kind of sensor, perhaps? ";
		else
			"Not much left of it now. ";
	}
	verifyRemove(actor) = "You can't really do much with the grip; it's part of the grenade launcher. "
	verDoTouch(actor) = "As your finger touches the grid, it flashes red and releases a loud buzz. "
;

// Separates first floor hall from second floor hall so that both security
// doors are not visible at the same time.  What a hassle that would be.
firststairs: room
	sdesc = "Top of Stairs"
	ldesc = {
		"You are standing on a landing at the top of a flight of steps that runs down to the south.  To the north is a steel sliding door, with a button on the wall beside it, and a brightly illuminated lamp above. ";
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if (firstfloordoor.forced) {
			"The door to the north appears to have been blasted open.  The thick steel is buckled and scorched. ";
		} else if (firstfloordoor.isblocked) {
			"The door to the north has been blocked open by a metal pole in its track. ";
		}
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			if (firstfloordoor.forced || firstfloordoor.isblocked) {
				"<<global.dispParagraph>>";
			}
			"Whirling red lights and warning alarms fill the landing. ";
		}
		str := outcapture(stat);
		return str;
	}
	enterRoom(actor) = {
		if (!global.insecond) {
			global.insecond := true;
			door1Hint.active := nil;
		}
		pass enterRoom;
	}
	north = {
		if (!firstfloordoor.isblocked) {
			if (!firstfloordoor.isopen) {
				"You hit the button beside the door to open it, then pass through.\b";
				firstfloordoor.isopen := true;
				notify(panel1, &closedoor, 2);
			}
		}
		return firstfloorhall;
	}
	south = secondfloorhall
	down = secondfloorhall
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"It's eerily quiet here. ";
	}
	exitlist = { "There's a door to the north and the stairs head down to the south. "; return nil; }
;

redbutton1: buttonitem
	sdesc = "red button"
	adjective = 'red' 'large'
	location = firststairs
	doPush(actor) = {
		if (firstfloordoor.forced) {
			"Nothing happens. ";
			return;
		}
		if (firstfloordoor.isopen) {
			"Nothing happens. ";
			return;
		}
		"You hit the button and the door slides open. ";
		firstfloordoor.isopen := true;
		notify(panel1, &closedoor, 2);
	}
;

secondfloorhall: darkroom
	sdesc = "Second Floor Hallway"
	ldesc = {
		"Unlike the first floor hallway, this corridor is coated with a thick layer of dust, not grime and dried mud.  Still, the basic structure is the same.  The floor is a grating of bright metal that glitters beneath your footprints in the dust and ivory walls frame this hallway that leads east and west.  Directly to the north is a flight of steps leading upwards to the first floor.  The security door in the south wall is very similar to the one you encountered above, including the panel embedded into the wall beside it. ";
		if (self.lightsOn) {
			"The room is filled with the hum of the overhead fluorescents. ";
		} else {
			"The overhead fluorescents on this floor are distinctly dark. ";
		}
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if (secondfloordoor.isblocked) {
			"The door to the south has been blocked open by a metal pole in its track. ";
		}
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			if (secondfloordoor.isblocked) {
				"<<global.dispParagraph>>";
			}
			"Whirling red lights and warning alarms fill the corridor. ";
		}
		str := outcapture(stat);
		return str;
	}
	east = outsidecommissary
	west = electricalroom
	south = secondfloordoor
	down = secondfloordoor
	north = firststairs
	up = firststairs
	lightsOn = {
		return global.secondpower;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"It's eerily quiet here. ";
	}
	exitlist = { "The corridor heads west and east.  To the south is a door, and to the north is the stairway up. "; return nil; }
	listContents = {
		"On the floor %you% can see%s% "; listcont(self); ". ";
	}
;

// Almost just like the first, except it can be opened or passed through
// automatically, and doesn't need to be fixed first.
secondfloordoor: doorway, floatingItem, qcontainer
	noun = 'door' 'doorway'
	adjective = 'south' 'southern' 'security' 'north' 'northern'
	sdesc = "security door"
	plural = 'doors'
	location = {
		if ((parserGetMe().location = secondfloorhall) || (parserGetMe().location = secondstairs)) {
			return parserGetMe().location;
		}
		return secondfloorhall;
	}
	doordest = secondstairs
	islocked = true
	isopen = nil
	destination = {
		if (!location.islit) {
			darkTravel();
			return nil;
		}
		if (self.isopen) {
			return self.doordest;
		} else if (global.opendoor) {
			if (!global.secondpower) {
				"With the grid and keypad dark, you can't manage to get the door open. ";
				return nil;
			}
			if (panel2.badgeread) {
				"You enter the security code and the door slides open.\b";
			} else if (isinside(securitybadge, parserGetMe())) {
				"You scan the security badge and enter the code and the door slides open.\b";
			} else {
				"You can't get the door open without the security badge. ";
				return nil;
			}
			self.isopen := true;
			global.opendoor := true;
			panel2.badgeread := nil;
			panel2.codetyped := nil;
			notify(panel2, &closedoor, 2);
			return self.doordest;
		} else {
			"%You%'ll have to open << self.thedesc >> first. ";
			setit(self);
			return nil;
		}
	}
	ldesc = {
		local second := (parserGetMe().location = secondfloorhall);
		if (second)
			"It looks like a fairly solid security door.  Built out of brushed stainless steel that's still fairly shiny after all this time.  White block letters reading \"SUBLEVEL 3\" have been stenciled across its surface.  Built into the wall right beside the door is a control panel. ";
		else
			"It looks like a fairly solid security door.  Built out of brushed stainless steel that's still fairly shiny after all this time.  White block letters reading \"SUBLEVEL 2\" have been stenciled across its surface.  Built into the wall right beside the door is a large red button. ";
		if (self.isopen)
			"The door is currently open. ";
		else
			"The door is closed. ";
	}
	verDoUnlock(actor) = {
		"You're convinced you'll need to use the panel to unlock this door. ";
	}
	verIoPutIn(actor) = {
		if (!self.isopen)
			"Not while the door is closed. ";
	}
	ioPutIn(actor, dobj) = {
		if (dobj = ladder) {
			"That's too large to brace the door open. ";
		} else if (dobj != pole) {
			"Something a little longer might brace the door open. ";
		} else {
			"You wedge the metal pole into the door, preventing it from closing. ";
			pole.moveInto(self);
			self.isblocked := true;
		}
	}
	verDoBraceWith(actor, iobj) = {
		if (self.forced) {
			"Not much point in it now.  That door doesn't look like it will ever close. ";
		} else if (!self.isopen) {
			"Not while the door is closed. ";
		}
	}
	doBraceWith(actor, iobj) = {
		self.ioPutIn(actor, iobj);
	}
	Grab(obj) = {
		if (obj = pole) {
			if (global.secondpower) {
				"As you pull the pole out of the door track, the door slides shut. ";
				self.isopen := nil;
			}
			self.isblocked := nil;
		}
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
	verDoOpen(actor) =
	{
		if (actor.location = firststairs)
			return;
		if (self.isopen) {
			"\^<<self.itisdesc>> already open. ";
		} else if (global.opendoor) {
			// Nothing
		} else if (self.islocked) {
			"\^<<self.itisdesc>> locked. ";
		}
	}
	doOpen(actor) = {
		if (actor.location = secondstairs) {
			if (!global.secondpower) {
				"You hit the button beside the door to open it, but nothing happens. ";
				return nil;
			} else {
				"You press the button and the door opens. ";
			}
		} else {
			if (!global.secondpower) {
				"With the grid and keypad dark, you can't manage to get the door open. ";
				return nil;
			} else if (panel2.badgeread) {
				"You enter the security code and the door slides open.";
			} else if (isinside(securitybadge, parserGetMe())) {
				"You scan the security badge and enter the code and the door slides open.";
			} else {
				"You can't get the door open without the security badge. ";
				return nil;
			}
		}
		self.isopen := true;
		panel2.badgeread := nil;
		panel2.codetyped := nil;
		notify(panel2, &closedoor, 2);
	}
;

panel2: fixeditem
	noun = 'panel' 'hinge' 'hinges'
	adjective = 'small'
	sdesc = "panel"
	location = secondfloorhall
	isopen = nil
	islocked = true
	isbroken = true
	badgeread = nil
	codetyped = nil
	ldesc = {
		"The panel is built into the southern wall, right beside the security door.  It's a smooth plastic rectangle that juts out from the surface of the wall about a half inch.  The majority of its area is taken up by ";
		if (global.secondpower)
			"a brightly illuminated ";
		else
			"an oddly dark ";
		"scanner grid.  You've seen these before; they're designed to react with security badges with embedded microchips.  When the badge is touched to the grid, the chip is activated and the user's identity is read.  Below the grid is a numeric keypad";
		if (global.secondpower && self.badgeread)
			" with glowing number keys";
		else if (!self.badgeread)
			" which is dark";
		else
			", also dark";
		".  In each of the right two corners is a single screw, and you see a pair of hinges along the left side. ";
		if (self.isopen) {
			"The panel is currently open.  On the inside back of the panel, there is a metal-lined socket.  A bundle of wires ending in a metal prong runs out of the wall ";
			if (!self.isbroken) {
				"and into the socket. ";
			} else {
				"and dangles loosely in the air. ";
			}
		}
	}
	closedoor = {
		if (secondfloordoor.isblocked) {
			if (parserGetMe().location = secondfloorhall)
				"<<global.dispParagraph>>The door to the south bangs against the metal pole laying in its track, but stays open. ";
			else if (parserGetMe().location = secondstairs)
				"<<global.dispParagraph>>The door at the top of the stairs bangs against the metal pole laying in its track, but stays open. ";
		} else {
			if (parserGetMe().location = secondfloorhall)
				"<<global.dispParagraph>>The door to the south slides shut. ";
			else if (parserGetMe().location = secondstairs)
				"<<global.dispParagraph>>The door slides shut. ";
			secondfloordoor.isopen := nil;
		}
	}
	typecode = {
		"The door slides open, revealing a stairway to the south. ";
		secondfloordoor.isopen := true;
		self.badgeread := nil;
		self.codetyped := nil;
		codeHint.active := nil;
		notify(self, &closedoor, 2);
	}
	readcard = {
		self.badgeread := true;
		panelHint.active := nil;
		"You touch the security badge to the scanning grid.  The keys on the keypad become lit. ";
		codeHint.active := true;
		return;
	}
	verDoOpen(actor) = {
		"\^<<self.thedesc>> won't budge. ";
	}
	verDoLock(actor) = {
		"\^<<self.thedesc>> isn't something that you can lock. ";
	}
	verDoUnlock(actor) = {
		"\^<<self.thedesc>> isn't something that you can unlock. ";
	}
	verDoScrew(actor) = {
		screws2.verDoScrew(actor);
	}
	verDoScrewWith(actor, iobj) = {
		screws2.verDoScrewWith(actor, iobj);
	}
	doScrew(actor) = {
		screws2.doScrew(actor);
	}
	verDoUnscrew(actor) = {
		screws2.verDoUnscrew(actor);
	}
	verDoUnscrewWith(actor, iobj) = {
		screws2.verDoUnscrewWith(actor, iobj);
	}
	doUnscrew(actor) = {
		screws2.doUnscrew(actor);
	}
	verIoPutIn(actor) = {
		"You can't put anything inside the panel. ";
	}
	verIoPutOn(actor) = {}
	ioPutOn(actor, dobj) = {
		grid2.ioPutOn(actor, dobj);
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	ioTouchTo(actor, dobj) = {
		grid2.ioTouchTo(actor, dobj);
	}
	ioScanOn -> grid2
	ioScanWith -> grid2
;

screws2: fixeditem
	noun = 'screw' 'screws'
	sdesc = "screws"
	location = secondfloorhall
	ldesc = {
		"Two simple steel screws with heads stripped beyond repair. ";
	}
	isThem = true
	verDoScrew(actor) = {
		"These screws are stripped beyond repair. ";
	}
	verDoScrewWith(actor, iobj) = {
		"These screws are stripped beyond repair. ";
	}
	doScrew(actor) = {
		askio(withPrep);
	}
	verDoUnscrew(actor) = {
		"These screws are stripped beyond repair. ";
	}
	verDoUnscrewWith(actor, iobj) = {
		"These screws are stripped beyond repair. ";
	}
	doUnscrew(actor) = {
		askio(withPrep); 
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

keypad2: fixeditem
	noun = 'keypad' 'pad' 'keys'
	adjective = 'numeric' 'number'
	sdesc = "keypad"
	ldesc = {
		"You can see it's a pretty standard numeric keypad.  There are ten off-white plastic buttons, numbered 0 through 9. ";
		if (panel2.badgeread) {
			"The keys on the panel are currently glowing faintly. ";
		}
	}
	location = secondfloorhall
	verIoTypeOn(actor) = {
	}
	ioTypeOn(actor, dobj) = {
		local val;
		if (isclass(dobj, basicStrObj)) {
			if (cvtstr(cvtnum(dobj.value)) = dobj.value) {
				val := cvtnum(dobj.value);
			} else {
				"You can only type numbers on this keypad. ";
				return;
			}
		} else if (!isclass(dobj, basicNumObj)) {
			"You can only type numbers on this keypad. ";
			return;
		} else {
			val := dobj.value;
		}
		if (!global.secondpower || !panel2.badgeread) {
			"Nothing happens. ";
			return;
		}
		if (val != personDB.getCode(securitybadge.person)) {
			"The panel emits a loud buzz and the keypad goes dark. ";
			panel2.badgeread := nil;
			return;
		}
		panel2.typecode;
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

grid2: fixeditem, surface
	noun = 'grid'
	adjective = 'scanning' 'brightly' 'illuminated' 'oddly' 'dark' 'scanner' 'sensor'
	sdesc = "scanning grid"
	location = secondfloorhall
	ldesc = {
		"The scanning grid is a ";
		if (!global.secondpower)
			"dark";
		else
			"softly glowing";
		" piece of glass, four inches square.  There are a number of white lines etched into the glass's surface, forming a pattern of squares. ";
	}
	ioPutOn(actor, dobj) = {
		if (!global.secondpower || (dobj != securitybadge)) {
			"\^%You% touch <<dobj.thedesc>> to <<self.thedesc>> but nothing happens. ";
			door2Hint.active := true;
			return;
		}
		panel2.readcard;
	}
	ioTouchTo(actor, dobj) = {
		if (dobj != securitybadge)
			"Nothing happens. ";
		else
			self.ioPutOn(actor, dobj);
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	verIoScanOn(actor) = {}
	verIoScanWith(actor) = {}
	ioScanOn(actor, dobj) = {
		self.ioPutOn(actor, dobj);
	}
	ioScanWith(actor, dobj) = {
		self.ioPutOn(actor, dobj);
	}
;

electricalroom: darkroom
	sdesc = "Electrical Room"
	ldesc = {
		"Plastic and metal electrical conduits press in on you from every side in this narrow room.  It seems to be the hub for all power distribution at least for this level of the facility, possibly for the entire base.  There are a number of transformers in the back of the room, some power meters spliced into several of the conduits, and a small breaker box in one corner.  The only exit leads east. ";
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the room. ';
		}
		return '';
	}
	east = secondfloorhall
	lightsOn = {
		return global.secondpower;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"You can barely detect a faint humming. ";
	}
	exitlist = { "The only exit you see is to the east. "; return nil; }
	listContents = {
		"In one corner of the room, %you% notice%s% "; listcont(self); ". ";
	}
	smelldesc = "You think you detect the muted tang of ozone. "
;

breakerbox1: fixeditem, openable
	sdesc = "breaker box"
	noun = 'box' 'slots' 'slot'
	adjective = 'breaker'
	location = electricalroom
	ldesc = {
		"The breaker box is a metal-framed box of dull gray built to control the flow of electricity through circuit breakers.  The front is hinged and swings open to access the internals.  The breaker box is currently ";
		if (self.isopen) {
			"open, showing ribbons of metal behind a row of ";
			if (circuitbreaker.location = self) {
				"slots, one of which contains a circuit breaker. ";
			} else {
				"empty slots. ";
			}
		} else {
			"closed. ";
		}
	}
	isopen = nil
	verIoPutIn(actor) = {
		if (!self.isopen)
			"\^<<self.thedesc>> is closed. ";
	}
	ioPutIn(actor, dobj) = {
		if (dobj != circuitbreaker) {
			if (dobj.isiron) {
				"Yeah, there's no death like electrocution death. ";
			} else {
				"That won't fit. ";
			}
			return;
		}
		"You slide the circuit breaker into the breaker box and switch it on.  Almost immediately, the overhead lights begin to glow.  In a moment, they are at full strength. ";
		dobj.moveInto(self);
		circuitbreaker.isActive := true;
		global.secondpower := true;
		if (!global.scSecondFloor) {
			incscore(1);
			global.scSecondFloor := true;
		}
		if (commissaryconsole.isbroken) {
			commissarydoor1.setIsopen(true);
			door2Hint.active := nil;
		}
		if (!secondfloordoor.isblocked)
			secondfloordoor.isopen := nil;
	}
	Grab(obj) = {
		if (circuitbreaker.isActive)
			"Switching the breaker off, you slide it out of the box.  The lights above go dark. ";
		else
			"Taken. ";
		global.secondpower := nil;
		circuitbreaker.isActive := nil;
	}
	verIoPlugIn(actor) = { self.verIoPutIn(actor); }
	ioPlugIn(actor, dobj) = { self.ioPutIn(actor, dobj); }
	doOpen(actor) =
	{
		if (itemcnt(self.contents))
		{
			"Opening "; self.thedesc; " reveals "; listcont(self); ". ";
		}
		else
		"The breaker box swings open, revealing a row of empty slots. ";
		self.isopen := true;
	}
	verDoTake(actor) = "\^<<self.thedesc>> is attached to the wall. "
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

ladder: nestedroom, openable
	sdesc = "aluminum ladder"
	adesc = "an <<self.sdesc>>"
	noun = 'ladder'
	adjective = 'aluminum' 'folding'
	isopen = nil
	location = electricalroom
	bulk = 21
	statusPrep = "on"
	outOfPrep = "off of"
	listendesc = { self.location.listendesc; }
//	isactor = {
//		return self.isopen;
//	}
	isfixed = {
		return self.isopen;
	}
	isListed = {
		if (parserGetMe().location = self)
			return nil;
		if (self.isopen)
			return nil;
		return true;
	}
	heredesc = {
		global.showParagraph;
		"There is an open ladder standing here. ";
	}
	up = {
		if (self.location && self.location.ladderup) {
//			self.location.laddermessage;
			return self.location.ladderup;
		}
		"There's nowhere to go. ";
		return nil;
	}
	down = {
		return self.location;
	}
	ldesc = {
		"A basic aluminum A-frame folding ladder.  This particular one opens up to a height of about six feet. ";
		if (self.isopen)
			"The ladder is currently open. ";
		else
			"The ladder is currently closed. ";
	}
	cantReach(actor) = {
		local dobject := parserGetObj(PO_DOBJ);
		if ((dobject = self) && (actor.location = self)) {
			"While you're standing on it?  I kind of doubt it. ";
			return;
		}
		pass cantReach;
	}
	verDoOpen(actor) = {
		if (self.isopen) {
			caps(); self.thedesc; " <<self.isdesc>> already open! ";
			return;
		}
		if (isinside(self,actor)) {
			"Not while you're carrying it. ";
			return;
		}
		if ((actor.location = duct1) || (actor.location = duct2) || (actor.location = duct3)) {
			"The duct is far too narrow to accommodate the ladder. ";
			return;
		}
	}
	verIoPutIn(actor) = {
		"%You% can't put anything into "; self.thedesc; ". ";
	}
	verDoLookin(actor) = {}
	doLookin(actor) = {
		"There's nothing in "; self.thedesc; ". ";
	}
	verDoSearch(actor) = {
		"%You% find%s% nothing of interest. ";
	}
	exitlist = {
		"%You're% not going anywhere until %you% get%s% <<self.outOfPrep>> <<self.thedesc>>. ";
		return nil;
	}
	verDoSiton(actor) = {
		"Ladders are for climbing, not sitting on. ";
	}
	verDoLieon(actor) = {
		"Ladders are for climbing, not laying on. ";
	}
	doSynonym('Climb') = 'Standon' 'Board'
	verDoClimb(actor) = {
		if (actor.location = self) {
			"%You're% already on "; self.thedesc; "! ";
		}
	}
	doClimb(actor) = {
		if (isinside(self, actor)) {
			"Not while you're carrying it. ";
			return;
		}
		if (!self.isopen) {
			"(Opening it first)\n";
			self.isopen := true;
		}
		"Okay, %you're% now standing on "; self.thedesc; ". ";
		actor.travelTo(self);
	}
	doTake(actor) = {
		if (self.isopen) {
			"(Closing it first)\n";
			self.isopen := nil;
		}
		pass doTake;
	}
	enterRoom(actor) = {}
	verIoBraceWith(actor) = {
		"That's too large to brace the door open. ";
	}
;

outsidecommissary: darkroom
	sdesc = "Outside Commissary"
	ldesc = {
		"The east-west hallway ends here at a sturdy sliding door.  Stenciled across its surface is the word \"COMMISSARY\" in white block letters.  In the wall beside it is a small window and a button.  Other than that, this end of the hallway is similar to the rest of the corridor.  Overhead, the string of fluorescent lights, interrupted only by an air conditioning vent, ";
		if (self.lightsOn)
			"hum as they illuminate the room. ";
		else
			"are conspicuously dark. ";
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the corridor. ';
		}
		return '';
	}
	enterRoom(actor) = {
		if (!global.incommissary) {
			commissaryHint.active := true;
		}
		pass enterRoom;
	}
	east = commissarydoor1
	west = secondfloorhall
	lightsOn = {
		return global.secondpower;
	}
	up = { return self.badexit('You can\'t reach the air conditioning vent. '); }
	laddermessage = "You pull yourself up into the ventilation duct above.  Once inside, you close the vent behind you.\b"
	ladderup = { return vent1; }
	contentsReachable = nil
	cantReach(actor) = {
		"You can't reach it from here. ";
	}
	roomAction(a, v, d, p, i) = {
		if (v = jumpVerb) {
			if (a.location = ladder) {
				uVerb.action(a);
				exit;
			} else {
				"Your vertical leap is not quite up to the task of reaching the ventilation duct. ";
				exit;
			}
		}
		pass roomAction;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"It's eerily quiet here. ";
	}
	exitlist = { "The corridor leads west and there is a door to the east. "; return nil; }
	listContents = {
		"\^%you% can see%s% "; listcont(self); " lying on the floor. ";
	}
;

button: buttonitem, floatingItem
	sdesc = "door control button"
	noun = 'button'
	adjective = 'door' 'control'
	location = outsidecommissary
	ldesc = "A simple button beside the door outside the commissary.  The color, in case you were wondering, is green. "
	doPush(actor) = {
		if (global.secondpower && !commissarydoor1.isopen) {
			"You get a nasty electric shock.  The door, however, stays closed. ";
		} else {
			"Nothing happens. ";
		}
	}
;

window: seethruItem, fixeditem
	sdesc = "commissary window"
	noun = 'window' 'screen'
	adjective = 'wire' 'commissary' 'mesh'
	location = outsidecommissary
	ldesc = "A small square window, about a foot wide, but blocked by a wide mesh wire screen. "
	isfixed = true
	isListed = nil
	thrudesc = {
		if (commissary.lightsOn) {
			"Through the window, you see a room with numerous empty shelves and some kind of control console. ";
		} else {
			"You shine your flashlight through the window, into the room beyond.  The beam of illumination reveals some empty shelving units and some kind of control console. ";
		}
		if (itemcnt(commissaryconsole.contents)) {
			"Sitting on the console you can just make out ";
			listcont(commissaryconsole);
			". ";
		}
		if (itemcnt(commissaryshelves.contents)) {
			"On the shelves, you can see ";
			listcont(commissaryshelves);
			". ";
		}
		"If there's anything on the floor in there, you can't tell. ";
	}
	verDoLookin(actor) = {}
	doLookin(actor) = {
		self.thrudesc;
	}
;

commissarydoor1: doorway, readable
	sdesc = "commissary door"
	noun = 'door'
	adjective = 'commissary'
	plural = 'doors'
	location = outsidecommissary
	otherside = commissarydoor2
	ldesc = {
		if (findRootRoom(parserGetMe()) != commissary)
			"A sturdy sliding door with the word \"COMMISSARY\" stenciled across its surface in white block letters. ";
		else
			"A sturdy sliding door. ";
	}
	readdesc = {
		if (findRootRoom(parserGetMe()) != commissary)
			"The word \"COMMISSARY\" is stenciled across its surface in white block letters. ";
		else
			"There's nothing written on this side of the door. ";
	}

	doordest = commissary
	islockable = nil
	islocked = true
	verDoUnlock(actor) = {
		if (not self.islocked)
			"\^<<self.itisdesc>> not locked! ";
		"There's no obvious lock on the door. ";
	}
	verDoClose(actor) = {
		"It won't budge. ";
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
;

commissarydoor2: doorway
	sdesc = "commissary door"
	doordest = outsidecommissary
	otherside = commissarydoor1
	islockable = nil
	islocked = true
;

vent1: doorway
	sdesc = "vent"
	noun = 'vent'
	adjective = 'air' 'conditioner' 'conditioning' 'duct'
	location = outsidecommissary
	ldesc = {
		"An air conditioner vent in the ceiling of the hallway. ";
		pass ldesc;
	}
	doordest = duct1
	otherside = vent2
	islockable = nil
	isobstacle = true
	noAutoOpen = nil
	isReachable(actor) = {
		if ((actor.location = ladder) && (ladder.location = self.location))
			return true;
		return nil;
	}
	verDoTake(actor) = "It's attached to the ceiling. "
	doOpen(actor) =
	{
		"The vent swings open. ";
		self.setIsopen(true);
	}
	doClose(actor) =
	{
		"You swing the vent closed. ";
		self.setIsopen(nil);
	}
;

duct1: darkroom
	sdesc = "Ventilation Duct"
	ldesc = {
		"A narrow ventilation duct with walls of smooth galvanized steel.  There's a";
		if (vent1.isopen) {
			"n open";
		}
		" vent directly below you, and the duct runs east to west. ";
	}
	east = duct2
	west = duct3
	down = vent2
	exitlist = { "This duct runs east-west and a vent down provides an exit from the duct altogether."; return nil; }
	scorchdesc = {
		"It appears that there's been an explosion in this section of ductwork.  A black-edged hole is visible in the floor and the room is hazy with smoke. ";
	}
;

vent2: doorway
	sdesc = "vent"
	noun = 'vent'
	adjective = 'air' 'conditioner' 'conditioning' 'duct'
	location = duct1
	ldesc = {
		"An air conditioner vent leading down. ";
		pass ldesc;
	}
	isobstacle = true
	doordest = outsidecommissary
	otherside = vent1
	islockable = nil
	noAutoOpen = nil
	verDoTake(actor) = "It's attached to the ductwork. "
	doOpen(actor) =
	{
		"The vent swings open. ";
		self.setIsopen(true);
	}
	doClose(actor) =
	{
		"You swing the vent closed. ";
		self.setIsopen(nil);
	}
;

duct2: darkroom
	sdesc = "Ventilation Duct"
	ldesc = {
		"The east end of a narrow ventilation duct with walls of smooth galvanized steel.  The duct runs back to the west.  You notice that the ceiling panel beneath you is ";
		if (vent3.isopen)
			"open. ";
		else
			"loose. ";
	}
	west = duct1
	down = vent3
	exitlist = {
		if (vent3.isopen)
			"The only exits are west along the duct and down through the panel. ";
		else
			"The only exit is west along the duct. ";
		return nil;
	}
	scorchdesc = {
		"It appears that there's been an explosion in this section of ductwork.  A black-edged hole is visible in the floor and the room is hazy with smoke. ";
	}
;

vent3: doorway
	sdesc = "ceiling panel"
	noun = 'panel'
	adjective = 'air' 'conditioner' 'conditioning' 'duct' 'ceiling'
	location = duct2
	ldesc = {
		if (self.isopen)
			"An open";
		else
			"A loose";
		" ceiling panel below you. ";
	}
	doordest = commissary
	otherside = vent4
	islockable = nil
	isobstacle = true
	noAutoOpen = nil
	verDoTake(actor) = "It's too large and unwieldy to bother with. "
	verDoPush(actor) = {}
	doPush(actor) = {
		if (self.isopen) {
			self.doClose(actor);
		} else {
			self.doOpen(actor);
		}
	}
	verDoPull(actor) = {}
	doPull(actor) = {
		self.doPush(actor);
	}
	doOpen(actor) =
	{
		"You slide the panel out of the way. ";
		self.setIsopen(true);
	}
	doClose(actor) =
	{
		"You slide the panel back in place. ";
		self.setIsopen(nil);
	}
	verDoMove(actor) = {}
	doMove(actor) = {
		if (self.isopen)
			self.doClose(actor);
		else
			self.doOpen(actor);
	}
;

duct3: darkroom
	sdesc = "Ventilation Duct"
	ldesc = "A narrow ventilation duct with walls of smooth galvanized steel.  The cramped passage continues to the east, but to the west it angles sharply upwards in an unclimbable shaft. "
	east = duct1
	west = { return self.badexit('The sides of the duct are too steep in that direction. '); }
	up = { return self.badexit('The sides of the duct are too steep in that direction. '); }
	exitlist = { "The duct only allows movement to the east. "; return nil; }
	scorchdesc = {
		"It appears that there's been an explosion in this section of ductwork.  A black-edged hole is visible in the floor and the room is hazy with smoke. ";
	}
;

commissary: darkroom, nestedroom
	sdesc = "Commissary"
	ldesc = {
		"You see a room that was probably once a busy storage center, but is now little more than an empty room.  Though the numerous shelves remain, their contents were probably removed several decades before you were born.  ";
		if (commissaryshelves.grenadeblast)
			"Several of the shelves, you can't help but notice, have been terribly mangled by an explosion. ";
		"Against the back wall a ";
		if (commissaryconsole.grenadeblast)
			"ruined";
		else
			"lonely";
		" control console remains the only other feature inside this room.  Up above, the fluorescent lights are ";
		if (self.lightsOn)
			"brightly glowing. ";
		else
			"notably dark. ";
		if (vent4.isopen) {
			"In the midst of the overhead lights is an open ceiling panel. ";
		}
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the room. ';
		}
		return '';
	}
	enterRoom(actor) = {
		commissaryHint.active := nil;
		global.incommissary := true;
		pass enterRoom;
	}
	roomAction(a, v, d, p, i) = {
		if ((v = standVerb) && (a.location = self)) {
			"%You're% already standing! ";
			exit;
		}
		if (v = jumpVerb) {
			if (a.location = ladder) {
				uVerb.action(a);
				exit;
			} else {
				"Your vertical leap is not quite up to the task of reaching the ceiling panel. ";
				exit;
			}
		}
		pass roomAction;
	}
	location = outsidecommissary
	reachable = ([commissarydoor1] + self)
	canReachContents = nil
	contentsReachable = nil
//	contentsVisible = nil
	isqcontainer = true
	isListed = nil
	isdroploc = true
	isRoom = true
	lightsOn = {
		return global.secondpower;
	}
	west = commissarydoor2
	up = { return self.badexit('You can\'t reach the vent. '); }
	isfixed = true
	cantReachRoom(otherRoom) = {
		"%You% can't reach that from <<self.thedesc>>. ";
	}
	laddermessage = "You pull yourself up into the ventilation duct above.\b"
	ladderup = { return vent4; }
	cantReach(actor) = {
		"You can't reach it from here. ";
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"It's eerily quiet here. ";
	}
	exitlist = { "The door lies to the west. "; return nil; }
	listContents = {
		"On the ground %you% see%s% "; listcont(self); ". ";
	}
	verGrab(obj) = {
		if (!isinside(parserGetMe(), commissary) && isinside(parserGetMe(), outsidecommissary)) {
			"You can't reach it from here. ";
		}
	}
;

commissaryshelves: fixeditem, surface
	noun = 'shelves' 'shelf'
	adjective = 'numerous'
	sdesc = "shelves"
	location = commissary
	ldesc = {
		"The shelves stretch along three of the four walls, reaching from floor to ceiling. ";
		if (commissaryshelves.grenadeblast)
			"A large section of the shelves, you can't help but notice, have been terribly mangled by an explosion.  Most, however, still provide adequate storage. ";
		if (itemcnt(self.contents)) {
			"On "; self.thedesc; " %you% see%s% "; listcont(self); ". ";
		} else {
			"There's nothing on "; self.thedesc; ". ";
		}
	}
	verDoClimb(actor) = {}
	doClimb(actor) = {
		"With a bit of effort, you manage to climb the shelves up to the opening in the ceiling and crawl inside.\b";
		vent3.setIsopen(true);
		actor.travelTo(duct2);
	}
	verDoStandon(actor) = {}
	doStandon(actor) = {
		self.doClimb(actor);
	}
	handleExplosion(actor, playerloc, heard) = {
		local o;
		scorchmark.moveInto(nil);
		while (length(self.contents) > 0) {
			o := car(self.contents);
			if (rand(2) = 1)
				o.moveInto(nil);
			else
				o.moveInto(commissary);
		}
	}
;

commissaryconsole: fixeditem, surface
	sdesc = "control console"
	noun = 'console' 'control' 'controls'
	adjective = 'control' 'unlabeled'
	location = commissary
	ldesc = {
		if (commissaryconsole.grenadeblast)
			"It's little more than a short stump now, a thin round cylinder stretching up a couple feet from the floor and ending in a jagged-edged jumble of plastic and wire. ";
		else
			"It's primarily a smooth, flat black surface.  There are various unlabeled controls and a single large switch. ";
		pass ldesc;
	}
	isbroken = nil
	verDoPush(actor) = {}
	doPush(actor) = {
		if ((self.isbroken) || (!global.secondpower)) {
			"Nothing happens. ";
			door2Hint.active := true;
			return;
		}
		if (commissarydoor1.isopen) {
			"The door slides closed. ";
			commissarydoor1.setIsopen(nil);
		} else {
			"The door slides open. ";
			door2Hint.active := nil;
			commissarydoor1.setIsopen(true);
		}
	}
	verDoFlip(actor) = {}
	doFlip(actor) = {
		self.doPush(actor);
	}
	smelldesc = {
		if (self.isbroken) {
			"There's a definite scent of smoke and ozone. ";
		} else {
			pass smelldesc;
		}
	}
	handleExplosion(actor, playerloc, heard) = {
		scorchmark.moveInto(nil);
		self.isbroken := true;
	}
	verDoClimb(actor) = {}
	doClimb(actor) = {
		"The console is a little too wobbly to climb.  However, the shelves might be scalable. ";
	}
	verDoStandon(actor) = {}
	doStandon(actor) = {
		"The console is a little too wobbly to climb.  However, the shelves might be scalable. ";
	}
	verDoSiton(actor) = {}
	doSiton(actor) = {
		"The console is a little too wobbly to climb.  However, the shelves might be scalable. ";
	}
;

commissaryswitch: switchItem
	sdesc = "console switch"
	noun = 'switch'
	adjective = 'control' 'single' 'large' 'console'
	location = commissary
	ldesc = "The large switch on the console is the only control you can see with an obvious function. "
	doPush -> commissaryconsole
	verDoFlip(actor) = {}
	doFlip(actor) = {
		commissaryconsole.doPush(actor);
	}
;

// It is only here to be shot.  If the player drinks the water, death soon
// follows!
waterbottle: transparentItem, qcontainer, openable
	sdesc = "water bottle"
	noun = 'bottle'
	adjective = 'plastic' 'water'
	location = commissaryconsole
	maxbulk = 0
	bulk = 7
	isshot = nil
	isopen = nil
	ldesc = {
		"A clear plastic bottle large enough to carry a liter of liquid. ";
		if (water.location = self)
			"The bottle appears to be about half full of filthy water. ";
		if (self.isshot)
			"There are two holes in the side of the bottle, each about a half-inch in diameter. ";
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if (weapon = pistol) {
			if ((self.location = commissaryconsole) && (parserGetMe().location = outsidecommissary)) {
				"You stick the barrel of the pistol in a gap between the window wires and take aim at the water bottle.  With the wire stabilizing your pistol, putting a bullet into the bottle is a simple task.\bA perfectly circular hole appears near the bottom of the bottle";
				self.isshot := true;
				if (water.location = self) {
					"; dirty water pours out of it and splashes all over the console. ";
					commissaryconsole.isbroken := true;
					if (global.secondpower) {
						"Amid the sound of sparks and transistors popping, thin plumes of smoke begin to rise up from the console. ";
						if (!commissarydoor1.isopen) {
							"Suddenly, the commissary door slides open most of the way before grinding to a halt. ";
							commissarydoor1.setIsopen(true);
							door2Hint.active := nil;
							water.location := nil;
						}
					} else {
						door2Hint.active := true;
					}
				} else {
					". ";
				}
			} else {
				"You get the feeling that that would be a waste of ammunition. ";
			}
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			"Attacking <<self.thedesc>> doesn't appear productive. ";
		}
		if (weapon = parserGetMe()) {
			"Your fists do little damage. ";
		}
	}
	smelldesc = "It smells rather repugnant. "

	verIoPourOn(actor) = {}
	verDoPourOn(actor, io) = {
		if (water.location = self) {
			water.verDoPourIn(actor, io);
			return;
		}
		"But <<self.thedesc>> is empty. ";
	}
	verDoPourIn(actor, io) = {
		if (water.location = self) {
			water.verDoPourIn(actor, io);
			return;
		}
		"But <<self.thedesc>> is empty. ";
	}
	verIoPourIn(actor) = {}
	doPourIn -> water
	ioPourIn(actor, dobj) = {
		if (dobj.location = self) {
			"\^<<dobj.thedesc>> <<dobj.isdesc>> already inside of <<self.thedesc>>. ";
			return;
		}
		"You pour <<dobj.thedesc>> into <<self.thedesc>>. ";
		dobj.moveInto(self);
	}
	doPourOn(actor, io) = {
		if (water.location = self) {
			"%You% pour <<water.thedesc>> all over <<io.thedesc>>, thoroughly wetting <<io.itobjdesc>>. ";
			water.moveInto(nil);
			return;
		}
		"But <<self.thedesc>> is empty. ";
	}
	doEmpty(actor)=
	{
		if (not self.isopen) {
			"\^<<self.thedesc>> is closed.";
		} else {
			"You empty the contents of <<self.thedesc>> onto the ground.";
			moveFromTo(self, nil);
		}
	}
;

water: item, liquid
	sdesc = "some filthy water"
	adesc = "some filthy water"
	thedesc = "the filthy water"
	noun = 'water'
	adjective = 'filthy'
	location = waterbottle
	trieddrink = nil
	ldesc = "You'd guess that water sitting around for a hundred years would look a lot like this.  Even though sealed, this water appears to have incubated quite the collection of nastiness. "
	warn = 0
	verDoTake(actor) = {
		"Your hands aren't waterproof. ";
	}
	verDoDrink(actor) = {}
	doDrink(actor) = {
		if (!self.trieddrink) {
			"If you really want to, you can drink it.  However, I wouldn't recommend it. ";
			self.trieddrink := true;
		} else {
			"Closing your eyes, you gulp down most of the disgusting liquid.  Some of it goes dripping down your shirt as you gag at the taste. ";
			global.sicktime := 1;
			self.moveInto(nil);
		}
	}
	smelldesc = "It smells rather repugnant. "
;

vent4: doorway
	sdesc = "ceiling panel"
	noun = 'panel'
	adjective = 'air' 'conditioner' 'conditioning' 'duct' 'ceiling'
	location = commissary
	ldesc = {
		"A loose ceiling panel far above you. ";
		pass ldesc;
	}
	doordest = duct2
	otherside = vent3
	islockable = nil
	isobstacle = true
	noAutoOpen = nil
	isReachable(actor) = {
		if ((actor.location = ladder) && (ladder.location = self.location))
			return true;
		return nil;
	}
	dobjGen(a, v, i, p) = {
		if (v <> askVerb and v <> tellVerb and v <> inspectVerb) {
			if ((a.location != ladder) || (ladder.location != self.location)) {
				"You can't quite reach it. ";
				exit;
			}
		}
	}
	iobjGen(a, v, d, p) = { self.dobjGen(a, v, d, p); }
	verDoTake(actor) = "It's too large and unwieldy to bother with. "
	verDoPush(actor) = {}
	doPush(actor) = {
		if (self.isopen) {
			self.doClose(actor);
		} else {
			self.doOpen(actor);
		}
	}
	verDoPull(actor) = {}
	doPull(actor) = {
		self.doPush(actor);
	}
	doOpen(actor) =
	{
		"You slide the panel out of the way. ";
		self.setIsopen(true);
	}
	doClose(actor) =
	{
		"You slide the panel back in place. ";
		self.setIsopen(nil);
	}
	verDoMove(actor) = {}
	doMove(actor) = {
		if (self.isopen)
			self.doClose(actor);
		else
			self.doOpen(actor);
	}
;

radsuit: clothingItem
	noun = 'radsuit' 'suit'
	adjective = 'rad' 'radiation'
	sdesc = "radiation suit"
	location = commissary
	bulk = 15
	weight = 8
	ldesc = "A full body suit with a tinted plexiglass mask and respiratory filters.  The suit is made out of some kind of shiny fabric you can't immediately identify.  While it has an almost mirror-like surface, it's also very flexible. "
	verIoAttachTo(actor) = {}
	ioAttachTo(actor, dobj) = {
		dobj.doWear(actor);
	}
	verDoWear(actor) = {
		if (uniform2.isworn)
			"\^<<self.thedesc>> won't fit over <<uniform2.thedesc>>. ";
		else if (uniform1.isworn)
			"\^<<self.thedesc>> won't fit over <<uniform1.thedesc>>. ";
		else
			pass verDoWear;
	}
;

secondstairs: darkroom
	sdesc = "Top of Stairs"
	ldesc = {
		"You are standing on a landing at the top of a flight of steps that runs down to the south.  To the north is a steel sliding door, with a button on the wall beside it, and a ";
		if (self.lightsOn)
			"brightly illuminated";
		else
			"dark";
		" lamp above. ";
	}
	specialdesc = {
		local str, stat;
		stat := outcapture(true);
		if (secondfloordoor.isblocked) {
			"The door to the north has been blocked open by a metal pole in its track. ";
		}
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			if (secondfloordoor.isblocked) {
				"<<global.dispParagraph>>";
			}
			"Whirling red lights and warning alarms fill the landing. ";
		}
		str := outcapture(stat);
		return str;
	}
	north = {
		if (!secondfloordoor.isblocked && !secondfloordoor.isopen) {
			if (!global.secondpower) {
				return self.badexit('You hit the button beside the door to open it, but nothing happens. ');
			} else {
				if (!secondfloordoor.isopen) {
					"You hit the button beside the door to open it, then pass through.\b";
					secondfloordoor.isopen := true;
					notify(panel2, &closedoor, 2);
				}
			}
		}
		return secondfloorhall;
	}
	lightsOn = {
		return global.secondpower;
	}
	south = thirdfloorhall
	down = thirdfloorhall
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"It's eerily quiet here. ";
	}
	exitlist = { "The security door lies north and the stairs lead down to the south. "; return nil; }
;

redbutton2: buttonitem
	sdesc = "red button"
	adjective = 'red' 'large'
	location = secondstairs
	doPush(actor) = {
		if (!global.secondpower || secondfloordoor.isopen) {
			"You hit the button beside the door to open it, but nothing happens. ";
		} else {
			"You hit the button and the door slides open. ";
			secondfloordoor.isopen := true;
			notify(panel2, &closedoor, 2);
		}
	}
;

thirdfloorhall: darkroom
	sdesc = "Third Floor Hallway"
	ldesc = {
		"Here the walls, floor, and even the air all seem to be permeated by a deep, resonating hum.  A faint odor also catches your attention: the scent of ozone laced with a hint of something burning.  The hall, however, doesn't appear to contain anything out of the ordinary.  Beside the stairway to the north that leads up to a security door, there are automated doors to the east and south, plus an entryway to the west. ";
		if (self.lightsOn) {
			"The room is filled with the grotesque bluish glow of the overhead fluorescents. ";
		} else {
			"The overhead fluorescents on this floor are distinctly dark. ";
		}
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the corridor. ';
		}
		return '';
	}
	enterRoom(actor) = {
		if (!global.inthird) {
			global.inthird := true;
			powerHint.active := true;
		}
		pass enterRoom;
	}
	east = {
		if (global.thirdpower)
			return self.badexit('You hear a grinding noise coming from the automatic door.  Apparently its mechanism has decayed beyond repair. ');
		else
			return self.badexit('You approach the door but it does not open. ');
	}
	south = {
		if (global.thirdpower)
			return self.badexit('You approach the automatic door but nothing happens.  It doesn\'t seem to be operational anymore. ');
		else
			return self.badexit('You approach the door but it does not open. ');
	}
	west = powerplant
	north = secondstairs
	up = secondstairs
	lightsOn = {
		return global.thirdpower;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"There's a deep resonance in the air.  It seems to be coming from all directions. ";
	}
	exitlist = { "Besides the stairs to the north, there are doors east, south, and west. "; return nil; }
	listContents = {
		"\^%you% notice%s% "; listcont(self); " lying on the ground. ";
	}
;

powerplant: darkroom
	sdesc = "Power Plant"
	ldesc = {
		"Every molecule of your body seems to vibrate in tune with the great turbines of the fusion generator that sits before you.  What you can see of it is an imposing edifice, slightly curved as if you are seeing one small section of a very large ring.  Massive pipes and conduits link to the power plant from numerous angles, connecting at multiple points on its surface.  Anchored to the ground beside the fusion generator is a console with controls and several small screens.  You also spot a gray breaker box nestled among one bundle of wires.  To the east is an open doorway, and south of this room is a corridor shrouded in vapor. ";
		if (self.lightsOn) {
			"The room is filled with the grotesque bluish glow of the overhead fluorescents. ";
		} else {
			"The overhead fluorescents on this floor are distinctly dark. ";
		}
	}
	specialdesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the room. ';
		}
		return '';
	}
	east = thirdfloorhall
	south = steamy
	lightsOn = {
		return global.thirdpower;
	}
	radmessage = 'Your Geiger counter is shrieking a blare of static and a piercing alarm bell.  A glance at the indicator reveals a red arrow pointing south. '
	listendesc= {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"You can't hear anything over the loud hum of the generator. ";
	}
	exitlist = { "The only visible exits lie east and south. "; return nil; }
	listContents = {
		"Discarded on the ground %you% can see%s% "; listcont(self); ". ";
	}
	smelldesc = "You think you detect the muted tang of ozone. "
	handleExplosion(actor, playerloc, heard) = {
		if (heard)
			"It is immediately followed by a deafening alarm and red ";
		else
			"<<global.dispParagraph>>A sudden alarm sounds, both shrill and deafeningly loud.  Red";
		" whirling lights descend from cleverly hidden compartments in the ceiling.  Before panic can even set in, however, a tremendous explosion rocks the building.  A wave of fire engulfs the room, immolating you almost instantly. ";
		lose(true);
	}
;

breakerbox2: fixeditem, openable
	sdesc = "breaker box"
	noun = 'box' 'slot' 'slots'
	adjective = 'breaker'
	location = powerplant
	ldesc = {
		"The breaker box is a metal-framed box of dull gray built to control the flow of electricity through circuit breakers.  The front is hinged and swings open to access the internals.  The breaker box is currently ";
		if (self.isopen) {
			"open, showing ribbons of metal behind a row of ";
			if (circuitbreaker.location = self) {
				"slots, one of which contains a circuit breaker. ";
			} else {
				"empty slots. ";
			}
		} else {
			"closed. ";
		}
	}
	isopen = nil
	verIoPutIn(actor) = {
		if (!isopen)
			"\^<<self.thedesc>> is closed. ";
	}
	ioPutIn(actor, dobj) = {
		if (dobj != circuitbreaker) {
			"That won't fit. ";
			return;
		}
		"You slide the circuit breaker into the breaker box and switch it on.  Almost immediately, the overhead lights begin to glow.  In a moment, they are at full strength. ";
		dobj.moveInto(self);
		powerHint.active := nil;
		global.thirdpower := true;
		circuitbreaker.isActive := true;
		if (!global.scThirdFloor) {
			incscore(1);
			global.scThirdFloor := true;
		}
	}
	verIoPlugIn(actor) = { self.verIoPutIn(actor); }
	ioPlugIn(actor, dobj) = { self.ioPutIn(actor, dobj); }
	Grab(obj) = {
		if (circuitbreaker.isActive)
			"Switching the breaker off, you slide it out of the box.  The lights above go dark. ";
		else
			"Taken. ";
		global.thirdpower := nil;
		circuitbreaker.isActive := nil;
	}
	doOpen(actor) =
	{
		if (itemcnt(self.contents))
		{
			"Opening "; self.thedesc; " reveals "; listcont(self); ". ";
		}
		else
		"The breaker box swings open, revealing a row of empty slots. ";
		self.isopen := true;
	}
	verDoTake(actor) = "\^<<self.thedesc>> is attached to the wall. "
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

generator: fixeditem, knownTopic
	noun = 'plant' 'electricity' 'turbines' 'generator' 'reactor'
	adjective = 'power' 'fusion' 'great'
	sdesc = "power plant"
	location = powerplant
	ldesc = "An imposing edifice, slightly curved as if you are seeing one small section of a very large ring.  Massive pipes and conduits link to it from numerous angles, connecting at multiple points on its surface. "
	explode = {
		if (global.chapter = 2) {
			"<<global.dispParagraph>>Deep within the heart of the base's fusion reactor, a coil of superheated plasma finally shatters the magnetic bottle containing it.   Unrestrained, the particle reaction escalates beyond any physical barrier.  The base, its inhabitants, and everything else within a half-mile radius is instantly vaporized by the explosion.  Unfortunately, that includes you. ";
			lose(true);
		}
	}
	listendesc = "It thrums with a deep power. "
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	smelldesc = "You think you detect the muted tang of ozone. "
;

// No real description of the console buttons is available.  The hesitation
// message provides a safety net to players who may feel the urge to mess
// with them.  Sort of a not-quite-instant death.
console2: fixeditem
	sdesc = "control console"
	noun = 'console' 'controls'
	adjective = 'control'
	location = powerplant
	ldesc = "This large control console is fastened to the floor right in front of the fusion generator.  Along the left side are a number of screens, displaying information about the status of the power plant.  To the right of these are a series of switches and buttons, only a few of which are clearly labeled.  At the top is a red button marked \"CYCLE\", followed by a white one with \"START UP\" written beside it.  Below them, a black button labeled \"SHUTDOWN\", a blue one marked \"VENT\", and a green button indicated as \"SHUNT\".  The rest of the controls are beyond your comprehension. "
	lasttried = -5
	buttonpushed = nil
;

screens: fixeditem, readable
	noun = 'screen' 'screens'
	adjective = 'several' 'small'
	location = powerplant
	ldesc = {
		local rounds, min, sec, txt;
		if (global.genvented) {
			rounds := 100 - (global.turnsofar - global.venttime);
			min := rounds/60;
			sec := rounds - (min * 60);
			sec := cvtstr(sec);
			if (length(sec)<2) {
				sec := '0' + sec;
			}
			"Very bad news.  The coolant meter is currently at zero and the temperature gauge is rising at an alarming rate.  There's a warning message that a run-away thermal reaction is underway inside the reactor.  There's an ominous countdown clock, dwindling before your eyes: ";
			say(min);
			":";
			say(sec);
		} else {
			"This fusion generator has been running for over a hundred years, though it doesn't look like it will last much longer.  You're no nuclear physicist, but judging by these warnings in big red letters, the lack of regular maintenance has taken its toll on the mechanisms of the plant.  While the fuel reservoirs are still in the green, the coolant levels are practically empty, there are an alarming number of shorted circuits across the facility, and the electromagnetic bottle containing the superheated plasma is on the verge of collapsing.  However, the readings don't give you any indication of when a failure might occur. ";
		}
	}
	
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;	

class genbutton: buttonitem
	color = "clear"
	label = "NONE"
	sdesc = "<<self.color>> button"
	ldesc = "A basic <<self.color>> button on the generator control console.  It has the word \"<<self.label>>\" stenciled beside it. "
	location = powerplant
	confirmPush = {
		if ((global.turnsofar - console2.lasttried>5) && !buttonpushed) {
			"You hesitate for a moment, intimidated by the reactor's power and complexity. ";
			console2.lasttried := global.turnsofar;
			return nil;
		}
		return true;
	}
	doPush(actor) = {
		if (confirmPush) {
			"There's a click, but nothing else seems to happen. ";
			buttonpushed := true;
		}
	}
;

genbutton1: genbutton
	adjective = 'red' 'cycle'
	color = "red"
	label = "CYCLE"
	doPush(actor) = {
		if (confirmPush) {
			"For a second, nothing happens.  Then you hear a loud clanking noise from the generator.  It lasts for a few seconds, then stops. ";
			buttonpushed := true;
		}
	}
;

genbutton2: genbutton
	adjective = 'white' 'start up' 'start' 'up'
	color = "white"
	label = "START UP"
;

genbutton3: genbutton
	adjective = 'black' 'shutdown'
	color = "black"
	label = "SHUTDOWN"
	doPush(actor) = {
		if (confirmPush) {
			"A loud buzzer sounds, and an armada of meaningless technical information sails across one of the screens. ";
			buttonpushed := true;
		}
	}
;

genbutton4: genbutton
	adjective = 'blue' 'vent'
	color = "blue"
	label = "VENT"
	doPush(actor) = {
		if (confirmPush) {
			if (!global.genvented) {
				buttonpushed := true;
				global.genvented := true;
				global.venttime := global.turnsofar;
				crowd.moveInto(nil);
				confguard.gone := true;
				confguard.moveInto(nil);
				guard.reassured := true;
				guard.moveInto(nil);
				phantomguard.moveInto(nil);
				grenadelauncher.moveInto(nil);
				notify(self, &alarm, 2);
				notify(generator, &explode, 101);
			}
			pass doPush;
		}
	}
	alarm = {
		if (findRootRoom(parserGetMe()).lightsOn) {
			"<<global.dispParagraph>>Without warning, an alarm klaxon sounds.  Cleverly hidden red lights in the ceiling come to life, bathing the room in an eerie crimson glow. ";
		} else {
			"<<global.dispParagraph>>You hear the faint sounds of an alarm coming from somewhere nearby. ";
		}
		reactorHint.active := true;
		if (parserGetMe().location = powerplant) {
			"The information displayed on the screens changes, flashing in a frantic pattern. ";
		}
	}
;

genbutton5: genbutton
	adjective = 'green' 'shunt'
	color = "green"
	label = "SHUNT"
	doPush(actor) = {
		if (confirmPush) {
			buttonpushed := true;
			"As soon as you bush the button, the vibrations in the room begin to subside.  Then as quickly as they started to fade, the vibrations return and you can't help but notice that they are increasing in intensity.  In fact, the oscillations become so violent that you can barely keep to your feet.  As several pipes burst open under the stress, spraying out superheated jets of gas, you try to scramble out of the room.  Unfortunately, a pipe directly above you splits open, washing you in a violently corrosive substance.  You survive just long enough to discover an entirely new definition of pain. ";
			lose(true);
		}
	}
;

// This is the dreaded radioactive room.
steamy: darkroom
	sdesc = "Steamy Passage"
	ldesc = {
		"Several of the thick pipes from the power plant run through this brief north-south hallway.  Three of them appear to have developed cracks over the years, as they are swathed in clouds of steam that trail upwards and disappear into a vent on the ceiling.  Fortunately, none of the steam completely obstructs movement to either the doorway to your north or the automated sliding doors to the east. ";
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the corridor. ';
		}
		return '';
	}
	north = powerplant
	east = labdoor
	lightsOn = {
		return global.thirdpower;
	}
	radmessage = 'The Geiger counter is alive with noise.  The combination of static and its deafening alarm are nearly deafening.  The display has flashing red arrows pointing in all directions. '
	enterRoom(actor) = {
		radExposure();
		pass enterRoom;
	}
	roomAction(a, v, d, p, i) = {
		radExposure();
		pass roomAction;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		"All you can hear is the hiss of vapor escaping from the pipes. ";
	}
	smelldesc = "The air here is humid, with an acrid scent of ozone. "
	exitlist = { "This passage only allows movement to the north and east. "; return nil; }
	listContents = {
		"Through the swirls of steam, %you% can see%s% "; listcont(self); " on the floor. ";
	}
	scorchdesc = {
		"It appears that there's been an explosion in this passage.  There's a small crater in the ground, surrounded by blackened scorch marks that radiate outwards.  Several more of the pipes have visible cracks in them. ";
	}
;

// Easter egg: the rad poisoning can be dropped, making the player instantly
// healed.
radpoisoning: scenery
	location = nil
	sdesc = "radiation poisoning"
	noun = 'poisoning' 'sickness'
	adjective = 'radiation' 'rad'
	bulk = 0
	weight = 0
	ldesc = "It looks a little bit like cellular decay and genetic damage. "
	smelldesc = "It smells like burning. "
	verDoDrop(actor) = {}
	doDrop(actor) = {
		local i;
		"The radiation poisoning dissolves into the air.  You feel a lot better. ";
		self.moveInto(nil);
		global.radtime := 0;
		actor.health := 0;
		for (i:=1; i<=6; i++) {
			global.radlevel[i] := nil;
		}
	}
;	

radExposure: function() {
	if (global.pilltime > 0)
		return;
	if (radsuit.isworn)
		return;
	global.radtime++;
	radpoisoning.moveInto(parserGetMe());
	radHint.active := true;
}

// This daemon handles decaying health.
healthDaemon: function(val) {
	local level, i, len;
	if (global.pilltime > 0) {
		global.pilltime--;
		if (global.pilltime = 1) {
			"<<global.dispParagraph>>The tingly sensation seems to be lessening. ";
		}
		if (global.pilltime = 0) {
			"<<global.dispParagraph>>The tingly sensation has faded completely. ";
		}
	}
	if (global.radtime > 0) {
		global.radtime++;
		level := global.radtime / 6;
		if ((level > 0) && (!global.radlevel[level])) {
			global.radlevel[level] := true;
			setHealth(level);
			switch (level) {
				case 1:
					"<<global.dispParagraph>>You find yourself wondering why your mouth is parched, like you have a mouthful of sand. "; break;
				case 2:
					"<<global.dispParagraph>>You feel a wave of weakness come over you.  The worst of it passes quickly, though you still feel somewhat feeble. "; break;
				case 3:
					if (global.sicktime < 20)
						"<<global.dispParagraph>>Your stomach suddenly cramps up and you find yourself vomiting up your last meal of survival rations.  Gasping, you wipe a trickle of blood from your mouth. ";
					else
						"<<global.dispParagraph>>Your stomach suddenly cramps up again and you find yourself dry heaving for almost a full minute.  Gasping, you wipe a trickle of blood from your mouth. ";
					break;
				case 4:
					"<<global.dispParagraph>>Feeling something slick on your fingers, you look down at your arm to find an open bleeding sore on your arm. "; break;
				case 5:
					"<<global.dispParagraph>>Your vision suddenly turns red and you wipe blood from your eye.  Another surge of weakness overcomes you and you find yourself on the floor, coughing up blood. ";
					len := length(parserGetMe().contents);
					for (i:=0; i<len; i++) {
						parserGetMe().contents[1].moveInto(parserGetMe().location);
					}
					break;
				case 6:
					"<<global.dispParagraph>>A wave of dizziness comes over you and you can't stop from collapsing limp to the floor.  You lie there, vainly struggling to get to your feet.  Eventually your body shuts down and your life fades away into darkness. ";
					lose(true);
					break;
			}
		}
	}
	if (global.sicktime > 0) {
		setHealth(global.sicktime / 5);
		switch (global.sicktime) {
			case 5:
				"<<global.dispParagraph>>Your stomach suddenly cramps and you involuntarily double over.  After a moment, it passes. "; break;
			case 10:
				"<<global.dispParagraph>>You suddenly break out in a cold sweat.  Your body shivers uncontrollably. "; break;
			case 15:
				"<<global.dispParagraph>>Your vision blurs without warning, everything you see swirls chaotically.  When your sight clears, you pick yourself up off the floor. ";
				len := length(parserGetMe().contents);
				for (i:=0; i<len; i++) {
					parserGetMe().contents[1].moveInto(parserGetMe().location);
				}
				break;
			case 20:
				if (global.radtime < 12)
					"<<global.dispParagraph>>The stomach cramps return and you find yourself vomiting up your last meal of survival rations. ";
				else
					"<<global.dispParagraph>>The stomach cramps return and you find yourself dry heaving, unable to stop for the longest time. ";
				break;
			case 25:
				"<<global.dispParagraph>>Your muscles abruptly turn to jelly and you find yourself sprawled on the floor, unable to move.  Your vision swims again and everything slowly fades to black.  You never regain consciousness. ";
				lose(true);
				break;
		}
		global.sicktime++;
	}
}

setHealth: function(val) {
	if (val > parserGetMe().health) {
		parserGetMe().health := val;
	}
}

labdoor: doorway, floatingItem
	sdesc = "lab doors"
	noun = 'doors'
	adjective = 'lab' 'door'
	location = {
		if ((parserGetMe().location = steamy) || (parserGetMe().location = lab)) {
			return parserGetMe().location;
		}
		return steamy;
	}
	ldesc = "A pair of automated sliding doors.  They have a brushed metal frame and panels of transparent plexiglass. "
	isopen = { return global.thirdpower; }
	doordest = lab
	destination = {
		if (!self.isopen) {
			"With the power on this floor out, the doors refuse to open. ";
			return nil;
		}
		"The sliding doors open as you approach them, allowing you to enter the lab.\b";
		return doordest;
	}
	verIoMoveTo(actor) = {
	}
	ioMoveTo(actor, dobj) = {
		if (dobj != gastank) {
			"That doesn't get us anywhere. ";
			return;
		}
		gastank.doMove(actor);
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
	verDoBraceWith(actor, iobj) = "That would be unnecessary. "
;

lab: darkroom
	sdesc = "Laboratory"
	ldesc = {
		"Even after decades of disuse, this room is still astonishingly free of dust.  Stainless steel countertops gleam in the muted glow of the overhead lights.  Four white solid-block tables, their surfaces made from some kind of synthetic material, occupy most of the space in this room.  Each of them has a deck of unfamiliar electronic equipment attached to the top, plus a pair of tall modular chairs. ";
		if (table.grenadeblast) {
			"One of the tabletops is scorched, though there appears to be little actual damage. ";
		}
		if (gastank.location = nil) {
			"A large area of scorched metal and plastic marks the location of a large explosion. ";
		}
		if (!vaultdoor.isbroken) {
			if (vaultdoor.isopen)
				"An open vault door makes up most of the eastern wall. ";
			else
				"An imposing silvery vault door makes up most of the eastern wall. ";
		} else {
			"The huge vault door lies impotent on the ground, its hinges blackened and mangled. ";
		}
		"Whatever else may have been in this room has since been cleared out";
		if (gastank.location = self) {
			if (gastank.position = 1)
				", except for a gas storage tank near the automated entrance. ";
			else
				", except for a gas storage tank near the vault door. ";
			if (gastank.onfire)
				"There is a huge sheet of flame pouring from the top of the tank. ";
		} else {
			". ";
		}
	}
	specialdesc = {
		if (((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) && self.lightsOn) {
			return 'Whirling red lights and warning alarms fill the room. ';
		}
		return '';
	}
	radmessage = 'Your Geiger counter is shrieking a blare of static and a piercing alarm bell.  A glance at the indicator reveals a red arrow pointing west. '
	west = leader
	east = vaultdoor
	enterRoom(actor) = {
		if ((global.radtime = 0) && !global.scPastRad) {
			incscore(1);
			global.scPastRad := true;
			radHint.active := nil;
			vaultHint.active := true;
		}
		pass enterRoom;
	}
	lightsOn = {
		return global.thirdpower;
	}
	scorchdesc = {
		if (vaultdoor.isbroken)
			"The vault door has been blown open by an explosion along its hinges.  It lies flat on the ground, leaving the vault easily accessible. ";
		else
			pass scorchdesc;
	}
	roomAction(a, v, d, p, i) = {
		if (a.location != self)
			pass roomAction;
		if (vaultdoor.isopen && leader.isalive && (leader.location = nil)) {
			if (isclass(v, travelVerb)) {
				if (crowd.isAlive)
					"Before you can move, a booming voice from behind stops you in your tracks.  \"Hello, little thief.  Some of my men commented to me on the odd new guy.  Only, we haven't had any new recruits for over two months.\"  A quick look over your shoulder reveals a huge form standing in front of the sliding glass doors.  The figure is at least seven feet tall with a massive, imposing build.  Only after a moment do you realize that this superhuman shape is in truth a man clad in a suit of military combat armor, a grenade launcher in his hand.  \"You have done me one favor, though,\" the titan continues in the voice of the wasteland itself.  \"I've been meaning to get down into these lower levels for a while and you kindly made it easier.  You're still going to have to die.\" ";
				else
					"Before you can move, a booming voice from behind stops you in your tracks.  \"You slaughtered my worthless men.  You and your friends, wherever they are.\"  A quick look over your shoulder reveals a huge form standing in front of the sliding glass doors.  The figure is at least seven feet tall with a massive, imposing build.  Only after a moment do you realize that this superhuman shape is in truth a man clad in a suit of military combat armor, a grenade launcher in his hand.  \"But you're not so tough all by yourself, are you?\" the titan continues in the voice of the wasteland itself.  \"I've been meaning to get down into these lower levels for a while and you kindly made it easier.  But I'm still going to kill you.\" ";
				leader.moveInto(self);
				grenadelauncher.moveInto(leader);
				notify(leader, &daemon, 1);
				bossHint.active := true;
				firstfloordoor.forced := true;
				firstfloordoor.isopen := true;
				firstfloordoor.isblocked := true;
				exit;
			}
		}
		pass roomAction;
	}
	listendesc = {
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1) && self.lightsOn) {
			"The whirling red lights are accompanied by a deafening alarm. ";
			return;
		}
		if (gastank.onfire) {
			"There's a roaring coming from the burning gas tank. ";
			return;
		}
		pass listendesc;
	}
	exitlist = { "The only doors out are to the east and west. "; return nil; }
	smelldesc = "It seems stale.  The air has an almost antiseptic aroma. "
;

equipment: fixeditem
	sdesc = "electronic equipment"
	noun = 'equipment'
	adjective = 'deck' 'of' 'unfamiliar' 'electronic'
	location = lab
	ldesc = {
		"Way out of your league.  Well, most of it, anyway.  You do notice one particularly simple piece of equipment: a log player. ";
		logplayer.isqcontainer := nil;
	}
	verDoTake(actor) = {
		"It's all bolted to the tables. ";
	}
;

// Sits either by the doors (where it can explode to kill the raider boss) 
// or by the vault door (where it can explode to open the door).  Shooting 
// or hitting it will knock of the regulator and spark a flame.  Explodes 1 
// turn later.
gastank: fixeditem
	sdesc = "gas storage tank"
	noun = 'canister' 'tank' 'cylinder' 'sign'
	adjective = 'gas' 'large' 'storage' 'steel'
	location = lab
	position = 1
	stripped = nil
	onfire = nil
	ldesc = {
		"A very large steel cylinder, measuring about five feet high and a foot in diameter. ";
		if (!self.onfire)
			"At the top of the tank is a regulator valve designed to control the flow of gas.  It looks particularly corroded. ";
		"The surface of the tank is green paint, peeling in several places.  A large sign is affixed to the front of the tank, a diamond with red edges and the word \"FLAMMABLE\" written in block letters. ";
		if (self.onfire)
			"There's a three-foot long gout of flame coming from the top of the tank where the regulator used to sit. ";
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if (weapon = pistol) {
			"Your aim goes a little high, blowing the regulator off of the top of the gas tank and releasing a jet of pressurized gas.  The bullet must have sparked as it ricocheted off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame. ";
			self.onfire := true;
			if ((leader.location = lab) && leader.isalive) {
				if (self.position = 1)
					"<<global.dispParagraph>>\"You missed--\" begins the raider, then seems to notice the burning gas tank beside him and turns towards it.  \"What the?\" ";
				else
					"<<global.dispParagraph>>The raider boss, across the room from the gas canister, seems to pause and takes a step back.  \"And what do you expect <I>that</I> to get you?\" ";
			}
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			if ((leader.location = lab) && leader.isalive && (self.position = 1)) {
				"You can't get close enough with the raider standing there beside it. ";
				return;
			} else {
				"You bash the top of the tank, managing to knock off the regulator valve and releasing a jet of pressurized gas.  The valve must have sparked as it tore off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame, sending you reeling backwards. ";
				self.onfire := true;
			}
		}
		if (weapon = parserGetMe()) {
			"Your fists do little damage. ";
			return;
		}
		notify(self, &explode, 2);
	}
	explode = {
		gastank.onfire := nil;
		if ((leader.location = lab) && leader.isalive && (gastank.position = 1)) {
			"\"Shit!\" shouts the armored giant and he turns to rush away from the burning tank of flammable gas.  But he's too late. ";
		}
		if ((parserGetMe().location = steamy) || (parserGetMe().location = table)) {
			if (parserGetMe().location = steamy) {
				"<<global.dispParagraph>>A deafening thunderclap erupts from the lab to the east.  A wall of flame washes over the door, leaving behind a haze of smoke as they fade. ";
			}
			if (parserGetMe().location = table) {
				"<<global.dispParagraph>><BIG>\(BOOM!\)</BIG>  The tank explodes in a deafening thunderclap, flames washing over the room.  You throw yourself down behind the table.  Fortunately, it manages to protect from the brunt of the fiery shockwave.  A wave of flame washes over the top of the desk and dissipates upwards, ";
				if (radsuit.isworn)
					"warming your suit";
				else
					"singeing your hair";
				" but otherwise leaving you relatively unharmed. ";
				if ((leader.location = lab) && leader.isalive) {
					if (gastank.position = 1) {
						"The bandit leader, however, was not quite so lucky. ";
						leader.kill;
					} else {
						"The bandit leader, unfortunately, also seems to have escaped without harm.  \"Nice try, worm,\" he scoffs. ";
					}
				}
			}
			if ((self.position = 2) && !vaultdoor.isbroken) {
				vaultdoor.isopen := true;
				vaultHint.active := nil;
				vaultdoor.islocked := true;
				vaultdoor.isbroken := true;
				hinges.issurface := nil;
				if (!global.scOpenVault) {
					incscore(1);
					global.scOpenVault := true;
				}
				if (parserGetMe().location = table) {
					if (!vaultdoor.isopen)
						"<<global.dispParagraph>>As the smoke clears, you hear the screech of twisting metal.  The vault door, its hinges mangled by the explosion, comes crashing to the floor.  The way into the vault is now clear. ";
					else
						"<<global.dispParagraph>>As the smoke clears, you hear the screech of twisting metal.  The vault door, its hinges mangled by the explosion, comes crashing to the floor. ";
				}
				doScoreNotify();
			}
		} else {
			"<<global.dispParagraph>><BIG>\(BOOM!\)</BIG>  The tank explodes in a deafening thunderclap, flames washing over the room.  A blinding wall of radiance slams into you... needless to say, you don't survive. ";
			lose(true);
		}
		self.moveInto(nil);
	}
	verDoOpen(actor) = {
		if (self.onfire)
			"Even if you could get close enough with burning your face off, you just blew off the regulator valve. ";
		else
			"The gas tank's regulator valve seems frozen shut. ";
	}
	verDoClose(actor) = {
		if (self.onfire)
			"Even if you could get close enough with burning your face off, you just blew off the regulator valve. ";
		else
			"The gas tank's regulator valve is already closed. ";
	}
	verDoTurn(actor) = {
		if (self.onfire)
			"Even if you could get close enough with burning your face off, you just blew off the regulator valve. ";
	}
	doTurn(actor) = {
		"The gas tank's regulator valve seems frozen shut. ";
	}
	verDoTurnWith(actor, iobj) = {
		if (self.onfire)
			"Even if you could get close enough with burning your face off, you just blew off the regulator valve. ";
		else if (iobj != wrench)
			"You can't get a good grip on <<self.thedesc>> with <<iobj.thedesc>>. ";
	}
	doTurnWith(actor, iobj) = {
		if (self.stripped)
			"The valve has been stripped.  You can't get any purchase with <<iobj.thedesc>>. ";
		else {
			"Wrapping the wrench tightly around the valve, you apply more and more pressure.  Without warning, the corroded regulator gives way, leaving behind the shine of stripped metal. ";
			self.stripped := true;
		}
	}
	verDoTake(actor) = {
		"\^<<self.thedesc>> is far too heavy to carry around.  Judging by its weight, it's probably still full. ";
	}
	verDoMove(actor) = {}
	doMove(actor) = {
		if ((leader.location = lab) && (leader.isalive)) {
			"You can't get close enough with the raider standing there beside it. ";
			return;
		}
		if (self.onfire) {
			"Even if you could get close enough with burning your face off, you just blew off the regulator valve. ";
			return;
		}
		if (self.position = 1) {
			"Straining, you manage to pull the large metal cylinder towards you.  As it leans, you roll it along the bottom edge until it's resting on the ground beside the vault door. ";
			self.position := 2;
		} else {
			"After a few moments of effort, you rock the gas canister back into its position beside the lab doors. ";
			self.position := 1;
		}
	}
	verDoMoveWith(actor, io) = {}
	doMoveWith(actor, io) = {
		self.doMove(actor);
	}
	verDoMoveTo(actor, io) = {
		if ((io != vaultdoor) && (io != labdoor)) {
			"That doesn't get us anywhere. ";
			return;
		}
		if ((self.position = 1) && (io = labdoor)) {
			"That's where it is already. ";
			return;
		}
		if ((self.position = 2) && (io = vaultdoor)) {
			"That's where it is already. ";
			return;
		}
	}
	doMoveTo(actor, io) = {
		self.doMove(actor);
	}
	verDoPush(actor) = {}
	doPush(actor) = {
		self.doMove(actor);
	}
	verDoPull(actor) = {}
	doPull(actor) = {
		self.doMove(actor);
	}
	smelldesc = "You can't detect any kind of odor.  You suppose that if there was a leak, it would be empty by now. "
;

regulator: fixeditem
	sdesc = "regulator valve"
	noun = 'valve' 'knob'
	adjective = 'corroded' 'regulator' 'hexagonal' 'gas'
	location = lab
	ldesc = {
		if (gastank.onfire) {
			"There's a three-foot long gout of flame coming from the top of the tank where the regulator used to sit. ";
		} else {
			"The regulator valve uses a knob to control the flow of gas from the tank.  The valve and knob are noticeably corroded";
			if (gastank.stripped)
				"and the knob has been stripped smooth";
			". ";
		}
	}

	verDoShootWith(actor, io) = {
		gastank.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if (weapon = pistol) {
			"Your shot blows the regulator off of the top of the gas tank, releasing a jet of pressurized gas.  The bullet must have sparked as it ricocheted off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame. ";
			gastank.onfire := true;
			if ((leader.location = lab) && leader.isalive) {
				if (gastank.position = 1)
					"<<global.dispParagraph>>\"You missed--\" begins the raider, then seems to notice the burning gas tank beside him and turns towards it.  \"What the?\" ";
				else
					"<<global.dispParagraph>>The raider boss, across the room from the gas canister, seems to pause and takes a step back.  \"And what do you expect <I>that</I> to get you?\" ";
			}
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			if ((leader.location = lab) && leader.isalive && (gastank.position = 1)) {
				"You can't get close enough with the raider standing there beside it. ";
				return;
			} else {
				"You bash off the regulator valve, releasing a jet of pressurized gas.  The valve must have sparked as it tore off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame, sending you reeling backwards. ";
				gastank.onfire := true;
			}
		}
		if (weapon = parserGetMe()) {
			"You manage to scrape up your knuckles. ";
			return;
		}
		notify(gastank, &explode, 2);
	}
	doOpen -> gastank
	doClose -> gastank
	doTurn -> gastank
	doTurnWith -> gastank
	verDoTake(actor) = "The regulator valve is firmly attached to the gas tank. "
	verDoMove(actor) = "The regulator valve is firmly attached to the gas tank. "
	verDoMoveWith(actor, io) = "The regulator valve is firmly attached to the gas tank. "
	smelldesc = "You can't detect any kind of odor.  You suppose that if there was a leak, it would be empty by now. "
;

table: hideitem, surface, chairitem
	sdesc = "table"
	noun = 'table' 'countertops' 'countertop' 'tables'
	adjective = 'large' 'white' 'stainless' 'steel'
	location = lab
	isThem = true
	thedesc = "the tables"
	adesc = "one of the tables"
	hidedesc = "one of the tables"
	ldesc = {
		"The four large white tables are arranged around the room in pairs.  Each is a solid block, capped by a smooth white top and a dizzying array of very advanced equipment.  ";
		if (self.grenadeblast) {
			"One of the tabletops is scorched, though there appears to be little actual damage to the actual surface. ";
		}
		if (itemcnt(self.contents) > 0)
	                "Sitting on one of the tables is <<listcont(self)>>. ";
	}
	showsurface = {
		"Sitting on one of the tables is "; listcont(self); ". ";
	}

	protects = true
	verDoStandon(actor) = {
		"And what exactly would that accomplish? ";
	}
	verDoSiton(actor) = {
		"And what exactly would that accomplish? ";
	}
	handleExplosion(actor, playerloc, heard) = {
		scorchmark.moveInto(nil);
	}
;

// Had one report of an error where no logs played properly.  Was not ever
// able to duplicate (sorry, lps).
logplayer: fixeditem, qcontainer
	sdesc = "log player"
	noun = 'player' 'receptacle' 'slot' 'reader'
	adjective = 'log'
	location = lab
	maxbulk = 2
	bulkthresh = 2
	count = 1
	playing = nil
	ldesc = {
		"A fairly basic piece of electronics.  It has a square receptacle, about an inch wide, on the top for inserting datacubes.  Beside it is a button marked 'Play' and another button marked 'Stop'. ";
	        if (self.contentsVisible and itemcnt(self.contents) <> 0)
	        {
	            "Within the receptacle %you% see%s% "; listcont(self); ". ";
	        }
	}
	verGrab(obj) = {
		if (obj = clay) {
			"Try as you might, you can't manage to scrape the clay out of the slot. ";
		}
	}
	ioPutIn(actor, dobj) = {
		if (dobj = clay) {
			"You mash the clay into the wide slot of the log player.  It is now thoroughly clogged. ";
			self.isbroken := true;
			clay.moveInto(self);
			return;
		}
		if (!dobj.datacube)
			"That won't fit. ";
		else if (self.isbroken)
			"You try to slide the datacube into the player, but the slot is clogged with clay. ";
		else if (addbulk(self.contents) + dobj.bulk > self.maxbulk)
			"There's only room in the player for one datacube. ";
		else
			pass ioPutIn;
	}
	play = {
		local text, cube, i, z;
		for (i:=1; i<=length(self.contents); i++) {
			z := self.contents[i];
			if (z.datacube)
				cube := z;
		}
		if (cube = nil) {
			"The log player beeps and then falls silent. ";
			self.count := 1;
			self.playing := nil;
			return;
		}
		if (count > length(cube.data)) {
			"The log player beeps and then falls silent. ";
			self.count := 1;
			self.playing := nil;
			return;
		}
		self.playing := true;
		text := cube.data[self.count];
		say(text);
		self.count++;
		notify(logplayer, &play, 1);
	}
	verDoTake(actor) = "\^<<self.thedesc>> appears to be attached to the table. "
;

playbutton: buttonitem
	sdesc = "play button"
	adjective = 'play'
	location = logplayer
	doPush(actor) = {
		if (itemcnt(logplayer.contents) = 0) {
			"The log player emits a quiet buzz but otherwise remains quiet. ";
			return;
		}
		if (logplayer.playing)
			return;
		notify(logplayer, &play, 1);
	}
;

stopbutton: buttonitem
	sdesc = "stop button"
	adjective = 'stop'
	location = logplayer
	doPush(actor) = {
		if (itemcnt(logplayer.contents) = 0) {
			"Nothing happens ";
			return;
		}
		"Click. ";
		logplayer.count := 1;
		unnotify(logplayer, &play);
	}
;

datacube2: item
	sdesc = "green datacube"
	noun = 'cube' 'datacube' 'prism'
	adjective = 'glass' 'green' 'data'
	plural = 'cubes' 'datacubes' 'prisms'
	location = logplayer
	datacube = true
	bulk = 2
	weight = 1
	ldesc = "It very much resembles a green-tinted glass prism that refracts and reflects the light in the room.  Datacubes like this were the standard portable digital storage medium before the war. "
	data = [self.line1 self.line2 self.line3]
	line1='A male voice from the log player greets you: \"Doctor Kevin Rock, personal log, fourteenth cube.\b\"November eighteenth.  Situation\'s only getting worse.  This is sure something they didn\'t cover in all my psych classes.  How exactly are people supposed to react to the possible end of the world?  What use is anything I say?  The Colonel seems to be holding up well, but I\'d say he\'s also in denial over the food and supplies situation.  A lot of good keeping everyone nice and orderly is going to be when my stomach is digesting itself.\"'
	line2='\bThe voice continues: \"November twenty-third.  I caught one of the damn grunts fooling around with the prototype.  Who gave these fucking idiots the access code anyway?  I imagine it\'s only going to get worse, too, what with all of us cooped up in here.  But this nonsense has got to stop.  What we\'ve made here, it\'s... it\'s not something... I don\'t know.  It was like a wondrous miracle, but all seems so pointless now.  It can\'t feed us.  It can\'t protect us from radiation.  But maybe it\'ll do some good... some day.  I\'m going to just lock the vault and change the code.  I\'ll give it to Colleen, in case she wants to continue to work.  Keep her mind busy.  I\'ll need to make it something she\'ll remember though.  She\'s been understandably preoccupied lately.  Maybe her birthday.\"'
	line3='\bThe man goes on: \"December first.  Does that jackass Miles really expect things to magically change?  We have a limited supply of food.  It is running out.  What\'s he waiting for, deus ex machina?  The nearest town is... was?  Is about a day\'s walk from here, maybe a little longer.  We should send someone, or a group, out there, armed, in suits.  See what we can barter or... scavenge, if it\'s as bad out there as they say.  But he refuses to listen.  Just tells us to hold tight.  Fucking military.\"'
;

// Added this as an optional item to use to detonate the gas tank to kill
// the leader.
wrench: item
	sdesc = "heavy wrench"
	noun = 'wrench' 'spanner'
	adjective = 'heavy'
	location = table
	ldesc = "A chrome-plated wrench about a eighteen inches long.  It looks rather heavy. "
	bulk = 6
	weight = 7
	doThrowAt(actor, io) =
	{
		if (io = gastank) {
			"Your aim goes a little high, knocking the regulator off of the top of the gas tank and releasing a jet of pressurized gas.  \^<<self.thedesc>> must have sparked as it ricocheted off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame. ";
			gastank.onfire := true;
			if ((leader.location = lab) && leader.isalive) {
				if (gastank.position = 1)
					"<<global.dispParagraph>>\"You missed--\" begins the raider, then seems to notice the burning gas tank beside him and turns towards it.  \"What the?\" ";
				else
					"<<global.dispParagraph>>The raider boss, across the room from the gas canister, seems to pause and takes a step back.  \"And what do you expect <I>that</I> to get you?\" ";
			}
			notify(gastank, &explode, 2);
			return;
		}
		if (io = regulator) {
			"Your aim is true and the wrench smashes the regulator off of the top of the gas tank, releasing a jet of pressurized gas.  \^<<self.thedesc>> must have sparked as it ricocheted off of the steel of the canister, because the plume of gas almost instantly transforms into a roaring gout of flame. ";
			gastank.onfire := true;
			if ((leader.location = lab) && leader.isalive) {
				if (gastank.position = 1)
					"<<global.dispParagraph>>\"You missed--\" begins the raider, then seems to notice the burning gas tank beside him and turns towards it.  \"What the?\" ";
				else
					"<<global.dispParagraph>>The raider boss, across the room from the gas canister, seems to pause and takes a step back.  \"And what do you expect <I>that</I> to get you?\" ";
			}
			notify(gastank, &explode, 2);
			return;
		}
		"\^<<self.thedesc>> bounce<<self.verbs>> off of <<io.thedesc>> and fall<<self.verbs>> to the ground. ";
		self.moveInto(actor.location);
	}
	verIoAttackWith(actor) = {}
	ioAttackWith(actor, dobj) = {
		dobj.doAttack(self);
	}
	verIoTurnWith(actor) = {}
	ioTurnWith(actor, dobj) = { dobj.doTurnWith(actor, self); }
;

leader: Actor, obstacle, knownTopic
	noun = 'boss' 'leader' 'man' 'form' 'figure' 'titan'
	adjective = 'bandit' 'raider' 'massive' 'huge' 'boss\'s' 'armor'
	sdesc = "bandit leader"
	location = nil
	isalive = true
	shot = 0
	step = 0
	actorDesc = {
		if (self.isalive)
			"The massive form of the bandit leader stands before the sliding doors, blocking any escape. ";
		else
			"The mangled remains of the bandit leader lie beside the sliding doors. ";
	}
	ldesc = {
		if (self.isalive) {
			"An imposing figure of seven feet tall.  There's no telling how much of that is the military combat armor, however.  The suit is similar to those worn by Enforcers in Arcadia, a fully enclosing outfit of interlocking ceramic plates with actuated assistance at the joints.  Extremely effective at stopping all but the most power calibers.  The face plate is tinted slightly, preventing you from even seeing the visage of your opponent. ";
		} else {
			"The force of the explosion appears to have torn his right arm off.  The combat armor around the shoulder joint is shattered, with blackened, jagged edges.  His grenade launcher is nothing more than a charred twisted lump of slag. ";
		}
	}
	destination = {
		if ((location = nil) || !isalive) {
			"The sliding doors open as you approach them, allowing you to enter the hall.\b";
			return steamy;
		}
		"You're not getting out of here with him in the way. ";
		return nil;
	}
	disavow = "The raider's booming laughter is your only reply. "
	doTalkto(actor) = {
		"The raider boss doesn't seem to even hear you. ";
	}
	verDoTake(actor) = "You find yourself overestimating your physique. "
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	verIoThrowAt(actor) = {}
	doAttack(weapon) = {
		if (!self.isalive) {
			"You beat a dead horse. ";
			return;
		}
		if ((weapon = pistol) || (weapon = rifle)) {
			if (self.shot = 0) {
				if (parserGetMe().location = lab) {
					"You whip up <<weapon.thedesc>>, take aim down the sights, and squeeze off a single shot. ";
				} else {
					"You peek your head up around the table, take aim down the sights, and squeeze off a single shot. ";
				}
				" Perfectly placed, center mass.<<global.dispParagraph>>Unfortunately, the bullet sparks off of the huge man's combat armor.  \"That's it?\" he asks derisively.  \"That's the best you've got?\" ";
			} else {
				"Again, you fire off another shot.  This one careens off of the raider boss's ";
				switch(rand(5)) {
						case 1: "face mask"; break;
						case 2: "shoulder"; break;
						case 3: "leg"; break;
						case 4: "chest"; break;
						case 5: "arm"; break;
				}
				", but does no damage. ";
			}
			self.shot++;
		}
		if (weapon = grenade) {
			"The bandit leader remains completely unscathed, the grenade's shrapnel easily deflected by his combat armor. ";
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench) || (weapon = parserGetMe())) {
			"Leaping forward, you make a desperate attack.  Unfortunately, the armor proves impervious to your makeshift weapon.  The bandit leader hurls you backwards across the room with a laugh. ";
		}
	}
	announce(actor, obj, val) = {
		if (obj = grenade) {
			switch (val) {
				case 1:
					"The bandit leader snickers contemptuously.  \"You expect to kill me with that pathetic firecracker?\" ";
					break;
				case 3:
					"With surprising speed, the armored man moves to position one of the tables between himself and the grenade. ";
					break;
			}
		}
	}
	verGrab(item) = {
		if (!self.isalive) {
			"It's completely ruined.  Don't bother. ";
		} else
			pass verGrab;
	}
	daemon = {
		if (!self.isalive)
			return;
		if (gastank.onfire) {
			notify(leader, &daemon, 1);
			return;
		}
		switch (self.step) {
			case 1:
				"<<global.dispParagraph>>\"Arcadia and its thieves,\" the raider boss growls.  \"Coming out here and stealing what's rightfully ours?  You must have some kind of death wish.\"  As his left fist shakes in your direction, his other hand angles the grenade launcher up in your direction. ";
				break;
			case 2:
				"<<global.dispParagraph>>The armored behemoth takes a step forward.  \"I hope you don't mind if I enjoy the moment.  This is the first time I've gotten the chance to kill one of you fuckers myself.\" ";
				break;
			case 3:
				if (crowd.isAlive)
					"<<global.dispParagraph>>The leader continues: \"And after I take care of you, we're moving on to your little city.  <I>We'll</I> be the ones taking from <I>Arcadia</I>... and enjoying every minute of it.  These pathetic wretches out here in the waste don't have shit worth stealing.  Your little city, I've heard, might actually have something of value.\" ";
				else
					"<<global.dispParagraph>>The leader continues: \"You've only prolonged the inevitable by killing my men.  The wastelands are filled with that sort.  And once my band is rebuilt, my next target is your pathetic city of Arcadia.\" ";
				break;
			case 4:
				"<<global.dispParagraph>>\"But first,\" the man says, \"I'm going to finish you off.\"  Bringing the grenade launcher up to his shoulder, he fires off a single explosive shell in your direction. ";
				if (clay.location = grenadelauncher) {
					"<<global.dispParagraph>><BIG>\(BOOM!\)</BIG>  There's a deafening explosion and the shockwave throws you to the floor.  When you manage to pick yourself up and turn to look, you see the mangled remains of the bandit leader.  It would appear that his grenade launcher suffered a catastrophic failure. ";
					leader.isalive := nil;
					clay.moveInto(nil);
					leader.kill;
					return;
				}
				if (parserGetMe().location = table) {
					"<<global.dispParagraph>>The grenade strikes the table and the explosion shatters it into hundreds of tiny fragments.  You are hurled backwards by the blast, deafened and dazed as you crash into the wall.  You can't even hear the second grenade fired or see it approach through the stars that swirl in your vision.  The end comes so quickly, in fact, that you don't even feel your body shattered by the explosion. ";
				} else {
					"<<global.dispParagraph>>The grenade whistles past you, striking against the wall to your rear and detonating with considerable force.  Your not entirely sure which kills you first: the concussive blast or the spray of shrapnel. ";
				}
				lose(true);
				break;
		}
		self.step++;
		notify(leader, &daemon, 1);
	}
	kill = {
		leader.isalive := nil;
		addword(leader, &noun, 'remains');
		addword(leader, &noun, 'body');
		addword(leader, &noun, 'corpse');
		addword(leader, &adjective, 'mangled');
		addword(leader, &adjective, 'dead');
		incscore(1);
		bossHint.active := nil;
		global.scKillLeader := true;
		doScoreNotify();
	}
	verDoAskAbout(actor, iobj) = {
		"The raider boss doesn't seem to even hear you. ";
	}
	verDoTellAbout(actor, iobj) = {
		"The raider boss doesn't seem to even hear you. ";
	}
	smelldesc = "A heady bouquet of cherry blossoms. "
	verDoSearch(actor) = {
		if (self.isalive)
			pass verDoSearch;
		else
			"You quickly look over the body.  The armor is a complete loss, as is the grenade launcher.  He doesn't seem to be carrying anything else. ";
	}
	doHug(actor) = {
		"You hug the armor-clad man and he soon realizes the errors of his violent ways.<<global.dispParagraph>>Actually, you realize what a ridiculous idea this is. ";
	}
	doConcentrateon(actor) = {
		if (helmet.isworn && helmet.done)
			"His essence is like a black hole.  It's almost like there's nothing there at all. ";
		pass doConcentrateon;
	}
;

vaultdoor: doorway
	sdesc = "vault door"
	noun = 'door' 'vault'
	adjective = 'vault'
	location = lab
	isopen = nil
	islocked = true
	ldesc = {
		if (self.isbroken) {
			"A fairly large door of brightly polished steel, laying on the floor.  Its hinges are blackened and mangled. ";
		} else {
			"A fairly large door of brightly polished steel.  Formidable in appearance, it looks quite secure.  On the left side is a numeric keypad and on the right side are the door's large discolored hinges. ";
			if (self.isopen)
				"The door is currently open. ";
			else
				"The door is currently closed. ";
		}
	}
	doordest = vault
	destination = {
		if (leader.isalive && (leader.location = lab)) {
			"You turn and dash towards the open vault door.  \"Not so fast,\" the raider says";
			if (clay.location = grenadelauncher) {
				"...<<global.dispParagraph>><BIG>\(BOOM!\)</BIG>  There's a deafening explosion behind you and the shockwave throws you to the floor.  When you manage to pick yourself up and turn to look, you see the mangled remains of the bandit leader.  It would appear that his grenade launcher suffered a catastrophic failure. ";
				clay.moveInto(nil);
				leader.kill;
				return nil;
			} else {
				", and you hear the hollow 'fwump' of his grenade launcher from behind.  The explosive strikes the doorway of the vault before you can take cover inside, and the shrapnel and concussive force deliver you unto an untimely end. ";
				lose(true);
			}
		}
		if (self.isopen)
			return self.doordest;
		else if (not self.islocked and not self.noAutoOpen) {
			self.setIsopen(true);
			"(Opening << self.thedesc >>)";
			return self.doordest;
		} else {
			"%You%'ll have to open << self.thedesc >> first. ";
			setit(self);
			return nil;
		}
	}
	verDoUnlock(actor) = {
		"There's no keyhole. ";
	}
	verDoUnlockWith(actor, io) = {
		"There's no keyhole. ";
	}
	verIoMoveTo(actor) = {
	}
	ioMoveTo(actor, dobj) = {
		if (dobj != gastank) {
			"That doesn't get us anywhere. ";
			return;
		}
		gastank.doMove(actor);
	}
	verDoLookbehind(actor) = {
		if (self.isopen)
			pass verDoLookbehind;
		else
			"You can't look behind a closed door. ";
	}
;

keypad3: fixeditem
	noun = 'keypad' 'pad' 'keys'
	adjective = 'numeric' 'number'
	sdesc = "keypad"
	location = lab
	ldesc = "You can see it's a pretty standard numeric keypad.  There are ten off-white plastic buttons, numbered 0 through 9. "
	code = 1102
	verIoTypeOn(actor) = {
	}
	ioTypeOn(actor, dobj) = {
		local val;
		if (isclass(dobj, basicStrObj)) {
			if (cvtstr(cvtnum(dobj.value)) = dobj.value) {
				val := cvtnum(dobj.value);
			} else {
				"You can only type numbers on this keypad. ";
				return;
			}
		} else if (!isclass(dobj, basicNumObj)) {
			"You can only type numbers on this keypad. ";
			return;
		} else {
			val := dobj.value;
		}
		if (vaultdoor.isopen) {
			"Nothing happens. ";
			return;
		}
		if (val != code) {
			"The panel emits a loud buzz. ";
			return;
		}
		"A brief klaxon sounds and from within the vault door comes the sound of bolts retracting.  Then the entire metal slab swings outwards shakily on its hinges, revealing a small brightly-lit room beyond. ";
		vaultdoor.isopen := true;
		vaultHint.active := nil;
		if (!global.scOpenVault) {
			incscore(1);
			global.scOpenVault := true;
		}
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
;

// Can place a grenade on the hinges to destroy them
hinges: fixeditem, surface
	sdesc = "vault door hinges"
	noun = 'hinges' 'hinge'
	adjective = 'vault' 'door' 'rusty' 'corroded' 'rust' 'layer'
	location = lab
	ldesc = {
		if (vaultdoor.isbroken)
			"The shattered remains of the vault door's hinges.  What's left of the top one is blackened, charred, and completely shattered.  The bottom hinge is twisted and sheared. ";
		else
			"Two wide and curving hinges that connect the door to the lab wall and allow it to pivot.  They must have been forged out of a different alloy than the door itself, because they seem have corroded considerably over their years of neglect.  It's not entirely clear, though, how deep the layer of rust goes. ";
	}
	maxbulk = 5
	verIoPutOn(actor) = {
		if (!self.issurface) {
			"There's no good surface on "; self.thedesc; ". ";
		}
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {}
	doAttack(weapon) = {
		if (weapon = pistol) {
			"The bullet ricochets off of the side of the hinge, throwing out sparks and flakes of metal.  Though the corrosion runs deep, it's obvious that the pistol doesn't pack enough of a wallop to break these hinges. ";
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			"You bash at one of the hinges, but achieve nothing more than flaking off a few pieces of metal. ";
		}
		if (weapon = parserGetMe()) {
			"Your fists do little damage. ";
		}
	}
	verDoLookbehind(actor) = "You can't see behind <<self.thedesc>>. "
	handleExplosion(actor, playerloc, heard) = {
		vaultdoor.isopen := true;
		vaultHint.active := nil;
		vaultdoor.islocked := true;
		vaultdoor.isbroken := true;
		hinges.issurface := nil;
		if (!global.scOpenVault) {
			incscore(1);
			global.scOpenVault := true;
		}
		if (playerloc = lab) {
			"As the smoke clears, you hear the screech of twisting metal.  The vault door, its hinges mangled by the explosion, comes crashing to the floor, leaving the way into the vault clear. ";
		}
		doScoreNotify();
	}
;

vault: darkroom
	sdesc = "Containment Vault"
	ldesc = "The air is still.  Even the vibrations from the ancient fusion reactor can't seem to penetrate the hush of this metal vault.  In the very center of the small room stands a sort of pedestal, reaching about waist high, with a circular surface approximately two feet in diameter. "
	west = lab
	lightsOn = {
		return global.thirdpower;
	}
	exitlist = { "There is only one exit: west. "; return nil; }
	scorchdesc = {
		if (pedestal.grenadeblast) {
			"Scorch marks radiate outwards from the center of the pedestal's surface. ";
		} else
			pass scorchdesc;
	}
;

pedestal: fixeditem, surface
	sdesc = "pedestal"
	noun = 'pedestal'
	location = vault
	ldesc = {
		"The pedestal is made of a smooth metal and is perfectly round.  Its base is slightly wider than the top surface.  Scorch marks radiate outwards from the center of the pedestal's surface. ";
	}
	maxbulk = 10
	handleExplosion(actor, playerloc, heard) = {
		local o;
		while (length(self.contents) > 0) {
			o := car(self.contents);
			o.moveInto(nil);
		}
	}
;

helmet: clothingItem, knownTopic
	sdesc = "odd-looking helmet"
	adesc = "an <<self.sdesc>>"
	noun = 'helmet' 'helm'
	adjective = 'odd' 'looking' 'odd-looking'
	location = pedestal
	isbroken = nil
	lasttried = -5
	ldesc = {
		if (self.isbroken)
			"You smashed it up pretty good.  Even though it still retains most of its original shape, the helmet is quite clearly broken beyond repair. ";
		else
			"Judging by the overall shape and contours, it sure looks like a helmet.  Just not like anything you've ever seen before.  On the inside, there are dozens, perhaps hundreds, of tiny little black circles laid across a surface of very smooth bright gray plastic.  Each of these circles has little specks of silver inside that catch and sparkle in the light.  On the outside is a chaos of integrated circuits, wires, and thin plastic tubes.  They twist and coil among each other in convoluted patterns across the entire surface of this bizarre headgear.  You can find no recognizable power switch. ";
	}
	bulk = 10
	weight = 6
	step = 0
	done = nil
	westrooms = [clearing hiddenpath fadedpath baseentrance]
	eastrooms = [edgeofvillage centerofvillage crumblingbuilding northvillage shack]
	firstfloor = [firstfloorwest firstfloorhall firstflooreast barracks conference firststairs]
	lowerfloors = [secondfloorhall electricalroom outsidecommissary commissary secondstairs thirdfloorhall powerplant steamy lab vault]
	arcadia = [startroom shop southgate]
	doTake(actor) = {
		global.gothelmet := true;
		pass doTake;
	}
	verDoWear(actor) = {
		if (radsuit.isworn)
			"\^<<self.thedesc>> won't fit over <<radsuit.thedesc>>. ";
		else
			pass verDoWear;
	}
	doWear(actor) = {
		/*
		*   if the actor is not directly carrying it (in other words,
		*   it's either not in the player's inventory at all, or it's
		*   within a container within the actor's inventory), take it --
		*   the actor must be directly holding the object for this to
		*   succeed
		*/
		if (self.location != actor) {
			/* try taking it */
			"(First taking <<self.thedesc>>)";
			if (execCommand(actor, takeVerb, self) != 0)
				exit;
		
			/*
			*   make certain it ended up where we want it - the command
			*   might have failed without actually indicating failure
			*/
			if (self.location != actor)
				exit;
		}
		
		/* wear it */
		self.isworn := true;
		if (self.isbroken) {
			"Okay, %you're% now wearing "; self.thedesc; ".  Nothing happens. ";
			return;
		}
		if (self.done) {
			"As you place the strange device upon your head, it activates with a few beeps and the same gentle whirring.  Slowly, the distant motes of consciousness dance into your awareness, raging tempests of emotion and thought. ";
		} else {
			"Tentatively, you hoist the helmet up above you and ease it down atop your head.  It must have some kind of hidden switch or sensor because once in place, there are a few faint beeps followed by a quiet whirring.  You don't notice anything else happen, however. ";
			notify(self, &daemon, 2);
		}
		self.step := 0;
	}
	daemon = {
		local bOutside, bInCity, bDown, bFirst, pRoom;
		if (!self.isworn)
			return;
		bDown := nil;
		bFirst := nil;
		bInCity := nil;
		bOutside := nil;
		pRoom := findRootRoom(parserGetMe().location);
		if (inList(pRoom, lowerfloors)) {
			bDown := true;
		}
		if (inList(pRoom, firstfloor)) {
			bFirst := true;
		}
		if (inList(pRoom, eastrooms + westrooms)) {
			bOutside := true;
		}
		if (inList(pRoom, arcadia)) {
			bInCity := true;
		}
		switch (self.step) {
			case 0:
				if (bDown || bFirst || bInCity)
					"<<global.dispParagraph>>There's a throbbing deep inside your skull, building to a peak and then slowly subsiding.  Your vision swims and you place a hand against a wall to steady yourself. ";
				if (bOutside)
					"<<global.dispParagraph>>There's a throbbing deep inside your skull, building to a peak and then slowly subsiding.  Your vision swims and you place a hand against a nearby tree trunk to steady yourself. ";
				break;
			case 1:
				"<<global.dispParagraph>>Slowly, you become aware of strange sensation, unlike anything you've ever experienced before.  Amidst your swirling emotions, you feel a sense of anger building.  Rage.  Hatred.  Loathing.  The bizarre thing is how detached you find yourself from this bubble of seething animosity floating within your consciousness.  You ";
				if (isclass(parserGetMe().location, chairitem))
					"shudder";
				else
					"sway on your feet";
				" as a wave of nausea overtakes you. ";
				break;
			case 2:
				if (bDown)
					"<<global.dispParagraph>>Focusing on the ball of rage adrift within your psyche, you come to the sudden realization that the sensation isn't internal at all.  You look upwards and gape.  The wrathful emotions emanate from above.  And as you blink in shock, information streams into your mind like a unstoppable tidal wave.  Names and images.  The ceiling before you is overlaid with the scene of two raiders arguing, one of them stalking away and the other left seething.  It loops, going faster and faster before fading away. ";
				if (bFirst)
					"<<global.dispParagraph>>Focusing on the ball of rage adrift within your psyche, you come to the sudden realization that the sensation isn't internal at all.  You look upwards and gape.  The wrathful emotions emanate from the east.  And as you blink in shock, information streams into your mind like a unstoppable tidal wave.  Names and images.  The ceiling before you is overlaid with the scene of two raiders arguing, one of them stalking away and the other left seething.  It loops, going faster and faster before fading away. ";
				if (bOutside)
					"<<global.dispParagraph>>Focusing on the ball of rage adrift within your psyche, you come to the sudden realization that the sensation isn't internal at all.  You look towards the south and gape.  The wrathful emotions emanate from the military base.  And as you blink in shock, information streams into your mind like a unstoppable tidal wave.  Names and images.  The scenery before you is overlaid with the vision of two raiders arguing, one of them stalking away and the other left seething.  It loops, going faster and faster before fading away. ";
				if (bInCity)
					"<<global.dispParagraph>>Focusing on the ball of rage adrift within your psyche, you come to the sudden realization that the sensation isn't internal at all.  You look around the city and gape.  The wrathful emotions emanate from all around you: all throughout the city.  And as you blink in shock, information streams into your mind like a unstoppable tidal wave.  Names and images.  The scenery before you is overlaid with the vision of two gangsters arguing and then one of them drawing a blade and ramming it into the other's chest.  It loops, going faster and faster before fading away. ";
				break;
			case 3:
				"<<global.dispParagraph>>Now your consciousness seems to expand.  More oily recesses of emotion, thought, and knowledge slip into your growing perception.  Just when the terrible flood of <I>awareness</I> seems about to overwhelm you, it dies down to a dull and manageable array of distant auras just on the fringes of your mind.  To your amazement, you find that by concentrating on a particular shard of emotion, you are able to actually scan the thoughts of a single ";
				if (bInCity)
					"person";
				else
					"raider";
				".  Feelings, images, and words slither like slimy worms through your brain.<<global.dispParagraph>>The idea that you've discovered a device to read minds from a distance leaves you stunned.  The possibilities dance in your mind, a dizzying galaxy of futures.  For now, though, the only thing you know for sure is that it's time to leave this place. ";
				self.done := true;
				break;
		}
		self.step++;
		notify(self, &daemon, 1);
	}
	detect(room) = {
		local match;
		match := nil;
		if ((self.location != parserGetMe()) || !self.isworn || !self.done || self.isbroken)
			return;
		if (guard.location = findRootRoom(room)) {
			say(match ? '' : "<<global.dispParagraph>>");
			"Wispy tendrils of thought drift lazily above the raider's head.  Sterile hatred and memories of ruthless violence leave a foul taste in your mouth. ";
			match := true;
		}
		if (findRootRoom(room) = southgate) {
			say(match ? '' : "<<global.dispParagraph>>");
			"From the bunkers and outposts come alternating waves of greed, irritation, and hatred. ";
			match := true;
		}
		if (coyotes.location = findRootRoom(room)) {
			say(match ? '' : "<<global.dispParagraph>>");
			"Primal emotions well up from the animal";
			say(coyotes.num = 1 ? '' : 's');
			" before you.  It's a jumble of sensations that feels almost alien to your human mind, and yet you can discern the overwhelming sense of hunger that dominates ";
			say(coyotes.num = 1 ? 'this emaciated beast' : 'these emaciated beasts');
			". ";
			match := true;
		}
		if (findRootRoom(iona.location) = findRootRoom(room)) {
			say(match ? '' : "<<global.dispParagraph>>");
			match := true;
			if (iona.location = boards)
				"Like an unexpected cool breeze cutting through a stifling desert heat, pure and innocent emotions roll over you from behind the boards.  You feel curiosity and loneliness mixed with confusion and fear. ";
			else
				"Like an unexpected cool breeze cutting through a stifling desert heat, pure and innocent emotions roll over you from the little girl.  You feel curiosity and loneliness mixed with confusion and fear. ";
		}
		if ((global.venttime > 0) && (global.turnsofar - global.venttime > 1)) {
			if (inList(room, firstfloor)) {
				say(match ? '' : "<<global.dispParagraph>>");
				"This room reeks of panic and confusion. ";
				match := true;
			} else if (inList(room, lowerfloors)) {
				say(match ? '' : "<<global.dispParagraph>>");
				"A cloud of panic and confusion hovers above you. ";
				match := true;
			}
		} else {
			if (room = conference) {
				say(match ? '' : "<<global.dispParagraph>>");
				"Waves of annoyance and boredom emanate from the guard beside the door.  In particular, you identify some rather unpleasant opinions of yourself within that stream of consciousness.  To the north, a swirling miasmal vortex of ghastly hideousness rages. ";
				match := true;
			} else if (room = barracks) {
				say(match ? '' : "<<global.dispParagraph>>");
				"The flood of venomous hatred from this room is overwhelming.  You tear the helmet from your head and fall to your knees... to find every eye in the room upon you.  The noisome marauders move closer, surrounding you.  Then the questioning starts.  Your identity.  Your origins.  The details of your membership in the group.  The stammered explanations fail to satisfy, and you are roughly subdued and bound, knowing all the while that the best you can hope for is a quick and painless death. ";
				lose(nil);
			} else if (room = securitystation) {
				say(match ? '' : "<<global.dispParagraph>>");
				"You are assailed by a rush of sensations from the northeast so noxious and loathsome that they are almost tangible. ";
				match := true;
			} else if (inList(room, firstfloor)) {
				say(match ? '' : "<<global.dispParagraph>>");
				"You are assailed by a rush of sensations from the east so noxious and loathsome that they are almost tangible. ";
				match := true;
			} else if (inList(room, westrooms)) {
				say(match ? '' : "<<global.dispParagraph>>");
				"To the southeast, just on the fringe of consciousness, a black pit of malignancy leaves an oily sensation on your mind. ";
				match := true;
			} else if (inList(room, eastrooms)) {
				say(match ? '' : "<<global.dispParagraph>>");
				"To the south, just on the fringe of consciousness, a black pit of malignancy leaves an oily sensation on your mind. ";
				match := true;
			}
		}
	}
	verDoShootWith(actor, io) = {
		self.verDoAttackWith(actor, io);
	}
	verDoAttackWith(actor, io) = {
		if (!self.done) {
			"Destroy it?  You don't even know what it is! ";
			return;
		}
		if (isinside(helmet, actor)) {
			"Not while you're carrying it. ";
			return;
		}
	}
	doAttack(weapon) = {
		if (global.turnsofar - self.lasttried>5) {
			"You look at the odd helmet for a moment first.  This is may be your last chance to reconsider. ";
			self.lasttried := global.turnsofar;
			return;
		}
		if (self.isbroken) {
			"It's already broken.  You're just beating a dead horse. ";
			return;
		}
		if (weapon = pistol) {
			"Taking a step back from the helmet, you raise your pistol.  A single shot is fired.  Sparks fly from the point of impact, accompanied by an acrid smoke. ";
			self.isbroken := true;
		}
		if ((weapon = flashlight) || (weapon = screwdriver) || (weapon = pole) || (weapon = wrench)) {
			"You bring your <<weapon.sdesc>> down upon the strange device.  Again.  And again.  When sparks begin to fly, you take a step back.  Thin plumes of smoke rise from the helmet for a few seconds before dissipating. ";
			self.isbroken := true;
		}
		if (weapon = parserGetMe()) {
			"Your fists do little damage. ";
		}
	}
;
